"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var runtime_1 = require("./runtime");
var microReact_1 = require("../microReact");
var performance_1 = require("./performance");
function isID(v) {
    return typeof v === "string" && (v.indexOf("|") > -1 || (v[8] === "-" && v.length === 36));
}
//------------------------------------------------------------------------
// UI helpers
//------------------------------------------------------------------------
function handleArgs(args) {
    if (typeof args[0] === "object" && args[0].constructor !== Array)
        return args;
    args.unshift({});
    return args;
}
function $row() {
    var args = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        args[_i] = arguments[_i];
    }
    var _a = handleArgs(args), elem = _a[0], children = _a[1];
    elem.t = "row";
    elem.children = children;
    return elem;
}
function $col() {
    var args = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        args[_i] = arguments[_i];
    }
    var _a = handleArgs(args), elem = _a[0], children = _a[1];
    elem.t = "column";
    elem.children = children;
    return elem;
}
function $text() {
    var args = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        args[_i] = arguments[_i];
    }
    var _a = handleArgs(args), elem = _a[0], text = _a[1];
    elem.t = "text";
    elem.text = text;
    return elem;
}
function $button() {
    var args = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        args[_i] = arguments[_i];
    }
    var _a = handleArgs(args), elem = _a[0], click = _a[1], content = _a[2];
    elem.t = "button";
    if (typeof content === "string") {
        elem.text = content;
    }
    else {
        elem.children = [content];
    }
    elem.click = click;
    return elem;
}
function $spacer() {
    var args = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        args[_i] = arguments[_i];
    }
    var elem = handleArgs(args)[0];
    elem.t = "spacer";
    return elem;
}
//------------------------------------------------------------------------
// Tracer
//------------------------------------------------------------------------
var TraceNode;
(function (TraceNode) {
    TraceNode[TraceNode["Join"] = 0] = "Join";
    TraceNode[TraceNode["Choose"] = 1] = "Choose";
    TraceNode[TraceNode["Union"] = 2] = "Union";
    TraceNode[TraceNode["LinearFlow"] = 3] = "LinearFlow";
    TraceNode[TraceNode["BinaryJoin"] = 4] = "BinaryJoin";
    TraceNode[TraceNode["AntiJoin"] = 5] = "AntiJoin";
    TraceNode[TraceNode["AntiJoinPresolvedRight"] = 6] = "AntiJoinPresolvedRight";
    TraceNode[TraceNode["Aggregate"] = 7] = "Aggregate";
    TraceNode[TraceNode["AggregateOuterLookup"] = 8] = "AggregateOuterLookup";
    TraceNode[TraceNode["Output"] = 9] = "Output";
    TraceNode[TraceNode["Watch"] = 10] = "Watch";
})(TraceNode = exports.TraceNode || (exports.TraceNode = {}));
var TraceFrameType;
(function (TraceFrameType) {
    TraceFrameType[TraceFrameType["Program"] = 0] = "Program";
    TraceFrameType[TraceFrameType["Transaction"] = 1] = "Transaction";
    TraceFrameType[TraceFrameType["Input"] = 2] = "Input";
    TraceFrameType[TraceFrameType["Block"] = 3] = "Block";
    TraceFrameType[TraceFrameType["Node"] = 4] = "Node";
    TraceFrameType[TraceFrameType["MaybeOutput"] = 5] = "MaybeOutput";
    TraceFrameType[TraceFrameType["MaybeExternalInput"] = 6] = "MaybeExternalInput";
})(TraceFrameType = exports.TraceFrameType || (exports.TraceFrameType = {}));
var typeToParentField = (_a = {},
    _a[TraceFrameType.Transaction] = "transactions",
    _a[TraceFrameType.Input] = "inputs",
    _a[TraceFrameType.Block] = "blocks",
    _a[TraceFrameType.Node] = "nodes",
    _a[TraceFrameType.MaybeOutput] = "outputs",
    _a[TraceFrameType.MaybeExternalInput] = "externalInputs",
    _a);
;
var Tracer = (function () {
    function Tracer(context, shouldDraw) {
        if (shouldDraw === void 0) { shouldDraw = true; }
        var _this = this;
        this.context = context;
        this.stack = [{ type: TraceFrameType.Program, transactions: [] }];
        this.inputsToOutputs = {};
        this.outputsToInputs = {};
        this.eToChange = {};
        this.activeBlock = "";
        this.tracker = new performance_1.PerformanceTracker();
        //------------------------------------------------------------------------
        // UI
        //------------------------------------------------------------------------
        this.activeSearch = "";
        this.$interface = function () {
            var program = _this.stack[0];
            return $row({ c: "trace" }, [
                _this.$searcher(program),
                _this.$visualization(program)
            ]);
        };
        this.makeSearch = function (query) {
            var _a = query.split(",").map(function (v) { return v.trim(); }), e = _a[0], a = _a[1], v = _a[2];
            var conditions = [];
            if (e && e !== "?") {
                conditions.push("input.e == " + +e);
            }
            if (a && a !== "?") {
                conditions.push("input.a === " + runtime_1.GlobalInterner.intern(a));
            }
            if (v && v !== "?") {
                if (_this.eToChange[+v]) {
                    conditions.push("input.v == " + +v);
                }
                else {
                    conditions.push("input.v == " + runtime_1.GlobalInterner.intern(isNaN(+v) ? v : +v));
                }
            }
            if (!conditions.length) {
                conditions.push("true");
            }
            return new Function("input", "return " + conditions.join(" && "));
        };
        this.inSearch = function (input) {
            return true;
        };
        this.$searcher = function (program) {
            var inputs = [];
            outer: for (var _i = 0, _a = program.transactions; _i < _a.length; _i++) {
                var transaction = _a[_i];
                if (!transaction)
                    continue;
                for (var _b = 0, _c = transaction.inputs; _b < _c.length; _b++) {
                    var input = _c[_b];
                    if (_this.inSearch(input.input)) {
                        inputs.push(_this.$changeLink(input.input));
                        if (inputs.length === 500) {
                            break outer;
                        }
                    }
                }
            }
            return $col({ c: "searcher" }, [
                { t: "input", type: "text", placeholder: "search", keydown: function (e) {
                        if (e.keyCode === 13) {
                            _this.activeSearch = e.target.value.trim();
                            _this.inSearch = _this.makeSearch(_this.activeSearch);
                            _this.draw();
                        }
                    } },
                $col(inputs)
            ]);
        };
        this.$visualization = function (program) {
            var _a = _this, $changeLink = _a.$changeLink, activeInput = _a.activeInput, getInputFrame = _a.getInputFrame, $block = _a.$block;
            if (activeInput) {
                var frame = getInputFrame(program, activeInput);
                var key = _this.changeKey(activeInput);
                var from = _this.outputsToInputs[key];
                var to = _this.inputsToOutputs[key];
                var fromInfo = void 0;
                if (from) {
                    fromInfo = $col([
                        $text("generated by: "),
                        $changeLink(from),
                    ]);
                }
                else {
                    fromInfo = $col([
                        $text("generated by: "),
                        $text("unknown"),
                    ]);
                }
                var toInfo = void 0;
                if (to) {
                    toInfo = $col([
                        $text("causes output:"),
                        $col(to.map($changeLink)),
                    ]);
                }
                var counts = _this.context.distinctIndex.getCounts(activeInput);
                console.log(frame);
                return $col({ c: "vis" }, [
                    $changeLink(activeInput),
                    fromInfo,
                    $col(frame.blocks.map($block)),
                    toInfo,
                    $text("counts: [" + counts.join(", ") + "]")
                ]);
            }
            return $text("select an input");
            // return $col({c: "program"}, program.transactions.map(this.$transaction));
        };
        this.$block = function (block) {
            return $col({ c: "block" }, [
                $text({ c: "name" }, block.name),
                $col(block.nodes.map(_this.$node)),
            ]);
        };
        this.$node = function (node) {
            var _a = _this, $prefix = _a.$prefix, $node = _a.$node, $changeLink = _a.$changeLink;
            var subs = node.nodes.map($node);
            var out;
            if (node.prefixes.length) {
                out = $row([
                    $text("out: "),
                    $col(node.prefixes.map($prefix)),
                ]);
            }
            if (node.outputs.length) {
                var outs = [];
                for (var _i = 0, _b = node.outputs; _i < _b.length; _i++) {
                    var output = _b[_i];
                    outs.push($col([
                        $row([
                            $text("maybe out: "),
                            $changeLink(output.change),
                        ]),
                        $text("pre: " + output.distinct.pre),
                        $text("post: " + output.distinct.post),
                        $row([
                            $text("distinct out: "),
                            $col(output.outputs.map($changeLink)),
                        ]),
                    ]));
                }
                out = $col(outs);
            }
            return $col({ c: "node" }, [
                $text(TraceNode[node.nodeType]),
                $row([
                    $text("in: "),
                    $prefix(node.inputPrefix),
                ]),
                $row(subs),
                out,
            ]);
        };
        this.$prefix = function (prefix) {
            var items = [];
            var hasValue = false;
            for (var ix = 0; ix < prefix.length - 2; ix++) {
                var value = prefix[ix];
                if (value === undefined) {
                    value = "?";
                }
                else if (!isID(runtime_1.GlobalInterner.reverse(value))) {
                    hasValue = true;
                    value = runtime_1.GlobalInterner.reverse(value);
                }
                else {
                    hasValue = true;
                }
                items.push(value);
            }
            if (!hasValue) {
                return $text("(empty)");
            }
            var round = prefix[prefix.length - 2];
            var count = prefix[prefix.length - 1];
            return $text("(" + items.join(", ") + ") [" + round + ", " + count + "]");
        };
        this.setLink = function (e, elem) {
            _this.activeInput = elem.input;
            _this.draw();
        };
        this.$changeLink = function (change) {
            var e = change.e, a = change.a, v = change.v;
            a = runtime_1.GlobalInterner.reverse(a);
            v = runtime_1.GlobalInterner.reverse(v);
            if (typeof v === "string" && (v.indexOf("|") > -1 || (v[8] === "-" && v.length === 36))) {
                v = change.v;
            }
            return $button({ c: "change-link", input: change }, _this.setLink, $row([
                $text(e + ", " + a + ", " + v),
                $spacer(),
                $text(" [" + change.transaction + ", " + change.round + ", " + change.count + "]"),
            ]));
        };
        if (typeof window !== "undefined" && shouldDraw) {
            var renderer = this.renderer = new microReact_1.Renderer();
            document.body.appendChild(renderer.content);
        }
    }
    Tracer.prototype.changeKey = function (change) {
        var e = change.e, a = change.a, v = change.v, n = change.n, round = change.round, transaction = change.transaction, count = change.count;
        return e + "|" + a + "|" + v + "|" + n + "|" + round + "|" + transaction + "|" + count;
    };
    Tracer.prototype.current = function () {
        return this.stack[this.stack.length - 1];
    };
    Tracer.prototype.transaction = function (id) {
        this.stack.push({ type: TraceFrameType.Transaction, id: id, externalInputs: [], inputs: [] });
        this.tracker.time("transaction");
    };
    Tracer.prototype.frame = function (commits) {
        // @TODO
    };
    Tracer.prototype.indexChange = function (change) {
        var found = this.eToChange[change.e];
        if (!found)
            found = this.eToChange[change.e] = [];
        found.push(change);
    };
    Tracer.prototype.input = function (input) {
        this._currentInput = input;
        this.indexChange(input);
        this.stack.push({ type: TraceFrameType.Input, input: input, blocks: [] });
    };
    Tracer.prototype.block = function (name) {
        this.activeBlock = name;
        this.stack.push({ type: TraceFrameType.Block, name: name, nodes: [] });
        this.tracker.block(name);
    };
    Tracer.prototype.node = function (node, inputPrefix) {
        this.stack.push({ type: TraceFrameType.Node, nodeType: node.traceType, node: node, inputPrefix: inputPrefix.slice(), nodes: [], prefixes: [], outputs: [], commits: [] });
    };
    Tracer.prototype.capturePrefix = function (prefix) {
        var parent = this.current();
        parent.prefixes.push(prefix.slice());
    };
    Tracer.prototype._mapOutput = function (output) {
        var _currentInput = this._currentInput;
        if (_currentInput) {
            var outKey = this.changeKey(output);
            var inKey = this.changeKey(_currentInput);
            this.outputsToInputs[outKey] = _currentInput;
            var inList = this.inputsToOutputs[inKey];
            if (!inList)
                inList = this.inputsToOutputs[inKey] = [];
            inList.push(output);
        }
    };
    Tracer.prototype.maybeOutput = function (change) {
        // this._mapOutput(change);
        var cur = this.current();
        var type = TraceFrameType.MaybeOutput;
        if (cur.type === TraceFrameType.Transaction) {
            type = TraceFrameType.MaybeExternalInput;
        }
        var counts = (this.context.distinctIndex.getCounts(change) || []).slice();
        this.stack.push({ type: type, distinct: { pre: counts, post: undefined }, change: change, outputs: [] });
    };
    Tracer.prototype.postDistinct = function () {
        var cur = this.current();
        var counts = this.context.distinctIndex.getCounts(cur.change).slice();
        cur.distinct.post = counts;
    };
    Tracer.prototype.output = function (output) {
        var safe = output.clone();
        this._mapOutput(safe);
        var parent = this.current();
        parent.outputs.push(safe);
    };
    Tracer.prototype.commit = function (commit) {
        var safe = commit.clone();
        this._mapOutput(safe);
        var parent = this.current();
        parent.commits.push(safe);
    };
    Tracer.prototype.distinctCheck = function () {
        var error = false;
        var index = this.context.distinctIndex.index;
        for (var key in index) {
            var counts = index[key];
            var sum = 0;
            for (var _i = 0, counts_1 = counts; _i < counts_1.length; _i++) {
                var c = counts_1[_i];
                if (!c)
                    continue;
                sum += c;
                if (sum < 0) {
                    console.error("Negative postDistinct: ", key, counts.slice());
                    error = true;
                    // throw new Error("Negative postDistinct at the end of a transaction")
                }
            }
        }
        return error;
    };
    Tracer.prototype.pop = function (type) {
        var stack = this.stack;
        var cur = stack.pop();
        if (cur.type !== type) {
            if (cur.type !== TraceFrameType.MaybeExternalInput || type !== TraceFrameType.MaybeOutput) {
                throw new Error("Popping the wrong type! expected: " + TraceFrameType[type] + ", actual: " + TraceFrameType[cur.type]);
            }
        }
        var parent = this.current();
        if (!parent) {
            throw new Error("Removed everything from the stack");
        }
        if (cur.type === TraceFrameType.Transaction) {
            parent = this.stack[0];
            parent.transactions[cur.id] = cur;
        }
        else {
            var field = typeToParentField[cur.type];
            if (!parent[field])
                throw new Error("Trying to write trace field '" + field + "', but " + TraceFrameType[parent.type] + " doesn't have it");
            parent[field].push(cur);
        }
        if (cur.type === TraceFrameType.Block)
            this.tracker.blockEnd(cur.name);
        if (cur.type === TraceFrameType.Input)
            this._currentInput = undefined;
        if (cur.type === TraceFrameType.Transaction) {
            this.tracker.timeEnd("transaction");
            var error = this.distinctCheck();
            this.draw();
        }
    };
    Tracer.prototype.draw = function () {
        var renderer = this.renderer;
        if (!renderer)
            return;
        renderer.render([this.$interface()]);
    };
    Tracer.prototype.getInputFrame = function (program, input) {
        var trans = program.transactions[input.transaction];
        for (var _i = 0, _a = trans.inputs; _i < _a.length; _i++) {
            var frame = _a[_i];
            if (frame.input === input)
                return frame;
        }
    };
    return Tracer;
}());
exports.Tracer = Tracer;
var NoopTracer = (function (_super) {
    __extends(NoopTracer, _super);
    function NoopTracer(context) {
        var _this = _super.call(this, context, false) || this;
        _this.context = context;
        _this.activeBlock = "";
        return _this;
    }
    NoopTracer.prototype.transaction = function (id) { };
    NoopTracer.prototype.frame = function (commits) { };
    NoopTracer.prototype.input = function (input) { };
    NoopTracer.prototype.block = function (name) { this.activeBlock = name; };
    NoopTracer.prototype.node = function (node, inputPrefix) { };
    NoopTracer.prototype.capturePrefix = function (prefix) { };
    NoopTracer.prototype.maybeOutput = function (change) { };
    NoopTracer.prototype.postDistinct = function () { };
    NoopTracer.prototype.output = function (output) { };
    NoopTracer.prototype.commit = function (commit) { };
    NoopTracer.prototype.distinctCheck = function () { return false; };
    NoopTracer.prototype.pop = function (type) { };
    return NoopTracer;
}(Tracer));
exports.NoopTracer = NoopTracer;
var _a;
//# sourceMappingURL=trace.js.map