import { Watcher, RawMap, RawValue } from "./watcher";
import { HTMLWatcher } from "./html";
export interface Canvas extends HTMLCanvasElement {
    __element?: RawValue;
}
export declare type OperationType = keyof Path2D;
export interface Operation {
    type: OperationType;
    args: any;
    paths: RawValue[];
}
export interface PathStyle {
    [key: string]: RawValue | undefined;
    fillStyle?: string;
    strokeStyle?: string;
    lineWidth?: number;
    lineCap?: string;
    lineJoin?: string;
}
export declare class CanvasWatcher extends Watcher {
    html: HTMLWatcher;
    canvases: RawMap<RawValue[] | undefined>;
    paths: RawMap<RawValue[] | undefined>;
    operations: RawMap<Operation | undefined>;
    canvasPaths: RawMap<RawValue[] | undefined>;
    pathToCanvases: RawMap<RawValue[] | undefined>;
    pathStyles: RawMap<PathStyle | undefined>;
    pathCache: RawMap<Path2D | undefined>;
    dirty: RawMap<boolean | undefined>;
    addCanvasInstance(canvasId: RawValue, instanceId: RawValue): void;
    clearCanvasInstance(canvasId: RawValue, instanceId: RawValue): void;
    getCanvasInstances(canvasId: RawValue): (string | number)[];
    getCanvasPaths(canvasId: RawValue): (string | number)[] | undefined;
    addPath(id: RawValue): never[];
    clearPath(id: RawValue): void;
    getPath(id: RawValue): (string | number)[];
    addOperation(id: RawValue, type: RawValue): {
        type: "rect" | "ellipse" | "addPath" | "closePath" | "moveTo" | "lineTo" | "bezierCurveTo" | "quadraticCurveTo" | "arc" | "arcTo" | "constructor" | "toString" | "toLocaleString" | "valueOf" | "hasOwnProperty" | "isPrototypeOf" | "propertyIsEnumerable";
        args: {};
        paths: never[];
    };
    clearOperation(id: RawValue): void;
    getOperation(id: RawValue): Operation;
    getOperationArgs(operation: Operation): (string | number | boolean)[] | undefined;
    updateCache(dirtyPaths: RawValue[]): void;
    rerender(dirtyPaths: RawValue[]): void;
    changed: () => void;
    setup(): void;
}
