"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var watcher_1 = require("./watcher");
function asValue(value) {
    if (typeof value == "string") {
        if (value == "true")
            return true;
        if (value == "false")
            return false;
    }
    return value;
}
function ixComparator(idMap) {
    return function (a, b) {
        return idMap[a].ix - idMap[b].ix;
    };
}
var operationFields = {
    moveTo: ["x", "y"],
    lineTo: ["x", "y"],
    bezierQuadraticCurveTo: ["cp1x", "cp1y", "cp2x", "cp2y", "x", "y"],
    quadraticCurveTo: ["cpx", "cpy", "x", "y"],
    arc: ["x", "y", "radius", "startAngle", "endAngle", "anticlockwise"],
    arcTo: ["x1", "y1", "x2", "y2", "radius"],
    ellipse: ["x", "y", "radiusX", "radiusY", "rotation", "startAngle", "endAngle", "anticlockwise"],
    rect: ["x", "y", "width", "height"],
    closePath: []
};
function isOperationType(val) {
    return !!operationFields[val];
}
var EMPTY = {};
;
;
var CanvasWatcher = (function (_super) {
    __extends(CanvasWatcher, _super);
    function CanvasWatcher() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.canvases = {};
        _this.paths = {};
        _this.operations = {};
        _this.canvasPaths = {};
        _this.pathToCanvases = {};
        _this.pathStyles = {};
        _this.pathCache = {};
        _this.dirty = {};
        _this.changed = function () {
            var dirtyPaths = Object.keys(_this.dirty);
            _this.updateCache(dirtyPaths);
            _this.rerender(dirtyPaths);
            _this.dirty = {};
        };
        return _this;
    }
    // addCanvas(canvasId:RawValue, instanceId:RawValue) {
    //   if(this.canvases[id]) throw new Error(`Recreating canvas instance ${maybeIntern(id)}`);
    //   let elements = this.html.elementToInstances[id];
    //   // if(!elements || !elements.length) throw new Error(`No matching canvas instance found for ${id}.`);
    //   if(!elements || !elements.length) return; // @FIXME: Really seems like this is an error case...
    //   if(elements.length > 1) throw new Error(`Multiple canvas instances found for ${id}.`);
    //   return this.canvases[id] = this.html.getInstance(elements[0]) as HTMLCanvasElement;
    // }
    // clearCanvas(id:RawValue) {
    //   if(!this.canvases[id]) throw new Error(`Missing canvas instance ${maybeIntern(id)}`);
    //   this.canvases[id] = undefined;
    // }
    // getCanvas(id:RawValue) {
    //   let canvas = this.canvases[id];
    //   if(!canvas) throw new Error(`Missing canvas instance ${maybeIntern(id)}`);
    //   return canvas;
    // }
    CanvasWatcher.prototype.addCanvasInstance = function (canvasId, instanceId) {
        var instances = this.canvases[canvasId] = this.canvases[canvasId] || [];
        instances.push(instanceId);
    };
    CanvasWatcher.prototype.clearCanvasInstance = function (canvasId, instanceId) {
        var instances = this.canvases[canvasId];
        if (!instances)
            return; // @FIXME: Seems like an error though
        var ix = instances.indexOf(instanceId);
        if (ix !== -1) {
            instances.splice(ix, 1);
            if (!instances.length)
                this.canvases[canvasId] = undefined;
        }
    };
    CanvasWatcher.prototype.getCanvasInstances = function (canvasId) {
        var instances = this.canvases[canvasId];
        if (!instances)
            throw new Error("Missing canvas instance(s) for " + watcher_1.maybeIntern(canvasId));
        return instances;
    };
    CanvasWatcher.prototype.getCanvasPaths = function (canvasId) {
        return this.canvasPaths[canvasId];
    };
    CanvasWatcher.prototype.addPath = function (id) {
        if (this.paths[id])
            throw new Error("Recreating path instance " + watcher_1.maybeIntern(id));
        this.pathStyles[id] = {};
        return this.paths[id] = [];
    };
    CanvasWatcher.prototype.clearPath = function (id) {
        if (!this.paths[id])
            throw new Error("Missing path instance " + watcher_1.maybeIntern(id));
        this.pathStyles[id] = undefined;
        this.paths[id] = undefined;
    };
    CanvasWatcher.prototype.getPath = function (id) {
        var path = this.paths[id];
        if (!path)
            throw new Error("Missing path instance " + watcher_1.maybeIntern(id));
        return path;
    };
    CanvasWatcher.prototype.addOperation = function (id, type) {
        if (this.operations[id])
            throw new Error("Recreating operation instance " + watcher_1.maybeIntern(id));
        if (!isOperationType(type))
            throw new Error("Invalid operation type " + type);
        return this.operations[id] = { type: type, args: {}, paths: [] };
    };
    CanvasWatcher.prototype.clearOperation = function (id) {
        if (!this.operations[id])
            throw new Error("Missing operation instance " + watcher_1.maybeIntern(id));
        this.operations[id] = undefined;
    };
    CanvasWatcher.prototype.getOperation = function (id) {
        var operation = this.operations[id];
        if (!operation)
            throw new Error("Missing operation instance " + watcher_1.maybeIntern(id));
        return operation;
    };
    CanvasWatcher.prototype.getOperationArgs = function (operation) {
        var type = operation.type, args = operation.args;
        var fields = operationFields[type];
        var input = [];
        for (var _i = 0, fields_1 = fields; _i < fields_1.length; _i++) {
            var field = fields_1[_i];
            if (args[field] == undefined)
                return;
            var value = asValue(args[field]);
            input.push(value);
        }
        return input;
    };
    CanvasWatcher.prototype.updateCache = function (dirtyPaths) {
        for (var _i = 0, dirtyPaths_1 = dirtyPaths; _i < dirtyPaths_1.length; _i++) {
            var id = dirtyPaths_1[_i];
            if (!this.dirty[id])
                continue;
            var path = this.paths[id];
            if (!path)
                continue;
            var path2d = this.pathCache[id] = new window.Path2D();
            for (var _a = 0, path_1 = path; _a < path_1.length; _a++) {
                var opId = path_1[_a];
                var operation = this.getOperation(opId);
                var input = this.getOperationArgs(operation);
                if (!input) {
                    console.warn("Skipping incomplete or invalid operation " + watcher_1.maybeIntern(opId), operation.type, operation.args);
                    continue;
                }
                if (!path2d[operation.type]) {
                    console.warn("Skipping unavailable operation type " + operation.type + ". Check your browser's Path2D compatibility.");
                    continue;
                }
                path2d[operation.type].apply(path2d, input);
            }
        }
    };
    CanvasWatcher.prototype.rerender = function (dirtyPaths) {
        var dirtyCanvases = {};
        for (var _i = 0, dirtyPaths_2 = dirtyPaths; _i < dirtyPaths_2.length; _i++) {
            var id = dirtyPaths_2[_i];
            var canvasIds = this.pathToCanvases[id];
            if (!canvasIds)
                continue;
            for (var _a = 0, canvasIds_1 = canvasIds; _a < canvasIds_1.length; _a++) {
                var canvasId = canvasIds_1[_a];
                dirtyCanvases[canvasId] = true;
            }
        }
        for (var _b = 0, _c = Object.keys(dirtyCanvases); _b < _c.length; _b++) {
            var canvasId = _c[_b];
            var pathIds = this.canvasPaths[canvasId];
            for (var _d = 0, _e = this.getCanvasInstances(canvasId); _d < _e.length; _d++) {
                var instanceId = _e[_d];
                var canvas = this.html.getInstance(instanceId);
                var ctx = canvas.getContext("2d");
                ctx.clearRect(0, 0, canvas.width, canvas.height);
                if (!pathIds)
                    continue;
                for (var _f = 0, pathIds_1 = pathIds; _f < pathIds_1.length; _f++) {
                    var id = pathIds_1[_f];
                    var cached = this.pathCache[id];
                    if (!cached)
                        continue; // This thing isn't a path (yet?)
                    var style = this.pathStyles[id] || EMPTY;
                    var _g = style.fillStyle, fillStyle = _g === void 0 ? "#000000" : _g, _h = style.strokeStyle, strokeStyle = _h === void 0 ? "#000000" : _h, _j = style.lineWidth, lineWidth = _j === void 0 ? 1 : _j, _k = style.lineCap, lineCap = _k === void 0 ? "butt" : _k, _l = style.lineJoin, lineJoin = _l === void 0 ? "miter" : _l;
                    ctx.fillStyle = fillStyle;
                    ctx.strokeStyle = strokeStyle;
                    ctx.lineWidth = lineWidth;
                    ctx.lineCap = lineCap;
                    ctx.lineJoin = lineJoin;
                    if (style.strokeStyle)
                        ctx.stroke(cached);
                    if (style.fillStyle || !style.strokeStyle)
                        ctx.fill(cached);
                }
            }
        }
    };
    CanvasWatcher.prototype.setup = function () {
        var _this = this;
        this.html = this.program.attach("html");
        this.program
            .bind("Canvas roots are html elements.", function (_a) {
            var find = _a.find;
            var canvas = find("canvas/root");
            return [canvas.add({ tag: "html/element", tagname: "canvas" })];
        })
            .watch("Export canvas instances.", function (_a) {
            var find = _a.find;
            var canvas = find("canvas/root");
            var instance = find("html/instance", { element: canvas });
            return [canvas.add("instance", instance)];
        })
            .asDiffs(function (diffs) {
            for (var _i = 0, _a = diffs.adds; _i < _a.length; _i++) {
                var _b = _a[_i], canvas = _b[0], _ = _b[1], instance = _b[2];
                _this.addCanvasInstance(canvas, instance);
            }
            for (var _c = 0, _d = diffs.removes; _c < _d.length; _c++) {
                var _e = _d[_c], canvas = _e[0], _ = _e[1], instance = _e[2];
                _this.clearCanvasInstance(canvas, instance);
            }
            setImmediate(_this.changed);
        })
            .watch("Export canvas paths.", function (_a) {
            var find = _a.find;
            var path = find("canvas/path");
            return [path.add("tag", "canvas/path")];
        })
            .asDiffs(function (diffs) {
            for (var _i = 0, _a = diffs.adds; _i < _a.length; _i++) {
                var e = _a[_i][0];
                _this.addPath(e);
                _this.dirty[e] = true;
            }
            for (var _b = 0, _c = diffs.removes; _b < _c.length; _b++) {
                var e = _c[_b][0];
                _this.clearPath(e);
                _this.dirty[e] = true;
            }
            setImmediate(_this.changed);
        })
            .watch("Export canvas operations.", function (_a) {
            var find = _a.find;
            var path = find("canvas/path");
            var operation = path.children;
            return [operation.add("type", operation.type)];
        })
            .asDiffs(function (diffs) {
            for (var _i = 0, _a = diffs.adds; _i < _a.length; _i++) {
                var _b = _a[_i], e = _b[0], _ = _b[1], type = _b[2];
                _this.addOperation(e, type);
            }
            for (var _c = 0, _d = diffs.removes; _c < _d.length; _c++) {
                var e = _d[_c][0];
                _this.clearOperation(e);
            }
            setImmediate(_this.changed);
        })
            .watch("Export paths of canvas.", function (_a) {
            var find = _a.find, gather = _a.gather, record = _a.record;
            var canvas = find("canvas/root");
            var child = canvas.children;
            // @FIXME: non-deterministic sort bug :(
            //let ix = gather(child.sort).per(canvas).sort();
            var ix = child.sort;
            return [record({ canvas: canvas, child: child, ix: ix })];
        })
            .asObjects(function (diffs) {
            var removeIds = Object.keys(diffs.removes);
            removeIds.sort(ixComparator(diffs.removes)).reverse();
            for (var _i = 0, removeIds_1 = removeIds; _i < removeIds_1.length; _i++) {
                var removeId = removeIds_1[_i];
                var _a = diffs.removes[removeId], canvasId = _a.canvas, childId = _a.child, ix = _a.ix;
                var instances = _this.canvases[canvasId];
                var paths = _this.canvasPaths[canvasId];
                if (paths)
                    paths.splice(ix - 1, 1);
                var canvases = _this.pathToCanvases[childId] = _this.pathToCanvases[childId];
                if (canvases) {
                    var ix_1 = canvases.indexOf(canvasId);
                    if (ix_1 !== -1)
                        canvases.splice(ix_1, 1);
                }
                // @FIXME: need a proper way to indicate dirtyness when an unchanged path is added a canvas.
                // This hack just marks the path dirty, which will rerender any other canvases containing it o_o
                _this.dirty[childId] = true;
            }
            var addIds = Object.keys(diffs.adds);
            addIds.sort(ixComparator(diffs.adds));
            for (var _b = 0, addIds_1 = addIds; _b < addIds_1.length; _b++) {
                var addId = addIds_1[_b];
                var _c = diffs.adds[addId], canvasId = _c.canvas, childId = _c.child, ix = _c.ix;
                var paths = _this.canvasPaths[canvasId] = _this.canvasPaths[canvasId] || [];
                paths.splice(ix - 1, 0, childId);
                var canvases = _this.pathToCanvases[childId] = _this.pathToCanvases[childId] || [];
                canvases.push(canvasId);
                // @FIXME: need a proper way to indicate dirtyness when an unchanged path is added a canvas.
                // This hack just marks the path dirty, which will rerender any other canvases containing it o_o
                _this.dirty[childId] = true;
            }
            setImmediate(_this.changed);
        })
            .watch("Export operations of paths.", function (_a) {
            var find = _a.find, gather = _a.gather, record = _a.record;
            var path = find("canvas/path");
            var child = path.children;
            // @FIXME: non-deterministic sort bug :(
            //let ix = gather(child.sort).per(path).sort();
            var ix = child.sort;
            return [record({ path: path, child: child, ix: ix })];
        })
            .asObjects(function (diffs) {
            var removeIds = Object.keys(diffs.removes);
            removeIds.sort(ixComparator(diffs.removes)).reverse();
            for (var _i = 0, removeIds_2 = removeIds; _i < removeIds_2.length; _i++) {
                var removeId = removeIds_2[_i];
                var _a = diffs.removes[removeId], pathId = _a.path, childId = _a.child, ix = _a.ix;
                var path = _this.paths[pathId];
                if (path)
                    path.splice(ix - 1, 1);
                var operation = _this.operations[childId];
                if (operation) {
                    var ix_2 = operation.paths.indexOf(pathId);
                    if (ix_2 !== -1)
                        operation.paths.splice(ix_2, 1);
                }
                _this.dirty[pathId] = true;
            }
            var addIds = Object.keys(diffs.adds);
            addIds.sort(ixComparator(diffs.adds));
            for (var _b = 0, addIds_2 = addIds; _b < addIds_2.length; _b++) {
                var addId = addIds_2[_b];
                var _c = diffs.adds[addId], pathId = _c.path, childId = _c.child, ix = _c.ix;
                var path = _this.getPath(pathId);
                path.splice(ix - 1, 0, childId);
                var operation = _this.getOperation(childId);
                operation.paths.push(pathId);
                _this.dirty[pathId] = true;
            }
            setImmediate(_this.changed);
        })
            .watch("Export attributes of operations.", function (_a) {
            var find = _a.find, lookup = _a.lookup, record = _a.record;
            var path = find("canvas/path");
            var child = path.children;
            var _b = lookup(child), attribute = _b.attribute, value = _b.value;
            return [child.add(attribute, value)];
        })
            .asDiffs(function (diffs) {
            for (var _i = 0, _a = diffs.removes; _i < _a.length; _i++) {
                var _b = _a[_i], opId = _b[0], attribute = _b[1], value = _b[2];
                var operation = _this.operations[opId];
                if (!operation)
                    continue;
                operation.args[attribute] = undefined;
                for (var _c = 0, _d = operation.paths; _c < _d.length; _c++) {
                    var pathId = _d[_c];
                    _this.dirty[pathId] = true;
                }
            }
            for (var _e = 0, _f = diffs.adds; _e < _f.length; _e++) {
                var _g = _f[_e], opId = _g[0], attribute = _g[1], value = _g[2];
                var operation = _this.operations[opId];
                if (!operation)
                    throw new Error("Missing operation " + watcher_1.maybeIntern(opId) + " for AV " + attribute + ": $[value}");
                if (operation.args[attribute])
                    throw new Error("Attempting to overwrite existing attribute " + attribute + " of " + opId + ": " + operation.args[attribute] + " => " + value);
                operation.args[attribute] = value;
                for (var _h = 0, _j = operation.paths; _h < _j.length; _h++) {
                    var pathId = _j[_h];
                    _this.dirty[pathId] = true;
                }
            }
            setImmediate(_this.changed);
        })
            .watch("Export path styles.", function (_a) {
            var find = _a.find, lookup = _a.lookup, record = _a.record;
            var path = find("canvas/path");
            var _b = lookup(path), attribute = _b.attribute, value = _b.value;
            attribute != "children";
            attribute != "tag";
            attribute != "sort";
            return [path.add(attribute, value)];
        })
            .asDiffs(function (diffs) {
            for (var _i = 0, _a = diffs.removes; _i < _a.length; _i++) {
                var _b = _a[_i], pathId = _b[0], attribute = _b[1], value = _b[2];
                var pathStyle = _this.pathStyles[pathId];
                if (!pathStyle)
                    continue;
                pathStyle[attribute] = undefined;
                _this.dirty[pathId] = true;
            }
            for (var _c = 0, _d = diffs.adds; _c < _d.length; _c++) {
                var _e = _d[_c], pathId = _e[0], attribute = _e[1], value = _e[2];
                var pathStyle = _this.pathStyles[pathId];
                if (!pathStyle)
                    throw new Error("Missing path style for " + pathId + ".");
                // if(pathStyle[attribute]) throw new Error(`Attempting to overwrite existing attribute ${attribute} of ${pathId}: ${pathStyle[attribute]} => ${value}`);
                pathStyle[attribute] = value;
                _this.dirty[pathId] = true;
            }
            setImmediate(_this.changed);
        });
    };
    return CanvasWatcher;
}(watcher_1.Watcher));
exports.CanvasWatcher = CanvasWatcher;
watcher_1.Watcher.register("canvas", CanvasWatcher);
/*
 * [#canvas/root width height children:
 *  [#canvas/rect x y width height fill? stroke?]]
 */
//# sourceMappingURL=canvas.js.map