import { Watcher, RawValue, RawEAV, RawEAVC } from "./watcher";
export interface Map<V> {
    [key: string]: V;
}
export interface Style extends Map<RawValue | undefined> {
    __size: number;
}
export interface ElemInstance extends Element {
    __element?: RawValue;
    __styles?: RawValue[];
    __sort?: RawValue;
    style?: any;
    listeners?: {
        [event: string]: boolean;
    };
}
export declare abstract class DOMWatcher<Instance extends ElemInstance> extends Watcher {
    styles: Map<Style | undefined>;
    roots: Map<Instance | undefined>;
    instances: Map<Instance | undefined>;
    elementToInstances: Map<RawValue[] | undefined>;
    styleToInstances: Map<RawValue[] | undefined>;
    abstract tagPrefix: string;
    abstract createInstance(id: RawValue, element: RawValue, tagname: RawValue): Instance;
    abstract getInstance(id: RawValue): Instance | undefined;
    abstract createRoot(id: RawValue): Instance | undefined;
    abstract addAttribute(instance: Instance, attribute: RawValue, value: RawValue | boolean): void;
    abstract removeAttribute(instance: Instance, attribute: RawValue, value: RawValue | boolean): void;
    protected _sendEvent(eavs: (RawEAV | RawEAVC)[]): void;
    getStyle(id: RawValue): Style;
    isInstance(elem?: any): elem is Instance;
    addInstance(id: RawValue, element: RawValue, tagname: RawValue): Instance | undefined;
    clearInstance(id: RawValue): void;
    getRoot(id: RawValue, tagname?: RawValue): Instance | undefined;
    clearRoot(id: RawValue): void;
    insertChild(parent: Element | null, child: Instance, at?: string | number | undefined): void;
    setStyleAttribute(styleId: RawValue, attribute: RawValue, value: RawValue, count: -1 | 1): void;
    addStyleInstance(styleId: RawValue, instanceId: RawValue): void;
    removeStyleInstance(styleId: RawValue, instanceId: RawValue): void;
    setup(): void;
}
