"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var watcher_1 = require("./watcher");
var dom_1 = require("./dom");
var HTMLWatcher = (function (_super) {
    __extends(HTMLWatcher, _super);
    function HTMLWatcher() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.tagPrefix = "html";
        _this._keyMap = {
            9: "tab",
            13: "enter",
            16: "shift",
            17: "control",
            18: "alt",
            27: "escape",
            37: "left",
            38: "up",
            39: "right",
            40: "down",
            91: "meta"
        };
        return _this;
    }
    HTMLWatcher.prototype.addExternalRoot = function (tag, element) {
        var elemId = watcher_1.createId();
        var eavs = [
            [elemId, "tag", tag],
            [elemId, "tag", "html/root/external"]
        ];
        this.instances[elemId] = element;
        this._sendEvent(eavs);
    };
    HTMLWatcher.prototype.createInstance = function (id, element, tagname) {
        var elem;
        if (tagname === "svg")
            elem = document.createElementNS("http://www.w3.org/2000/svg", tagname);
        else
            elem = document.createElement(tagname);
        elem.setAttribute("instance", "" + watcher_1.maybeIntern(id));
        elem.setAttribute("element", "" + watcher_1.maybeIntern(element));
        elem.__element = element;
        elem.__styles = [];
        return elem;
    };
    HTMLWatcher.prototype.getInstance = function (id) {
        return this.instances[id];
    };
    HTMLWatcher.prototype.createRoot = function (id) {
        var elem = this.instances[id];
        if (!elem)
            throw new Error("Orphaned instance '" + id + "'");
        document.body.appendChild(elem);
        return elem;
    };
    HTMLWatcher.prototype.addAttribute = function (instance, attribute, value) {
        // @TODO: Error checking to ensure we don't double-set attributes.
        if (attribute == "value") {
            if (instance.classList.contains("html-autosize-input") && instance instanceof HTMLInputElement) {
                instance.size = (instance.value || "").length || 1;
            }
            instance.value = "" + value;
        }
        else if (attribute == "tag") {
            if (value === "html/autosize-input" && instance instanceof HTMLInputElement) {
                setImmediate(function () { return instance.size = (instance.value || "").length || 1; });
            }
            else if (value === "html/trigger/focus" && instance instanceof HTMLInputElement) {
                setImmediate(function () { return instance.focus(); });
            }
            else if (value === "html/trigger/blur" && instance instanceof HTMLInputElement) {
                setImmediate(function () { return instance.blur(); });
            }
            else {
                instance.setAttribute(attribute, "" + value);
            }
        }
        else if (value === false) {
            instance.removeAttribute(attribute);
        }
        else {
            instance.setAttribute(attribute, "" + watcher_1.maybeIntern(value));
        }
    };
    HTMLWatcher.prototype.removeAttribute = function (instance, attribute, value) {
        // @TODO: Error checking to ensure we don't double-remove attributes or remove the wrong value.
        instance.removeAttribute(attribute);
        if (attribute === "value") {
            var input = instance;
            if (input.value === value)
                input.value = "";
        }
    };
    HTMLWatcher.prototype._updateURL = function (tagname) {
        if (tagname === void 0) { tagname = "url-change"; }
        var eventId = watcher_1.createId();
        var eavs = [
            [eventId, "tag", "html/event"],
            [eventId, "tag", "html/event/" + tagname]
        ];
        var hash = window.location.hash.slice(1);
        var ix = 1;
        for (var _i = 0, _a = hash.split("/"); _i < _a.length; _i++) {
            var segment = _a[_i];
            var segmentId = watcher_1.createId();
            eavs.push([eventId, "hash-segment", segmentId], [segmentId, "index", ix], [segmentId, "value", segment]);
            ix += 1;
        }
        this._sendEvent(eavs);
    };
    //------------------------------------------------------------------
    // Event handlers
    //------------------------------------------------------------------
    HTMLWatcher.prototype._mouseEventHandler = function (tagname) {
        var _this = this;
        return function (event) {
            var target = event.target;
            if (!_this.isInstance(target))
                return;
            var eventId = watcher_1.createId();
            var eavs = [
                [eventId, "tag", "html/event"],
                [eventId, "tag", "html/event/" + tagname],
                [eventId, "page-x", event.pageX],
                [eventId, "page-y", event.pageY],
                [eventId, "window-x", event.clientX],
                [eventId, "window-y", event.clientY],
                [eventId, "target", target.__element]
            ];
            var button = event.button;
            if (button === 0)
                eavs.push([eventId, "button", "left"]);
            else if (button === 2)
                eavs.push([eventId, "button", "right"]);
            else if (button === 1)
                eavs.push([eventId, "button", "middle"]);
            else if (button)
                eavs.push([eventId, "button", button]);
            var current = target;
            var elemIds = [];
            var capturesContextMenu = false;
            while (current && _this.isInstance(current)) {
                eavs.push([eventId, "element", current.__element]);
                if (button === 2 && current.listeners && current.listeners["context-menu"] === true) {
                    capturesContextMenu = true;
                }
                current = current.parentElement;
            }
            // @NOTE: You'll get a mousedown but no mouseup for a right click if you don't capture the context menu,
            //   so we throw out the mousedown entirely in that case. :(
            if (button === 2 && !capturesContextMenu)
                return;
            if (eavs.length)
                _this._sendEvent(eavs);
        };
    };
    HTMLWatcher.prototype._captureContextMenuHandler = function () {
        var _this = this;
        return function (event) {
            var captureContextMenu = false;
            var current = event.target;
            while (current && _this.isInstance(current)) {
                if (current.listeners && current.listeners["context-menu"] === true) {
                    captureContextMenu = true;
                }
                current = current.parentElement;
            }
            if (captureContextMenu && event.button === 2) {
                event.preventDefault();
            }
        };
    };
    HTMLWatcher.prototype._inputEventHandler = function (tagname) {
        var _this = this;
        return function (event) {
            var target = event.target;
            var elementId = target.__element;
            if (elementId) {
                if (target.classList.contains("html-autosize-input")) {
                    target.size = target.value.length || 1;
                }
                var eventId = watcher_1.createId();
                var eavs = [
                    [eventId, "tag", "html/event"],
                    [eventId, "tag", "html/event/" + tagname],
                    [eventId, "element", elementId],
                    [eventId, "value", target.value]
                ];
                if (eavs.length)
                    _this._sendEvent(eavs);
            }
        };
    };
    HTMLWatcher.prototype._keyEventHandler = function (tagname) {
        var _this = this;
        return function (event) {
            if (event.repeat)
                return;
            var current = event.target;
            var code = event.keyCode;
            var key = _this._keyMap[code];
            var eventId = watcher_1.createId();
            var eavs = [
                [eventId, "tag", "html/event"],
                [eventId, "tag", "html/event/" + tagname],
                [eventId, "key-code", code]
            ];
            if (key)
                eavs.push([eventId, "key", key]);
            while (current && _this.isInstance(current)) {
                var elemId = current.__element;
                eavs.push([eventId, "element", elemId]);
                current = current.parentElement;
            }
            ;
            if (eavs.length)
                _this._sendEvent(eavs);
        };
    };
    HTMLWatcher.prototype._focusEventHandler = function (tagname) {
        var _this = this;
        return function (event) {
            var target = event.target;
            var elementId = target.__element;
            if (elementId) {
                var eventId = watcher_1.createId();
                var eavs = [
                    [eventId, "tag", "html/event"],
                    [eventId, "tag", "html/event/" + tagname],
                    [eventId, "element", elementId]
                ];
                if (target.value !== undefined)
                    eavs.push([eventId, "value", target.value]);
                if (eavs.length)
                    _this._sendEvent(eavs);
            }
        };
    };
    HTMLWatcher.prototype._hoverEventHandler = function (tagname) {
        var _this = this;
        return function (event) {
            var target = event.target;
            if (!_this.isInstance(target))
                return;
            var eavs = [];
            var elemId = target.__element;
            if (target.listeners && target.listeners["hover"]) {
                var eventId = watcher_1.createId();
                eavs.push([eventId, "tag", "html/event"], [eventId, "tag", "html/event/" + tagname], [eventId, "element", elemId]);
            }
            if (eavs.length)
                _this._sendEvent(eavs);
        };
    };
    HTMLWatcher.prototype._hashChangeHandler = function (tagname) {
        var _this = this;
        return function (event) {
            _this._updateURL(tagname);
        };
    };
    //------------------------------------------------------------------
    // Watcher handlers
    //------------------------------------------------------------------
    HTMLWatcher.prototype.exportListeners = function (_a) {
        var adds = _a.adds, removes = _a.removes;
        for (var _i = 0, _b = Object.keys(adds); _i < _b.length; _i++) {
            var e = _b[_i];
            var _c = adds[e], listener = _c.listener, elemId = _c.elemId, instanceId = _c.instanceId;
            var instance = this.getInstance(instanceId);
            if (!instance.listeners)
                instance.listeners = {};
            instance.listeners[listener] = true;
        }
        for (var _d = 0, _e = Object.keys(removes); _d < _e.length; _d++) {
            var e = _e[_d];
            var _f = removes[e], listener = _f.listener, elemId = _f.elemId, instanceId = _f.instanceId;
            var instance = this.getInstance(instanceId);
            if (!instance || !instance.listeners)
                continue;
            instance.listeners[listener] = false;
        }
    };
    //------------------------------------------------------------------
    // Setup
    //------------------------------------------------------------------
    HTMLWatcher.prototype.setup = function () {
        var _this = this;
        if (typeof window === "undefined")
            return;
        this.tagPrefix = "html"; // @FIXME: hacky, due to inheritance chain evaluation order.
        _super.prototype.setup.call(this);
        this.program
            .bind("All html elements add their tags as classes", function (_a) {
            var find = _a.find, string = _a.lib.string, record = _a.record;
            var element = find("html/element");
            element.tag != "html/element";
            var klass = string.replace(element.tag, "/", "-");
            return [
                element.add("class", klass)
            ];
        });
        window.addEventListener("click", this._mouseEventHandler("click"));
        window.addEventListener("dblclick", this._mouseEventHandler("double-click"));
        window.addEventListener("mousedown", this._mouseEventHandler("mouse-down"));
        window.addEventListener("mouseup", this._mouseEventHandler("mouse-up"));
        window.addEventListener("contextmenu", this._captureContextMenuHandler());
        window.addEventListener("input", this._inputEventHandler("change"));
        window.addEventListener("keydown", this._keyEventHandler("key-down"));
        window.addEventListener("keyup", this._keyEventHandler("key-up"));
        window.addEventListener("focus", this._focusEventHandler("focus"), true);
        window.addEventListener("blur", this._focusEventHandler("blur"), true);
        document.body.addEventListener("mouseenter", this._hoverEventHandler("hover-in"), true);
        document.body.addEventListener("mouseleave", this._hoverEventHandler("hover-out"), true);
        window.addEventListener("hashchange", this._hashChangeHandler("url-change"));
        this.program
            .bind("Elements with no parents are roots.", function (_a) {
            var find = _a.find, record = _a.record, lib = _a.lib, not = _a.not;
            var elem = find("html/element");
            not(function () { return find("html/element", { children: elem }); });
            return [
                elem.add("tag", "html/root")
            ];
        })
            .bind("Create an instance for each child of an external root.", function (_a) {
            var find = _a.find, record = _a.record, lib = _a.lib, not = _a.not;
            var elem = find("html/element");
            var parent = find("html/root/external", { children: elem });
            return [
                record("html/instance", { element: elem, tagname: elem.tagname, parent: parent }),
                parent.add("tag", "html/element")
            ];
        });
        this.program
            .commit("Remove html events.", function (_a) {
            var find = _a.find, choose = _a.choose;
            var event = find("html/event");
            return [event.remove()];
        })
            .bind("Inputs with an initial but no value use the initial.", function (_a) {
            var find = _a.find, choose = _a.choose;
            var input = find("html/element", { tagname: "input" });
            var value = choose(function () { return input.value; }, function () { return input.initial; })[0];
            return [input.add("value", value)];
        })
            .commit("Apply input value changes.", function (_a) {
            var find = _a.find;
            var _b = find("html/event/change"), element = _b.element, value = _b.value;
            return [element.remove("value").add("value", value)];
        })
            .commit("When an element is entered, mark it hovered.", function (_a) {
            var find = _a.find, record = _a.record;
            var element = find("html/event/hover-in").element;
            return [element.add("tag", "html/hovered")];
        })
            .commit("When an element is left, clear it's hovered.", function (_a) {
            var find = _a.find, record = _a.record;
            var element = find("html/event/hover-out").element;
            return [element.remove("tag", "html/hovered")];
        })
            .watch("When an element is hoverable, it subscribes to mouseover/mouseout.", function (_a) {
            var find = _a.find, record = _a.record;
            var elemId = find("html/listener/hover");
            var instanceId = find("html/instance", { element: elemId });
            return [record({ listener: "hover", elemId: elemId, instanceId: instanceId })];
        })
            .asObjects(function (diffs) { return _this.exportListeners(diffs); })
            .watch("When an element listeners for context-menu, it prevents default on right click.", function (_a) {
            var find = _a.find, record = _a.record;
            var elemId = find("html/listener/context-menu");
            var instanceId = find("html/instance", { element: elemId });
            return [record({ listener: "context-menu", elemId: elemId, instanceId: instanceId })];
        })
            .asObjects(function (diffs) { return _this.exportListeners(diffs); })
            .commit("When the url changes, delete its previous segments.", function (_a) {
            var find = _a.find, record = _a.record;
            var change = find("html/event/url-change");
            var url = find("html/url");
            return [
                url.remove("hash-segment"),
                url["hash-segment"].remove()
            ];
        })
            .commit("When the url changes, commit its new state.", function (_a) {
            var find = _a.find, lookup = _a.lookup, record = _a.record;
            var change = find("html/event/url-change");
            var _b = lookup(change), attribute = _b.attribute, value = _b.value;
            attribute != "tag";
            return [
                record("html/url").add(attribute, value)
            ];
        });
        //setTimeout(() => this._updateURL(), 100);
        this._updateURL();
    };
    return HTMLWatcher;
}(dom_1.DOMWatcher));
exports.HTMLWatcher = HTMLWatcher;
watcher_1.Watcher.register("html", HTMLWatcher);
//# sourceMappingURL=html.js.map