"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var watcher_1 = require("./watcher");
var Notice = (function () {
    function Notice(program, id, type) {
        this.program = program;
        this.id = id;
        this.type = type;
        this.element = document.createElement("notice");
        var html = program.attach("html");
        html.addExternalRoot(id, this.element);
    }
    Notice.prototype.clear = function () {
        var parent = this.element.parentElement;
        if (parent)
            parent.removeChild(this.element);
        // @FIXME: html.removeExternalRoot.
    };
    return Notice;
}());
exports.Notice = Notice;
// @FIXME: do tihs with two program isolation instead of manual rendering?
var NotifyWatcher = (function (_super) {
    __extends(NotifyWatcher, _super);
    function NotifyWatcher() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.notices = {};
        return _this;
    }
    NotifyWatcher.prototype.setup = function () {
        var _this = this;
        this.html = this.program.attach("html");
        this.wrapper = document.createElement("div");
        this.wrapper.className = "notify-wrapper";
        document.body.appendChild(this.wrapper);
        this.scroller = document.createElement("div");
        this.scroller.className = "notify-scroller";
        this.wrapper.appendChild(this.scroller);
        this.root = document.createElement("column");
        this.root.className = "notify-root ui-column";
        this.scroller.appendChild(this.root);
        this.html.addExternalRoot("notify/root", this.root);
        this.program
            .bind("Notices that aren't dismissed are children of the notify root.", function (_a) {
            var find = _a.find, not = _a.not;
            var root = find("notify/root");
            var wrapper = find("notify/notice-wrapper");
            not(function () { return wrapper.notice.tag == "notify/dismissed"; });
            return [root.add("children", wrapper)];
        })
            .bind("Decorate notices.", function (_a) {
            var find = _a.find, choose = _a.choose, record = _a.record;
            var notice = find("notify/notice");
            var type = choose(function () { return notice.type; }, function () { return "info"; });
            return [
                record("notify/notice-wrapper", "ui/row", { notice: notice, type: type }).add("children", [
                    notice.add("tag", "ui/row"),
                    record("ui/spacer", { sort: 5, notice: notice })
                ])
            ];
        })
            .bind("Notices which are dismissable get a button to do so.", function (_a) {
            var find = _a.find, record = _a.record;
            var notice = find("notify/notice", "notify/dismissible");
            var wrapper = find("notify/notice-wrapper", { notice: notice });
            return [wrapper.add("children", [
                    record("notify/dismiss", "ui/button", { class: "flat", notice: notice, sort: 15, icon: "close-round" })
                ])];
        })
            .commit("Dismissed notices are marked.", function (_a) {
            var find = _a.find;
            var notice = find("notify/notice");
            find("html/event/click", { element: find("notify/dismiss", { notice: notice }) });
            return [notice.remove().add("tag", "notify/dismissed")];
        })
            .bind("If a notice has a timestamp, display it.", function (_a) {
            var find = _a.find, date = _a.lib.date, record = _a.record;
            var notice = find("notify/notice");
            var wrapper = find("notify/notice-wrapper", { notice: notice });
            return [
                wrapper.add("children", [
                    record("ui/text", { sort: 10, notice: notice, text: date.format(notice.timestamp, "HH:MM:ss") })
                ])
            ];
        })
            .commit("Retract timestamps for bound notices that have ceased to be.", function (_a) {
            var find = _a.find;
            var notice = find("notify/retract-timestamp").notice;
            return [notice.remove("timestamp")];
        })
            .watch("The notify watcher attaches a timestamp to notices without one.", function (_a) {
            var find = _a.find, not = _a.not;
            var notice = find("notify/notice");
            not(function () { return notice.timestamp; });
            return [notice.add("tag", "notify/notice")];
        })
            .asDiffs(function (_a) {
            var adds = _a.adds;
            var timestamp = Date.now();
            var eavs = [];
            for (var _i = 0, adds_1 = adds; _i < adds_1.length; _i++) {
                var notice = adds_1[_i][0];
                eavs.push([notice, "timestamp", timestamp, 1]);
            }
            if (eavs.length)
                _this.program.inputEAVs(eavs);
        })
            .watch("The notify watcher also cleans up those timestamps when the notice goes away.", function (_a) {
            var find = _a.find;
            var notice = find("notify/notice");
            return [notice.add("timestamp", notice.timestamp)];
        })
            .asDiffs(function (_a) {
            var removes = _a.removes;
            if (!removes.length)
                return;
            var eavs = [];
            eavs.push(["||notify/retract-timestamp", "tag", "notify/retract-timestamp", 1]);
            for (var _i = 0, removes_1 = removes; _i < removes_1.length; _i++) {
                var _b = removes_1[_i], notice = _b[0], _ = _b[1], timestamp = _b[2];
                eavs.push([notice, "timestamp", timestamp, -1]);
                //eavs.push(["||notify/retract-timestamp", "notice", notice, 1]);
            }
            if (eavs.length)
                _this.program.inputEAVs(eavs);
        });
    };
    return NotifyWatcher;
}(watcher_1.Watcher));
exports.NotifyWatcher = NotifyWatcher;
watcher_1.Watcher.register("notify", NotifyWatcher);
//# sourceMappingURL=notify.js.map