"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var watcher_1 = require("./watcher");
var ShapeWatcher = (function (_super) {
    __extends(ShapeWatcher, _super);
    function ShapeWatcher() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    ShapeWatcher.prototype.setup = function () {
        this.program.attach("html");
        this.program.attach("canvas");
        this.hexagon();
        this.hexGrid();
        this.squarePath();
        this.hexagonPath();
    };
    //--------------------------------------------------------------------
    // #shape/hexagon
    //--------------------------------------------------------------------
    ShapeWatcher.prototype.hexagonHTML = function () {
        this.program
            .bind("Draw a hexagon", function (_a) {
            var find = _a.find, choose = _a.choose, record = _a.record, math = _a.lib.math;
            var hex = find("shape/hexagon");
            var side = hex.side;
            var tri_height = math.round(side * 0.5); // sin(30deg)
            var tri_width = math.round(side * 0.86603); // cos(30deg)
            var width = 2 * tri_width;
            var background = choose(function () { hex.tag == "shape/outline"; return hex.border; }, function () { return hex.background; })[0];
            var sideBorder = tri_width + "px";
            var activeBorder = tri_height + "px";
            return [
                hex.add({ tag: "html/element", tagname: "div", class: "shape-hexagon", style: record({ width: width + "px" }), children: [
                        record("shape/hexagon/cap", "html/element", { sort: 1, tagname: "div", class: ["shape-hexagon-cap", "first"], style: record({
                                width: 0, height: 0,
                                "border": "0 solid transparent",
                                "border-left-width": sideBorder, "border-right-width": sideBorder,
                                "border-bottom-width": activeBorder, "border-bottom-color": background
                            }) }),
                        record("shape/hexagon/body", "ui/column", { hex: hex, sort: 2, style: record({ height: side + "px", width: "" + width, background: background }), class: "shape-hexagon-body" }),
                        record("shape/hexagon/cap", "html/element", { sort: 3, tagname: "div", class: ["shape-hexagon-cap", "last"], style: record({
                                width: 0, height: 0,
                                "border": "0 solid transparent",
                                "border-left-width": sideBorder, "border-right-width": sideBorder,
                                "border-top-width": activeBorder, "border-top-color": background
                            }) }),
                    ] })
            ];
        })
            .bind("Hexagons with border and thickness are outlined.", function (_a) {
            var find = _a.find;
            var hex = find("shape/hexagon");
            hex.border;
            hex.thickness;
            return [
                hex.add("tag", "shape/outline")
            ];
        })
            .bind("An outlined hexagon contains another hexagon inset by thickness.", function (_a) {
            var find = _a.find, record = _a.record;
            var hex = find("shape/hexagon", "shape/outline");
            var thickness = hex.thickness;
            var side = hex.side - thickness;
            var side_thickness = thickness * 0.86603; // cos(30deg)
            return [
                hex.add("children", [
                    record("shape/hexagon", "shape/hexagon/inner", { outer: hex, sort: 4, side: side, background: hex.background, class: "shape-hexagon-inner", style: record({
                            position: "absolute", top: 0, left: 0, "margin-top": thickness + "px", "margin-left": "" + side_thickness
                        }) })
                ])
            ];
        })
            .bind("Populate hexagon with content", function (_a) {
            var find = _a.find, not = _a.not;
            var hex_body = find("shape/hexagon/body");
            not(function () { return hex_body.hex.tag == "shape/outline"; });
            var content = hex_body.hex.content;
            return [
                hex_body.add("children", [
                    content
                ])
            ];
        })
            .bind("Populate an outlined hexagon's inner with content", function (_a) {
            var find = _a.find;
            var hex_inner = find("shape/hexagon/inner");
            return [
                hex_inner.add("content", hex_inner.outer.content)
            ];
        });
    };
    //--------------------------------------------------------------------
    // #shape/hex-grid
    //--------------------------------------------------------------------
    ShapeWatcher.prototype.hexGrid = function () {
        // [#hex-grid cells side gap]
        this.program.bind("Decorate all the hex-grid cells as hexagons.", function (_a) {
            var find = _a.find, math = _a.lib.math, record = _a.record;
            var hex_grid = find("shape/hex-grid");
            var side = hex_grid.side, gap = hex_grid.gap;
            var cell = hex_grid.cell;
            var x_ix = cell.x, y_ix = cell.y;
            var top_gap = gap * 0.86603; // sin(60deg)
            var tri_height = side * 0.5;
            var tri_width = side * 0.86603;
            var width = math.round(2 * tri_width + gap);
            var x_offset = math.round(math.mod(math.absolute(y_ix), 2) * width / 2);
            var height = math.round(side + tri_height + top_gap);
            var x = math.round(width * x_ix + x_offset);
            var y = math.round(height * y_ix);
            return [
                hex_grid.add({ tag: "html/element", tagname: "div", class: "shape-hex-grid" }),
                hex_grid.add("children", [
                    cell.add({
                        style: record({ position: "absolute", left: x + "px", top: y + "px" })
                    })
                ])
            ];
        });
    };
    ShapeWatcher.prototype.hexagon = function () {
        this.program
            .bind("Decorate a shape/hexagon as a canvas.", function (_a) {
            var find = _a.find, choose = _a.choose, math = _a.lib.math, record = _a.record;
            var hex = find("shape/hexagon");
            var side = hex.side;
            var tri_height = side * 0.5;
            var tri_width = side * 0.86603;
            var pad = choose(function () { return hex.lineWidth; }, function () { return 0; })[0];
            var dpad = 2 * pad;
            var width = math.ceiling(2 * tri_width + dpad);
            var height = math.ceiling(2 * side + dpad);
            return [
                hex.add({ tag: "html/element", tagname: "div", style: record({ width: width + "px", height: height + "px" }) }).add("children", [
                    record("canvas/root", { sort: 1, hex: hex, width: width + "px", height: height + "px" }).add("children", [
                        record("shape/hexagon-path", { sort: 1, hex: hex, x: pad, y: pad, side: side })
                    ]),
                    record("shape/hexagon/content", "html/element", { sort: 2, hex: hex, tagname: "div", style: record({ top: tri_height + "px", bottom: tri_height + "px", left: pad / 2 + "px", right: pad + "px" }) })
                ])
            ];
        })
            .bind("Copy hexagon content into it's appropriate container.", function (_a) {
            var find = _a.find, record = _a.record;
            var hex = find("shape/hexagon");
            var container = find("shape/hexagon/content", { hex: hex });
            return [container.add("children", hex.content)];
        })
            .bind("Copy style properties onto hexagon path.", function (_a) {
            var find = _a.find, lookup = _a.lookup;
            var hex = find("shape/hexagon");
            var path = find("shape/hexagon-path", { hex: hex });
            var _b = lookup(hex), attribute = _b.attribute, value = _b.value;
            attribute != "tag";
            attribute != "class";
            attribute != "tagname";
            attribute != "style";
            attribute != "children";
            attribute != "x";
            attribute != "y";
            return [path.add(attribute, value)];
        });
    };
    ShapeWatcher.prototype.squarePath = function () {
        this.program.bind("Decorate a shape/square-path as a canvas/path", function (_a) {
            var find = _a.find, record = _a.record;
            var square = find("shape/square-path");
            var x = square.x, y = square.y, side = square.side;
            return [
                square.add({ tag: "canvas/path" }).add("children", [
                    record({ sort: 1, type: "rect", x: x, y: y, width: side, height: side }),
                ])
            ];
        });
    };
    ShapeWatcher.prototype.hexagonPath = function () {
        this.program.bind("Decorate shape/hexagon-path as a canvas/path", function (_a) {
            var find = _a.find, record = _a.record;
            var hex = find("shape/hexagon-path");
            var x = hex.x, y = hex.y, side = hex.side;
            var tri_height = side * 0.5;
            var tri_width = side * 0.86603;
            var width = 2 * tri_width;
            var xl = x + width;
            var xm = x + tri_width;
            var y14 = y + tri_height;
            var y34 = y + 3 * tri_height;
            var yb = y + 2 * side;
            return [
                hex.add({ tag: "canvas/path" }).add("children", [
                    record({ sort: 1, type: "moveTo", x: xm, y: y }),
                    record({ sort: 2, type: "lineTo", x: xl, y: y14 }),
                    record({ sort: 3, type: "lineTo", x: xl, y: y34 }),
                    record({ sort: 4, type: "lineTo", x: xm, y: yb }),
                    record({ sort: 5, type: "lineTo", x: x, y: y34 }),
                    record({ sort: 6, type: "lineTo", x: x, y: y14 }),
                    record({ sort: 7, type: "closePath" }),
                ])
            ];
        });
    };
    return ShapeWatcher;
}(watcher_1.Watcher));
exports.ShapeWatcher = ShapeWatcher;
watcher_1.Watcher.register("shape", ShapeWatcher);
//# sourceMappingURL=shape.js.map