"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var dsl2_1 = require("../src/runtime/dsl2");
var util_1 = require("./util");
var test = require("tape");
test("Aggregate: Count in choose", function (assert) {
    var prog = new dsl2_1.Program("test");
    prog.bind("test count in choose", function (_a) {
        var find = _a.find, choose = _a.choose, gather = _a.gather, record = _a.record;
        var person = find("person");
        var count = choose(function () { return gather(person.pet).count(); }, function () { return 0; })[0];
        return [
            record("result", { person: person, count: count })
        ];
    });
    util_1.verify(assert, prog, [
        ["A", "tag", "person"],
    ], [
        [1, "tag", "result", 1],
        [1, "person", "A", 1],
        [1, "count", 0, 1],
    ]);
    util_1.verify(assert, prog, [
        ["A", "pet", "B"],
    ], [
        [1, "tag", "result", 1, -1],
        [1, "person", "A", 1, -1],
        [1, "count", 0, 1, -1],
        [2, "tag", "result", 1, 1],
        [2, "person", "A", 1, 1],
        [2, "count", 1, 1, 1],
    ]);
    util_1.verify(assert, prog, [
        ["A", "pet", "C"],
    ], [
        [2, "tag", "result", 1, -1],
        [2, "person", "A", 1, -1],
        [2, "count", 1, 1, -1],
        [3, "tag", "result", 1],
        [3, "person", "A", 1],
        [3, "count", 2, 1],
    ]);
    assert.end();
});
test("Aggregate: direction-less sort", function (assert) {
    var prog = new dsl2_1.Program("test");
    prog.bind("test direction-less sort", function (_a) {
        var find = _a.find, choose = _a.choose, gather = _a.gather, record = _a.record;
        var person = find("person");
        var pos = gather(person.name).sort();
        return [
            person.add("pos", pos)
        ];
    });
    util_1.verify(assert, prog, [
        ["A", "tag", "person"],
        ["A", "name", "Jane"],
    ], [
        ["A", "pos", 1, 1],
    ]);
    util_1.verify(assert, prog, [
        ["B", "tag", "person"],
        ["B", "name", "Chris"],
    ], [
        ["B", "pos", 1, 1],
        ["A", "pos", 1, 1, -1],
        ["A", "pos", 2, 1],
    ]);
    util_1.verify(assert, prog, [
        ["C", "tag", "person"],
        ["C", "name", "Zaria"],
    ], [
        ["C", "pos", 3, 1],
    ]);
    util_1.verify(assert, prog, [
        ["B", "tag", "person", 0, -1],
        ["B", "name", "Chris", 0, -1],
    ], [
        ["B", "pos", 1, 1, -1],
        ["A", "pos", 1, 1, 1],
        ["A", "pos", 2, 1, -1],
        ["C", "pos", 2, 1, 1],
        ["C", "pos", 3, 1, -1],
    ]);
    assert.end();
});
test("Aggregate: down sort", function (assert) {
    var prog = new dsl2_1.Program("test");
    prog.bind("test down sort", function (_a) {
        var find = _a.find, choose = _a.choose, gather = _a.gather, record = _a.record;
        var person = find("person");
        var pos = gather(person.name).sort("down");
        return [
            person.add("pos", pos)
        ];
    });
    util_1.verify(assert, prog, [
        ["A", "tag", "person"],
        ["A", "name", "Jane"],
    ], [
        ["A", "pos", 1, 1],
    ]);
    util_1.verify(assert, prog, [
        ["B", "tag", "person"],
        ["B", "name", "Chris"],
    ], [
        ["B", "pos", 2, 1],
    ]);
    util_1.verify(assert, prog, [
        ["C", "tag", "person"],
        ["C", "name", "Zaria"],
    ], [
        ["C", "pos", 1, 1],
        ["A", "pos", 1, 1, -1],
        ["A", "pos", 2, 1],
        ["B", "pos", 2, 1, -1],
        ["B", "pos", 3, 1],
    ]);
    util_1.verify(assert, prog, [
        ["C", "tag", "person", 0, -1],
        ["C", "name", "Zaria", 0, -1],
    ], [
        ["C", "pos", 1, 1, -1],
        ["A", "pos", 1, 1, 1],
        ["A", "pos", 2, 1, -1],
        ["B", "pos", 2, 1, 1],
        ["B", "pos", 3, 1, -1],
    ]);
    assert.end();
});
test("Aggregate: multi-direction sort", function (assert) {
    var prog = new dsl2_1.Program("test");
    prog.bind("test multi-direction sort", function (_a) {
        var find = _a.find, choose = _a.choose, gather = _a.gather, record = _a.record;
        var person = find("person");
        var pos = gather(person.name, person.age).sort("down", "up");
        return [
            person.add("pos", pos)
        ];
    });
    util_1.verify(assert, prog, [
        ["A", "tag", "person"],
        ["A", "name", "Jane"],
        ["A", "age", 27],
    ], [
        ["A", "pos", 1, 1],
    ]);
    util_1.verify(assert, prog, [
        ["B", "tag", "person"],
        ["B", "name", "Chris"],
        ["B", "age", 25],
    ], [
        ["B", "pos", 2, 1],
    ]);
    util_1.verify(assert, prog, [
        ["C", "tag", "person"],
        ["C", "name", "Jane"],
        ["C", "age", 19],
    ], [
        ["C", "pos", 1, 1],
        ["A", "pos", 1, 1, -1],
        ["A", "pos", 2, 1],
        ["B", "pos", 2, 1, -1],
        ["B", "pos", 3, 1],
    ]);
    util_1.verify(assert, prog, [
        ["C", "tag", "person", 0, -1],
    ], [
        ["C", "pos", 1, 1, -1],
        ["A", "pos", 1, 1, 1],
        ["A", "pos", 2, 1, -1],
        ["B", "pos", 2, 1, 1],
        ["B", "pos", 3, 1, -1],
    ]);
    assert.end();
});
test("Aggregate: group sort", function (assert) {
    var prog = new dsl2_1.Program("test");
    prog.bind("test group sort", function (_a) {
        var find = _a.find, choose = _a.choose, gather = _a.gather, record = _a.record;
        var person = find("person");
        var pos = gather(person.name).per(person.age).sort("down");
        return [
            person.add("pos", pos)
        ];
    });
    util_1.verify(assert, prog, [
        ["A", "tag", "person"],
        ["A", "name", "Jane"],
        ["A", "age", 27],
    ], [
        ["A", "pos", 1, 1],
    ]);
    util_1.verify(assert, prog, [
        ["B", "tag", "person"],
        ["B", "name", "Chris"],
        ["B", "age", 27],
    ], [
        ["B", "pos", 2, 1],
    ]);
    util_1.verify(assert, prog, [
        ["C", "tag", "person"],
        ["C", "name", "Zaria"],
        ["C", "age", 25],
    ], [
        ["C", "pos", 1, 1],
    ]);
    util_1.verify(assert, prog, [
        ["D", "tag", "person"],
        ["D", "name", "Dana"],
        ["D", "age", 27],
    ], [
        ["D", "pos", 2, 1],
        ["B", "pos", 2, 1, -1],
        ["B", "pos", 3, 1],
    ]);
    util_1.verify(assert, prog, [
        ["C", "tag", "person", 0, -1],
    ], [
        ["C", "pos", 1, 1, -1],
    ]);
    assert.end();
});
test("Aggregate: committed sort with post filtering", function (assert) {
    var prog = new dsl2_1.Program("test")
        .commit("Clear events when they come in.", function (_a) {
        var find = _a.find;
        var event = find("event/create-widget");
        return [event.remove()];
    })
        .commit("Create a new widget of the given model.", function (_a) {
        var find = _a.find, choose = _a.choose, gather = _a.gather, record = _a.record;
        var model = find("event/create-widget").model;
        // The serial number of our next widget is the highest serial we've issued for this model so far + 1.
        var other = model.widget;
        1 == gather(other.serial).per(model).sort("down");
        var serial = other.serial + 1;
        return [model.add("widget", record("widget", { serial: serial }))];
    });
    util_1.verify(assert, prog, [
        [1, "tag", "model"],
        [1, "widget", 2],
        [1, "widget", 3],
        [2, "tag", "widget"],
        [2, "serial", 3],
        [3, "tag", "widget"],
        [3, "serial", 5],
        [4, "tag", "event/create-widget"],
        [4, "model", 1],
    ], [
        [4, "tag", "event/create-widget", 0, -1],
        [4, "model", 1, 0, -1],
        [1, "widget", "tag|widget|serial|6", 0],
        ["tag|widget|serial|6", "tag", "widget", 0],
        ["tag|widget|serial|6", "serial", 6, 0],
    ]);
    assert.end();
});
test("Aggregate: committed sort in choose", function (assert) {
    var prog = new dsl2_1.Program("test")
        .commit("Clear events when they come in.", function (_a) {
        var find = _a.find;
        var event = find("event/create-widget");
        return [event.remove()];
    })
        .commit("Create a new widget of the given model.", function (_a) {
        var find = _a.find, choose = _a.choose, gather = _a.gather, record = _a.record;
        var model = find("event/create-widget").model;
        // The serial number of our next widget is the highest serial we've issued for this model so far + 1.
        var serial = choose(function () {
            var other = model.widget;
            1 == gather(other.serial).per(model).sort("down"); // @NOTE: This breaks differently due to equality bug.
            return other.serial + 1;
        }, function () { return 1; })[0];
        return [model.add("widget", record("widget", { serial: serial }))];
    });
    util_1.verify(assert, prog, [
        [1, "tag", "model"],
        [1, "widget", 2],
        [1, "widget", 3],
        [2, "tag", "widget"],
        [2, "serial", 3],
        [3, "tag", "widget"],
        [3, "serial", 5],
        [4, "tag", "event/create-widget"],
        [4, "model", 1],
    ], [
        [4, "tag", "event/create-widget", 0, -1],
        [4, "model", 1, 0, -1],
        [1, "widget", "tag|widget|serial|6", 0],
        ["tag|widget|serial|6", "tag", "widget", 0],
        ["tag|widget|serial|6", "serial", 6, 0],
    ]);
    assert.end();
});
test("Aggregate: committed sort in choose with post filtering greater than", function (assert) {
    var prog = new dsl2_1.Program("test")
        .commit("Clear events when they come in.", function (_a) {
        var find = _a.find;
        var event = find("event/create-widget");
        return [event.remove()];
    })
        .commit("Create a new widget of the given model.", function (_a) {
        var find = _a.find, choose = _a.choose, gather = _a.gather, record = _a.record;
        var model = find("event/create-widget").model;
        // The serial number of our next widget is the highest serial we've issued for this model so far + 1.
        var serial = choose(function () {
            var other = model.widget;
            2 > gather(other.serial).per(model).sort("down");
            return other.serial + 1;
        }, function () { return 1; })[0];
        return [model.add("widget", record("widget", { serial: serial }))];
    });
    util_1.verify(assert, prog, [
        [1, "tag", "model"],
        [1, "widget", 2],
        [1, "widget", 3],
        [2, "tag", "widget"],
        [2, "serial", 3],
        [3, "tag", "widget"],
        [3, "serial", 5],
        [4, "tag", "event/create-widget"],
        [4, "model", 1],
    ], [
        [4, "tag", "event/create-widget", 0, -1],
        [4, "model", 1, 0, -1],
        [1, "widget", "tag|widget|serial|6", 0],
        ["tag|widget|serial|6", "tag", "widget", 0],
        ["tag|widget|serial|6", "serial", 6, 0],
    ]);
    assert.end();
});
test("Aggregate: committed sort with multiple groups", function (assert) {
    var prog = new dsl2_1.Program("test")
        .commit("Clear events when they come in.", function (_a) {
        var find = _a.find;
        var event = find("event/create-widget");
        return [event.remove()];
    })
        .commit("Create a new widget of the given model.", function (_a) {
        var find = _a.find, choose = _a.choose, gather = _a.gather, record = _a.record;
        var model = find("event/create-widget").model;
        // The serial number of our next widget is the highest serial we've issued for this model so far + 1.
        var serial = choose(function () {
            var other = model.widget;
            2 > gather(other.serial).per(model).sort("down");
            return other.serial + 1;
        }, function () { return 1; })[0];
        return [model.add("widget", record("widget", { serial: serial }))];
    });
    util_1.verify(assert, prog, [
        [1, "tag", "model"],
        [1, "widget", 2],
        [1, "widget", 3],
        [2, "tag", "widget"],
        [2, "serial", 3],
        [3, "tag", "widget"],
        [3, "serial", 5],
        [5, "tag", "model"],
        [5, "widget", 6],
        [6, "tag", "widget"],
        [6, "serial", 28],
        [5, "widget", 7],
        [7, "tag", "widget"],
        [7, "serial", 30],
        [4, "tag", "event/create-widget"],
        [4, "model", 1],
    ], [
        [4, "tag", "event/create-widget", 0, -1],
        [4, "model", 1, 0, -1],
        [1, "widget", "tag|widget|serial|6", 0],
        ["tag|widget|serial|6", "tag", "widget", 0],
        ["tag|widget|serial|6", "serial", 6, 0],
    ]);
    assert.end();
});
test("Sort: incremental updates", function (assert) {
    var prog = new dsl2_1.Program("test");
    prog.bind("the block's next is the highest node sort + 1.", function (_a) {
        var find = _a.find, gather = _a.gather, record = _a.record;
        var block = find("block");
        var node = block.node;
        2 > gather(node.sort).per(block).sort("down");
        var sort = node.sort + 1;
        return [block.add("next", sort)];
    });
    util_1.verify(assert, prog, [
        [1, "tag", "block"],
        [1, "node", 2],
        [2, "sort", 1],
    ], [
        [1, "next", 2, 1]
    ]);
    util_1.verify(assert, prog, [
        [1, "node", 3],
        [3, "sort", 2],
    ], [
        [1, "next", 3, 1],
        [1, "next", 2, 1, -1],
    ]);
    util_1.verify(assert, prog, [
        [1, "node", 4],
        [4, "sort", 5],
    ], [
        [1, "next", 6, 1],
        [1, "next", 3, 1, -1],
    ]);
    assert.end();
});
test("Aggregate: inside choose without outer in key", function (assert) {
    var prog = new dsl2_1.Program("test");
    prog.bind("count the names of people", function (_a) {
        var find = _a.find, gather = _a.gather, record = _a.record, choose = _a.choose;
        var person = find("person");
        var sort = choose(function () {
            return gather(person.name).count();
        }, function () { return "yo yo yo"; })[0];
        return [person.add("next", sort)];
    });
    util_1.verify(assert, prog, [
        [1, "tag", "person"],
        [1, "name", "chris"],
        [1, "name", "christopher"],
        [2, "name", "joe"],
    ], [
        [1, "next", 2, 1],
    ]);
    assert.end();
});
test("Aggregate: no outer in key variations", function (assert) {
    var prog = new dsl2_1.Program("test");
    prog.bind("count the names of people", function (_a) {
        var find = _a.find, gather = _a.gather, record = _a.record, choose = _a.choose;
        var person = find("person");
        var sort = choose(function () {
            return gather(person.name).count();
        }, function () { return "yo yo yo"; })[0];
        return [person.add("next", sort)];
    });
    util_1.verify(assert, prog, [
        [1, "tag", "person"],
        [1, "name", "chris"],
        [1, "name", "christopher"],
        [2, "name", "joe"],
    ], [
        [1, "next", 2, 1],
    ]);
    util_1.verify(assert, prog, [
        [1, "tag", "person", 0, -1],
    ], [
        [1, "next", 2, 1, -1],
    ]);
    util_1.verify(assert, prog, [
        [1, "name", "chris", 0, -1],
        [1, "tag", "person"],
    ], [
        [1, "next", 1, 1],
    ]);
    util_1.verify(assert, prog, [
        [1, "name", "chris"],
        [1, "tag", "person", 0, -1],
    ], [
        [1, "next", 1, 1, -1],
    ]);
    assert.end();
});
test("Aggregate: Test limit", function (assert) {
    var prog = new dsl2_1.Program("test");
    prog.bind("Find up to two people", function (_a) {
        var find = _a.find, gather = _a.gather, record = _a.record, choose = _a.choose;
        var person = find("person");
        gather(person).sort() <= 2;
        return [record({ person: person })];
    });
    util_1.verify(assert, prog, [
        [1, "tag", "person"],
        [2, "tag", "person"],
        [3, "tag", "person"],
    ], [
        ["A", "person", 1, 1],
        ["B", "person", 2, 1],
    ]);
    util_1.verify(assert, prog, [
        [1, "tag", "person", 0, -1],
    ], [
        ["A", "person", 1, 1, -1],
        ["C", "person", 3, 1],
    ]);
    util_1.verify(assert, prog, [
        [2, "tag", "person", 0, -1],
    ], [
        ["B", "person", 2, 1, -1],
    ]);
    assert.end();
});
// @NOTE: The not following the choose required for this example is currently marked dangerous
// test("Aggregate: stratified after choose", (assert) => {
//   let prog = new Program("test");
//   prog.bind("Count the next of kin", ({find, gather, choose, not, record}) => {
//     let person = find("person");
//     let [kin] = choose(() => person.family, () => person.friend, () => person.acquaintance);
//     not(() => kin.nemesis == person);
//     let count = gather(kin).per(person).count();
//     return [person.add("kin_count", count)];
//   });
//   verify(assert, prog, [
//     [1, "tag", "person"],
//     [1, "name", "chris"],
//     [1, "friend", "joe"],
//     [1, "friend", "fred"],
//     [1, "friend", "steve"],
//     ["steve", "nemesis", 1],
//   ], [
//     [1, "kin_count", 2, 1],
//   ]);
//   verify(assert, prog, [
//     [1, "tag", "person", 0, -1],
//   ], [
//     [1, "kin_count", 2, 1, -1],
//   ]);
//   assert.end();
// });
//# sourceMappingURL=aggregate.js.map