"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var dsl2_1 = require("../src/runtime/dsl2");
var util_1 = require("./util");
var test = require("tape");
function createProgram() {
    var prog = new dsl2_1.Program("test");
    prog.bind("simple block", function (_a) {
        var find = _a.find, record = _a.record, not = _a.not;
        var left = find("left");
        not(function () {
            find("right", { left: left });
        });
        return [
            record("success")
        ];
    });
    return prog;
}
test("Antijoin: simple left", function (assert) {
    var prog = createProgram();
    util_1.verify(assert, prog, [
        [1, "tag", "left"]
    ], [
        [2, "tag", "success", 1]
    ]);
    assert.end();
});
test("Antijoin: simple right", function (assert) {
    var prog = createProgram();
    util_1.verify(assert, prog, [
        [1, "tag", "right"]
    ], []);
    util_1.verify(assert, prog, [
        [1, "left", 2]
    ], []);
    util_1.verify(assert, prog, [
        [2, "tag", "left"]
    ], []);
    assert.end();
});
test("Antijoin: simple left then right", function (assert) {
    var prog = createProgram();
    util_1.verify(assert, prog, [
        [2, "tag", "left"]
    ], [
        [3, "tag", "success", 1]
    ]);
    util_1.verify(assert, prog, [
        [1, "tag", "right"],
        [1, "left", 2],
    ], [
        [3, "tag", "success", 1, -1]
    ]);
    assert.end();
});
test("Antijoin: simple left then right same transaction", function (assert) {
    var prog = createProgram();
    util_1.verify(assert, prog, [
        [2, "tag", "left"],
        [1, "tag", "right", 1],
        [1, "left", 2, 1],
    ], [
        [3, "tag", "success", 1],
        [3, "tag", "success", 2, -1]
    ]);
    assert.end();
});
test("Antijoin: simple right then left same transaction", function (assert) {
    var prog = createProgram();
    util_1.verify(assert, prog, [
        [1, "tag", "right"],
        [1, "left", 2],
        [2, "tag", "left", 1],
    ], []);
    assert.end();
});
test("Antijoin: right -> left -> -right", function (assert) {
    var prog = createProgram();
    util_1.verify(assert, prog, [
        [1, "tag", "right"],
        [1, "left", 2],
        [2, "tag", "left", 1],
        [1, "tag", "right", 2, -1],
        [1, "left", 2, 2, -1],
    ], [
        [3, "tag", "success", 3],
    ]);
    assert.end();
});
test("Antijoin: right -> right -> left -> -right", function (assert) {
    var prog = createProgram();
    util_1.verify(assert, prog, [
        [1, "tag", "right"],
        [1, "left", 2],
        [4, "tag", "right", 5],
        [4, "left", 2, 5],
    ], []);
    util_1.verify(assert, prog, [
        [2, "tag", "left", 2],
        [1, "tag", "right", 3, -1],
        [1, "left", 2, 3, -1],
    ], [
        [3, "tag", "success", 4],
        [3, "tag", "success", 6, -1],
    ]);
    assert.end();
});
test("Antijoin: right -> -right -> right -> left", function (assert) {
    var prog = createProgram();
    util_1.verify(assert, prog, [
        [1, "tag", "right"],
        [1, "tag", "right", 3, -1],
        [4, "tag", "right", 5],
        [1, "left", 2],
        [1, "left", 2, 3, -1],
        [4, "left", 2, 5],
    ], []);
    util_1.verify(assert, prog, [
        [2, "tag", "left"],
    ], [
        [3, "tag", "success", 4],
        [3, "tag", "success", 6, -1],
    ]);
    assert.end();
});
test("Antijoin: left -> right -> -right", function (assert) {
    var prog = createProgram();
    util_1.verify(assert, prog, [
        [2, "tag", "left"],
        [4, "tag", "right", 5],
        [4, "left", 2, 5],
    ], [
        // nothing
        [3, "tag", "success", 1],
        [3, "tag", "success", 6, -1],
    ]);
    util_1.verify(assert, prog, [
        [4, "tag", "right", 5, -1],
        [4, "left", 2, 5, -1],
    ], [
        [3, "tag", "success", 6, 1],
    ]);
    assert.end();
});
test("Antijoin: right -> right -> left", function (assert) {
    var prog = createProgram();
    util_1.verify(assert, prog, [
        [1, "tag", "right", 4],
        [1, "left", 2, 4],
        [4, "tag", "right", 5],
        [4, "left", 2, 5],
    ], []);
    util_1.verify(assert, prog, [
        [2, "tag", "left"],
    ], [
        [3, "tag", "success", 1],
        [3, "tag", "success", 5, -1],
    ]);
    assert.end();
});
var programs = {
    "simple": function () {
        var prog = new dsl2_1.Program("simple");
        prog.bind("simple block", function (_a) {
            var find = _a.find, not = _a.not, record = _a.record;
            var input = find("input");
            not(function () { return input.arg0; });
            return [
                record("result")
            ];
        });
        return prog;
    },
    "dynamic": function () {
        var prog = new dsl2_1.Program("simple");
        prog.bind("simple block", function (_a) {
            var find = _a.find, not = _a.not, record = _a.record;
            var input = find("input");
            not(function () { return input.arg0; });
            return [
                record("result", { output: input })
            ];
        });
        return prog;
    },
};
var verifyIO = util_1.createVerifier(programs);
// -----------------------------------------------------
// simple
// -----------------------------------------------------
test("AntiJoin: simple +A; -A; +A", function (assert) {
    verifyIO(assert, "simple", "+A; -A; +A", [
        [[2, "tag", "result", 1, +1]],
        [[2, "tag", "result", 1, -1]],
        [[2, "tag", "result", 1, +1]]
    ]);
});
test("AntiJoin: simple +A; +B; -A; -B", function (assert) {
    verifyIO(assert, "simple", "+A; +B; -A; -B", [
        [[2, "tag", "result", 1, +1]],
        [],
        [],
        [[2, "tag", "result", 1, -1]],
    ]);
});
test("AntiJoin: simple +A, -A, +A", function (assert) {
    verifyIO(assert, "simple", "+A, -A, +A", [
        [[2, "tag", "result", 1, +1],
            [2, "tag", "result", 2, -1],
            [2, "tag", "result", 3, +1]]
    ]);
});
test("AntiJoin: simple +A, +B, -A, -B", function (assert) {
    verifyIO(assert, "simple", "+A, +B, -A, -B", [
        [[2, "tag", "result", 1, +1],
            [2, "tag", "result", 4, -1]],
    ]);
});
test("AntiJoin: simple +A; +A:1; -A:1; -A", function (assert) {
    verifyIO(assert, "simple", "+A; +A:1; -A:1; -A", [
        [[2, "tag", "result", 1, +1]],
        [[2, "tag", "result", 1, -1]],
        [[2, "tag", "result", 1, +1]],
        [[2, "tag", "result", 1, -1]]
    ]);
});
test("AntiJoin: simple +A; +A:1; +A:2; -A:1; -A:2", function (assert) {
    verifyIO(assert, "simple", "+A; +A:1; +A:2; -A:1; -A:2", [
        [[2, "tag", "result", 1, +1]],
        [[2, "tag", "result", 1, -1]],
        [],
        [],
        [[2, "tag", "result", 1, +1]]
    ]);
});
test("AntiJoin: simple +A; +B:1; -A; +C", function (assert) {
    verifyIO(assert, "simple", "+A; +B:1; -A; +C", [
        [[2, "tag", "result", 1, +1]],
        [],
        [[2, "tag", "result", 1, -1]],
        [[2, "tag", "result", 1, +1]]
    ]);
});
test("AntiJoin: simple +A; +B:1; +C; +A:1", function (assert) {
    verifyIO(assert, "simple", "+A; +B:1; +C; +A:1", [
        [[2, "tag", "result", 1, +1]],
        [],
        [],
        []
    ]);
});
// -----------------------------------------------------
// dynamic
// -----------------------------------------------------
test("AntiJoin: dynamic +A; -A; +A", function (assert) {
    verifyIO(assert, "dynamic", "+A; -A; +A", [
        [[2, "tag", "result", 1, +1], [2, "output", "A", 1, +1]],
        [[2, "tag", "result", 1, -1], [2, "output", "A", 1, -1]],
        [[2, "tag", "result", 1, +1], [2, "output", "A", 1, +1]]
    ]);
});
test("AntiJoin: dynamic +A; +B; -A; -B", function (assert) {
    verifyIO(assert, "dynamic", "+A; +B; -A; -B", [
        [[2, "tag", "result", 1, +1], [2, "output", "A", 1, +1]],
        [[2, "tag", "result", 1, +1], [2, "output", "B", 1, +1]],
        [[2, "tag", "result", 1, -1], [2, "output", "A", 1, -1]],
        [[2, "tag", "result", 1, -1], [2, "output", "B", 1, -1]],
    ]);
});
//# sourceMappingURL=antijoin.js.map