"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var dsl2_1 = require("../src/runtime/dsl2");
var util_1 = require("./util");
var test = require("tape");
test("Choose: basic", function (assert) {
    var prog = new dsl2_1.Program("test");
    prog.bind("simple block", function (_a) {
        var find = _a.find, record = _a.record, lib = _a.lib, choose = _a.choose;
        var person = find("person");
        var info = choose(function () {
            person.dog;
            return "cool";
        }, function () {
            return "not cool";
        })[0];
        return [
            record("dog-less", { info: info })
        ];
    });
    util_1.verify(assert, prog, [
        [1, "tag", "person"],
    ], [
        [2, "tag", "dog-less", 1],
        [2, "info", "not cool", 1],
    ]);
    util_1.verify(assert, prog, [
        [1, "dog", "spot"],
    ], [
        [2, "tag", "dog-less", 1, -1],
        [2, "info", "not cool", 1, -1],
        [3, "tag", "dog-less", 1],
        [3, "info", "cool", 1],
    ]);
    assert.end();
});
test("Choose: 3 branches", function (assert) {
    var prog = new dsl2_1.Program("test");
    prog.bind("simple block", function (_a) {
        var find = _a.find, record = _a.record, lib = _a.lib, choose = _a.choose;
        var person = find("person");
        var info = choose(function () {
            person.dog;
            return "cool";
        }, function () {
            person.foo;
            return "zomg";
        }, function () {
            return "not cool";
        })[0];
        return [
            record("dog-less", { info: info })
        ];
    });
    util_1.verify(assert, prog, [
        [1, "tag", "person"],
    ], [
        [2, "tag", "dog-less", 1],
        [2, "info", "not cool", 1],
    ]);
    util_1.verify(assert, prog, [
        [1, "dog", "spot"],
    ], [
        [2, "tag", "dog-less", 1, -1],
        [2, "info", "not cool", 1, -1],
        [3, "tag", "dog-less", 1],
        [3, "info", "cool", 1],
    ]);
    util_1.verify(assert, prog, [
        [1, "dog", "spot", 0, -1],
        [1, "foo", "woop"],
    ], [
        [3, "tag", "dog-less", 1, -1],
        [3, "info", "cool", 1, -1],
        [4, "tag", "dog-less", 1],
        [4, "info", "zomg", 1],
    ]);
    util_1.verify(assert, prog, [
        [1, "foo", "woop", 0, -1],
    ], [
        [3, "tag", "dog-less", 1, -1],
        [3, "info", "zomg", 1, -1],
        [4, "tag", "dog-less", 1],
        [4, "info", "not cool", 1],
    ]);
    assert.end();
});
test("Choose: 4 branches", function (assert) {
    var prog = new dsl2_1.Program("test");
    prog.bind("simple block", function (_a) {
        var find = _a.find, record = _a.record, lib = _a.lib, choose = _a.choose;
        var person = find("person");
        var boat = person.boat;
        var info = choose(function () {
            person.dog;
            return "cool";
        }, function () {
            person.foo;
            return "zomg";
        }, function () {
            boat.foo;
            return "woah";
        }, function () {
            return "not cool";
        })[0];
        return [
            record("dog-less", { info: info })
        ];
    });
    util_1.verify(assert, prog, [
        [1, "tag", "person"],
        [1, "boat", 9],
    ], [
        [2, "tag", "dog-less", 1],
        [2, "info", "not cool", 1],
    ]);
    util_1.verify(assert, prog, [
        [1, "dog", "spot"],
    ], [
        [2, "tag", "dog-less", 1, -1],
        [2, "info", "not cool", 1, -1],
        [3, "tag", "dog-less", 1],
        [3, "info", "cool", 1],
    ]);
    util_1.verify(assert, prog, [
        [1, "dog", "spot", 0, -1],
        [1, "foo", "woop"],
    ], [
        [3, "tag", "dog-less", 1, -1],
        [3, "info", "cool", 1, -1],
        [4, "tag", "dog-less", 1],
        [4, "info", "zomg", 1],
    ]);
    util_1.verify(assert, prog, [
        [1, "foo", "woop", 0, -1],
    ], [
        [3, "tag", "dog-less", 1, -1],
        [3, "info", "zomg", 1, -1],
        [4, "tag", "dog-less", 1],
        [4, "info", "not cool", 1],
    ]);
    util_1.verify(assert, prog, [
        [9, "foo", "meep moop"],
    ], [
        [3, "tag", "dog-less", 1, -1],
        [3, "info", "not cool", 1, -1],
        [4, "tag", "dog-less", 1],
        [4, "info", "woah", 1],
    ]);
    util_1.verify(assert, prog, [
        [9, "foo", "meep moop", 0, -1],
    ], [
        [3, "tag", "dog-less", 1, -1],
        [3, "info", "woah", 1, -1],
        [4, "tag", "dog-less", 1],
        [4, "info", "not cool", 1],
    ]);
    util_1.verify(assert, prog, [
        [1, "dog", "spot"],
    ], [
        [3, "tag", "dog-less", 1, -1],
        [3, "info", "not cool", 1, -1],
        [4, "tag", "dog-less", 1],
        [4, "info", "cool", 1],
    ]);
    util_1.verify(assert, prog, [
        [5, "tag", "person"],
        [5, "boat", 10],
    ], [
        [4, "tag", "dog-less", 1],
        [4, "info", "not cool", 1],
    ]);
    util_1.verify(assert, prog, [
        [5, "tag", "person", 0, -1],
        [1, "tag", "person", 0, -1],
    ], [
        [4, "tag", "dog-less", 1, -1],
        [4, "info", "not cool", 1, -1],
        [6, "tag", "dog-less", 1, -1],
        [6, "info", "cool", 1, -1],
    ]);
    util_1.verify(assert, prog, [
        [5, "tag", "person"],
        [1, "tag", "person"],
    ], [
        [4, "tag", "dog-less", 1],
        [4, "info", "not cool", 1],
        [6, "tag", "dog-less", 1],
        [6, "info", "cool", 1],
    ]);
    assert.end();
});
// @TODO: Give this a better name when we figure out the specific issue.
test("Choose: Busted partial identity", function (assert) {
    var prog = new dsl2_1.Program("test");
    prog.bind("Split up our cat attributes", function (_a) {
        var find = _a.find, lookup = _a.lookup, record = _a.record;
        var cat = find("cat");
        var _b = lookup(cat), attribute = _b.attribute, value = _b.value;
        return [
            // @NOTE: Issue has to do with add, can't repro if value is part of the identity.
            record("cat-attribute", { cat: cat, attribute: attribute }).add("value", value)
        ];
    });
    prog.bind("Create value records for each cat attribute.", function (_a) {
        var find = _a.find, lookup = _a.lookup, choose = _a.choose, record = _a.record;
        var catAttribute = find("cat-attribute");
        // Tags about cats are cool.
        // @FIXME: In some (but not all) cases where the first branch matches both branches emit.
        //         This may be multiplicity/retraction related.
        var attrName = choose(function () { catAttribute.attribute == "tag"; return "cool tags"; }, function () { return catAttribute.attribute; })[0];
        var cat = catAttribute.cat, value = catAttribute.value;
        return [
            record("cat-value", { cat: cat, attr: attrName, val: value })
        ];
    });
    util_1.verify(assert, prog, [
        [1, "tag", "pet"],
        [1, "tag", "cat"],
        [1, "name", "Felicia"],
    ], [
        [2, "tag", "cat-attribute", 1],
        [2, "cat", 1, 1],
        [2, "attribute", "tag", 1],
        [2, "value", "pet", 1],
        [2, "value", "cat", 1],
        [3, "tag", "cat-attribute", 1],
        [3, "cat", 1, 1],
        [3, "attribute", "name", 1],
        [3, "value", "Felicia", 1],
        [4, "tag", "cat-value", 2],
        [4, "cat", 1, 2],
        [4, "attr", "cool tags", 2],
        [4, "val", "pet", 2],
        [5, "tag", "cat-value", 2],
        [5, "cat", 1, 2],
        [5, "attr", "cool tags", 2],
        [5, "val", "cat", 2],
        [6, "tag", "cat-value", 2],
        [6, "cat", 1, 2],
        [6, "attr", "name", 2],
        [6, "val", "Felicia", 2],
    ]);
    assert.end();
});
test("Choose: multiple return", function (assert) {
    var prog = new dsl2_1.Program("test");
    prog.bind("simple block", function (_a) {
        var find = _a.find, record = _a.record, lib = _a.lib, choose = _a.choose;
        var person = find("person");
        var _b = choose(function () {
            return [person.nickName, "cool"];
        }, function () {
            return [person.name, "not cool"];
        }), displayName = _b[0], coolness = _b[1];
        return [
            person.add("displayName", displayName),
            person.add("coolness", coolness),
        ];
    });
    util_1.verify(assert, prog, [
        [1, "tag", "person"],
        [1, "name", "joseph"],
    ], [
        [1, "displayName", "joseph", 1],
        [1, "coolness", "not cool", 1],
    ]);
    util_1.verify(assert, prog, [
        [1, "nickName", "joey"],
    ], [
        [1, "displayName", "joseph", 1, -1],
        [1, "coolness", "not cool", 1, -1],
        [1, "displayName", "joey", 1],
        [1, "coolness", "cool", 1],
    ]);
    util_1.verify(assert, prog, [
        [1, "nickName", "joey", 0, -1],
    ], [
        [1, "displayName", "joseph", 1],
        [1, "coolness", "not cool", 1],
        [1, "displayName", "joey", 1, -1],
        [1, "coolness", "cool", 1, -1],
    ]);
    assert.end();
});
test("Choose: moves only", function (assert) {
    var prog = new dsl2_1.Program("test");
    prog.bind("simple block", function (_a) {
        var find = _a.find, record = _a.record, choose = _a.choose;
        var person = find("person");
        var name = person.name;
        var displayName = choose(function () { name == "christopher"; return "chris"; }, function () { return name; })[0];
        return [
            person.add({ displayName: displayName })
        ];
    });
    util_1.verify(assert, prog, [
        [1, "tag", "person"],
        [1, "name", "christopher"],
        [2, "tag", "person"],
        [2, "name", "jane"],
    ], [
        [1, "displayName", "chris", 1],
        [2, "displayName", "jane", 1],
    ]);
    assert.end();
});
test("Choose: post-filtering outer", function (assert) {
    var prog = new dsl2_1.Program("test");
    prog.bind("froofy", function (_a) {
        var find = _a.find, choose = _a.choose, record = _a.record;
        var person = find("person");
        var display = choose(function () { return person.display; })[0];
        display.name == "Ferdinand";
        return [record("result", { name: display.name })];
    });
    util_1.verify(assert, prog, [
        [1, "tag", "person"],
        [1, "display", 2],
        [2, "name", "Jess"],
        [3, "tag", "cat"],
        [3, "display", 4],
        [4, "name", "Ferdinand"],
    ], []);
    assert.end();
});
test("Choose: expression-only dynamic branch", function (assert) {
    var prog = new dsl2_1.Program("test");
    prog.bind("Choose non-static expression only.", function (_a) {
        var find = _a.find, choose = _a.choose, record = _a.record;
        var guy = find("guy");
        var radness = guy.radness;
        var radometer = choose(function () { return radness * 3; })[0]; // This does not.
        // let radometer = radness * 3; // This works
        return [guy.add("radometer", radometer)];
    });
    util_1.verify(assert, prog, [
        [1, "tag", "guy"],
        [1, "radness", 1],
    ], [
        [1, "radometer", 3, 1]
    ]);
    assert.end();
});
test("Choose: filter and expression-only dynamic branches", function (assert) {
    var prog = new dsl2_1.Program("test");
    prog.bind("Choose non-static expression only.", function (_a) {
        var find = _a.find, choose = _a.choose, record = _a.record;
        var guy = find("guy");
        var radness = guy.radness;
        // We need to adjust the scale since radness is roughly logarithmic.
        var radometer = choose(function () { radness < 2; return radness; }, function () { radness < 4; return radness * 2; }, function () { return radness * 3; })[0];
        // let radometer = radness * 3;
        return [guy.add("radometer", radometer)];
    });
    util_1.verify(assert, prog, [
        [1, "tag", "guy"],
        [1, "radness", 0],
        [2, "tag", "guy"],
        [2, "radness", 1],
        [3, "tag", "guy"],
        [3, "radness", 2],
        [4, "tag", "guy"],
        [4, "radness", 4],
    ], [
        [1, "radometer", 0, 1],
        [2, "radometer", 1, 1],
        [3, "radometer", 4, 1],
        [4, "radometer", 12, 1]
    ]);
    util_1.verify(assert, prog, [
        [1, "radness", 0, 0, -1],
        [1, "radness", 8],
    ], [
        [1, "radometer", 24, 1],
        [1, "radometer", 0, 1, -1],
    ]);
    assert.end();
});
var programs = {
    "1 static": function () {
        var prog = new dsl2_1.Program("1 static branch");
        prog.bind("simple block", function (_a) {
            var find = _a.find, choose = _a.choose, record = _a.record;
            var foo = find("input");
            var branch = choose(function () { return 1; })[0];
            return [
                record("result", { branch: branch })
            ];
        });
        return prog;
    },
    "1 dynamic": function () {
        var prog = new dsl2_1.Program("1 dynamic branch");
        prog.bind("simple block", function (_a) {
            var find = _a.find, choose = _a.choose, record = _a.record;
            var foo = find("input");
            var output = choose(function () { return foo.arg0; })[0];
            return [
                record("result", { output: output })
            ];
        });
        return prog;
    },
    "1 dynamic 1 static": function () {
        var prog = new dsl2_1.Program("1 dynamic branch");
        prog.bind("simple block", function (_a) {
            var find = _a.find, choose = _a.choose, record = _a.record;
            var foo = find("input");
            var output = choose(function () { foo.arg0 == 1; return "one"; }, function () { return "else"; })[0];
            return [
                record("result", { output: output })
            ];
        });
        return prog;
    },
    "2 dynamic": function () {
        var prog = new dsl2_1.Program("1 dynamic branch");
        prog.bind("simple block", function (_a) {
            var find = _a.find, choose = _a.choose, record = _a.record;
            var foo = find("input");
            var output = choose(function () { foo.arg0 == 1; return "one"; }, function () { return foo.arg0; })[0];
            return [
                record("result", { output: output })
            ];
        });
        return prog;
    },
};
var verifyBranches = util_1.createVerifier(programs);
// -----------------------------------------------------
// 1 Static branch
// -----------------------------------------------------
test("Choose: 1 static branch +A; -A; +A", function (assert) {
    verifyBranches(assert, "1 static", "+A; -A; +A", [
        [[2, "tag", "result", 1, +1], [2, "branch", 1, 1, +1]],
        [[2, "tag", "result", 1, -1], [2, "branch", 1, 1, -1]],
        [[2, "tag", "result", 1, +1], [2, "branch", 1, 1, +1]]
    ]);
});
test("Choose: 1 static branch +A; -A; +B", function (assert) {
    verifyBranches(assert, "1 static", "+A; -A; +B", [
        [[2, "tag", "result", 1, +1], [2, "branch", 1, 1, +1]],
        [[2, "tag", "result", 1, -1], [2, "branch", 1, 1, -1]],
        [[2, "tag", "result", 1, +1], [2, "branch", 1, 1, +1]]
    ]);
});
// @NOTE: Broken due to verify being too simple.
test("Choose: 1 static branch +A; +A; -A", function (assert) {
    verifyBranches(assert, "1 static", "+A; +A; -A", [
        [[2, "tag", "result", 1, +1], [2, "branch", 1, 1, +1]],
        [],
        []
    ]);
});
// @NOTE: Broken due to verify being too simple.
test("Choose: 1 static branch +A; +A; -A; -A", function (assert) {
    verifyBranches(assert, "1 static", "+A; +A; -A; -A", [
        [[2, "tag", "result", 1, +1], [2, "branch", 1, 1, +1]],
        [],
        [],
        [[2, "tag", "result", 1, -1], [2, "branch", 1, 1, -1]],
    ]);
});
test("Choose: 1 static branch +A +B; -A; +A", function (assert) {
    verifyBranches(assert, "1 static", "+A +B; -A; +A", [
        [[2, "tag", "result", 1, +1], [2, "branch", 1, 1, +1]],
        [],
        []
    ]);
});
test("Choose: 1 static branch +A +B; -A -B", function (assert) {
    verifyBranches(assert, "1 static", "+A +B; -A -B", [
        [[2, "tag", "result", 1, +1], [2, "branch", 1, 1, +1]],
        [[2, "tag", "result", 1, -1], [2, "branch", 1, 1, -1]]
    ]);
});
test("Choose: 1 static branch +A; -A +B", function (assert) {
    verifyBranches(assert, "1 static", "+A; -A +B", [
        [[2, "tag", "result", 1, +1], [2, "branch", 1, 1, +1]],
        []
    ]);
});
test("Choose: 1 static branch +A, -A", function (assert) {
    verifyBranches(assert, "1 static", "+A, -A", [
        [[2, "tag", "result", 1, +1], [2, "branch", 1, 1, +1],
            [2, "tag", "result", 2, -1], [2, "branch", 1, 2, -1]]
    ]);
});
test("Choose: 1 static branch +A, -A +B", function (assert) {
    verifyBranches(assert, "1 static", "+A, -A +B", [
        [[2, "tag", "result", 1, +1], [2, "branch", 1, 1, +1]]
    ]);
});
test("Choose: 1 static branch +A, +B", function (assert) {
    verifyBranches(assert, "1 static", "+A, +B", [
        [[2, "tag", "result", 1, +1], [2, "branch", 1, 1, +1]]
    ]);
});
test("Choose: 1 static branch +A, +B; -A", function (assert) {
    verifyBranches(assert, "1 static", "+A, +B; -A", [
        [[2, "tag", "result", 1, +1], [2, "branch", 1, 1, +1]],
        [[2, "tag", "result", 1, -1], [2, "branch", 1, 1, -1],
            [2, "tag", "result", 2, +1], [2, "branch", 1, 2, +1]]
    ]);
});
test("Choose: 1 static branch +A; -A, +B", function (assert) {
    verifyBranches(assert, "1 static", "+A; -A, +B", [
        [[2, "tag", "result", 1, +1], [2, "branch", 1, 1, +1]],
        [[2, "tag", "result", 1, -1], [2, "branch", 1, 1, -1],
            [2, "tag", "result", 2, +1], [2, "branch", 1, 2, +1]]
    ]);
});
// -----------------------------------------------------
// 1 dynamic branch
// -----------------------------------------------------
test("Choose: 1 dynamic branch +A:1; -A:1", function (assert) {
    verifyBranches(assert, "1 dynamic", "+A:1; -A:1", [
        [[1, "tag", "result", 1, +1], [1, "output", 1, 1, +1]],
        [[1, "tag", "result", 1, -1], [1, "output", 1, 1, -1]],
    ]);
});
test("Choose: 1 dynamic branch +A:1; -A:1; +A:1", function (assert) {
    verifyBranches(assert, "1 dynamic", "+A:1; -A:1; +A:1", [
        [[1, "tag", "result", 1, +1], [1, "output", 1, 1, +1]],
        [[1, "tag", "result", 1, -1], [1, "output", 1, 1, -1]],
        [[1, "tag", "result", 1, +1], [1, "output", 1, 1, +1]],
    ]);
});
test("Choose: 1 dynamic branch +A:1; -A:1; +A:2", function (assert) {
    verifyBranches(assert, "1 dynamic", "+A:1; -A:1; +A:2", [
        [[1, "tag", "result", 1, +1], [1, "output", 1, 1, +1]],
        [[1, "tag", "result", 1, -1], [1, "output", 1, 1, -1]],
        [[1, "tag", "result", 1, +1], [1, "output", 2, 1, +1]],
    ]);
});
test("Choose: 1 dynamic branch +A:1; +A:2; -A:1", function (assert) {
    verifyBranches(assert, "1 dynamic", "+A:1; +A:2; -A:1", [
        [[1, "tag", "result", 1, +1], [1, "output", 1, 1, +1]],
        [[1, "tag", "result", 1, +1], [1, "output", 2, 1, +1]],
        [[1, "tag", "result", 1, -1], [1, "output", 1, 1, -1]],
    ]);
});
test("Choose: 1 dynamic branch +A:1; +B:1; -A:1", function (assert) {
    verifyBranches(assert, "1 dynamic", "+A:1; +B:1; -A:1", [
        [[1, "tag", "result", 1, +1], [1, "output", 1, 1, +1]],
        [],
        [],
    ]);
});
test("Choose: 1 dynamic branch +A:1; +B:1; -A:1, -B:1", function (assert) {
    verifyBranches(assert, "1 dynamic", "+A:1; +B:1; -A:1, -B:1", [
        [[1, "tag", "result", 1, +1], [1, "output", 1, 1, +1]],
        [],
        [[1, "tag", "result", 2, -1], [1, "output", 1, 2, -1]],
    ]);
});
// -----------------------------------------------------
// 1 dynamic 1 static
// -----------------------------------------------------
test("Choose: 1 dynamic 1 static branch +A:1; -A:1; +A:1", function (assert) {
    verifyBranches(assert, "1 dynamic 1 static", "+A:1; -A:1; +A:1", [
        [[1, "tag", "result", 1, +1], [1, "output", "one", 1, +1]],
        [[1, "tag", "result", 1, -1], [1, "output", "one", 1, -1]],
        [[1, "tag", "result", 1, +1], [1, "output", "one", 1, +1]]
    ]);
});
test("Choose: 1 dynamic 1 static branch +A:2; -A:2; +A:2", function (assert) {
    verifyBranches(assert, "1 dynamic 1 static", "+A:2; -A:2; +A:2", [
        [[1, "tag", "result", 1, +1], [1, "output", "else", 1, +1]],
        [[1, "tag", "result", 1, -1], [1, "output", "else", 1, -1]],
        [[1, "tag", "result", 1, +1], [1, "output", "else", 1, +1]]
    ]);
});
test("Choose: 1 dynamic 1 static branch +A:1; -A:1; +A:2", function (assert) {
    verifyBranches(assert, "1 dynamic 1 static", "+A:1; -A:1; +A:2", [
        [[1, "tag", "result", 1, +1], [1, "output", "one", 1, +1]],
        [[1, "tag", "result", 1, -1], [1, "output", "one", 1, -1]],
        [[2, "tag", "result", 1, +1], [2, "output", "else", 1, +1]]
    ]);
});
test("Choose: 1 dynamic 1 static branch +A:1; +A:2", function (assert) {
    verifyBranches(assert, "1 dynamic 1 static", "+A:1; +A:2", [
        [[1, "tag", "result", 1, +1], [1, "output", "one", 1, +1]],
        []
    ]);
});
test("Choose: 1 dynamic 1 static branch +A:1; +B:2", function (assert) {
    verifyBranches(assert, "1 dynamic 1 static", "+A:1; +B:2", [
        [[1, "tag", "result", 1, +1], [1, "output", "one", 1, +1]],
        [[2, "tag", "result", 1, +1], [2, "output", "else", 1, +1]]
    ]);
});
test("Choose: 1 dynamic 1 static branch +A:1; +B:1; -A:1", function (assert) {
    verifyBranches(assert, "1 dynamic 1 static", "+A:1; +B:1; -A:1", [
        [[1, "tag", "result", 1, +1], [1, "output", "one", 1, +1]],
        [],
        []
    ]);
});
test("Choose: 1 dynamic 1 static branch +A:1, -A:1", function (assert) {
    verifyBranches(assert, "1 dynamic 1 static", "+A:1, -A:1", [
        [[1, "tag", "result", 1, +1], [1, "output", "one", 1, +1],
            [1, "tag", "result", 2, -1], [1, "output", "one", 2, -1]]
    ]);
});
test("Choose: 1 dynamic 1 static branch +A:1, -A:1, +A:1", function (assert) {
    verifyBranches(assert, "1 dynamic 1 static", "+A:1, -A:1, +A:1", [
        [[1, "tag", "result", 1, +1], [1, "output", "one", 1, +1],
            [1, "tag", "result", 2, -1], [1, "output", "one", 2, -1],
            [1, "tag", "result", 3, +1], [1, "output", "one", 3, +1]]
    ]);
});
test("Choose: 1 dynamic 1 static branch +A:1; +B:1; -B:1; +B:1", function (assert) {
    verifyBranches(assert, "1 dynamic 1 static", "+A:1; +B:1; -B:1; +B:1", [
        [[1, "tag", "result", 1, +1], [1, "output", "one", 1, +1]],
        [],
        [],
        []
    ]);
});
// -----------------------------------------------------
// 2 dynamics
// -----------------------------------------------------
test("Choose: 2 dynamic branch +A:1; -A:1; +A:1", function (assert) {
    verifyBranches(assert, "2 dynamic", "+A:1; -A:1; +A:1", [
        [[1, "tag", "result", 1, +1], [1, "output", "one", 1, +1]],
        [[1, "tag", "result", 1, -1], [1, "output", "one", 1, -1]],
        [[1, "tag", "result", 1, +1], [1, "output", "one", 1, +1]]
    ]);
});
test("Choose: 2 dynamic branch +A:2; -A:2; +A:2", function (assert) {
    verifyBranches(assert, "2 dynamic", "+A:2; -A:2; +A:2", [
        [[1, "tag", "result", 1, +1], [1, "output", 2, 1, +1]],
        [[1, "tag", "result", 1, -1], [1, "output", 2, 1, -1]],
        [[1, "tag", "result", 1, +1], [1, "output", 2, 1, +1]]
    ]);
});
test("Choose: 2 dynamic branch +A:1; -A:1; +A:2", function (assert) {
    verifyBranches(assert, "2 dynamic", "+A:1; -A:1; +A:2", [
        [[1, "tag", "result", 1, +1], [1, "output", "one", 1, +1]],
        [[1, "tag", "result", 1, -1], [1, "output", "one", 1, -1]],
        [[2, "tag", "result", 1, +1], [2, "output", 2, 1, +1]]
    ]);
});
test("Choose: 2 dynamic branch +A:1; +A:2", function (assert) {
    verifyBranches(assert, "2 dynamic", "+A:1; +A:2", [
        [[1, "tag", "result", 1, +1], [1, "output", "one", 1, +1]],
        []
    ]);
});
test("Choose: 2 dynamic branch +A:1; +B:2", function (assert) {
    verifyBranches(assert, "2 dynamic", "+A:1; +B:2", [
        [[1, "tag", "result", 1, +1], [1, "output", "one", 1, +1]],
        [[2, "tag", "result", 1, +1], [2, "output", 2, 1, +1]]
    ]);
});
test("Choose: 2 dynamic branch +A:1; +B:1; -A:1", function (assert) {
    verifyBranches(assert, "2 dynamic", "+A:1; +B:1; -A:1", [
        [[1, "tag", "result", 1, +1], [1, "output", "one", 1, +1]],
        [],
        []
    ]);
});
test("Choose: 2 dynamic branch +A:1, -A:1", function (assert) {
    verifyBranches(assert, "2 dynamic", "+A:1, -A:1", [
        [[1, "tag", "result", 1, +1], [1, "output", "one", 1, +1],
            [1, "tag", "result", 2, -1], [1, "output", "one", 2, -1]]
    ]);
});
test("Choose: 2 dynamic branch +A:1, -A:1, +A:1", function (assert) {
    verifyBranches(assert, "2 dynamic", "+A:1, -A:1, +A:1", [
        [[1, "tag", "result", 1, +1], [1, "output", "one", 1, +1],
            [1, "tag", "result", 2, -1], [1, "output", "one", 2, -1],
            [1, "tag", "result", 3, +1], [1, "output", "one", 3, +1]]
    ]);
});
test("Choose: 2 dynamic branch +A:1; +B:1; -B:1; +B:1", function (assert) {
    verifyBranches(assert, "2 dynamic", "+A:1; +B:1; -B:1; +B:1", [
        [[1, "tag", "result", 1, +1], [1, "output", "one", 1, +1]],
        [],
        [],
        []
    ]);
});
//# sourceMappingURL=choose.js.map