"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var indexes_1 = require("../src/runtime/indexes");
var runtime_1 = require("../src/runtime/runtime");
var test = require("tape");
function roundCountsToChanges(rcs) {
    var changes = [];
    for (var _i = 0, rcs_1 = rcs; _i < rcs_1.length; _i++) {
        var _a = rcs_1[_i], round = _a[0], count = _a[1];
        changes.push(new runtime_1.Change(1, 2, 3, 4, 1, round, count));
    }
    return changes;
}
function distinctTest(assert, roundCounts, expected) {
    var index = new indexes_1.DistinctIndex();
    var changes = roundCountsToChanges(roundCounts);
    var final = {};
    for (var _i = 0, changes_1 = changes; _i < changes_1.length; _i++) {
        var change = changes_1[_i];
        var neueChanges = new runtime_1.Iterator();
        index.distinct(change, neueChanges);
        var neue = void 0;
        while ((neue = neueChanges.next())) {
            final[neue.round] = (final[neue.round] || 0) + neue.count;
        }
    }
    var badKeys = {};
    for (var key in expected) {
        var finalValue = final[key];
        var expectedValue = expected[key];
        if (finalValue || expectedValue) {
            var valid = finalValue == expectedValue;
            assert.true(valid, "round " + key + " :: expected " + expected[key] + ", actual " + final[key]);
            if (!valid) {
                badKeys[key] = true;
            }
        }
    }
    for (var key in final) {
        if (badKeys[key])
            continue;
        var finalValue = final[key];
        var expectedValue = expected[key];
        if (finalValue || expectedValue) {
            var valid = finalValue == expectedValue;
            assert.true(valid, "round " + key + " :: expected " + expected[key] + ", actual " + final[key]);
            if (!valid) {
                badKeys[key] = true;
            }
        }
    }
}
var distinct = (function (name, roundCounts, expected) {
    test("Distinct: " + name, function (assert) {
        distinctTest(assert, roundCounts, expected);
        assert.end();
    });
});
distinct.only = function distinctOnly(name, roundCounts, expected) {
    test.only("Distinct: " + name, function (assert) {
        distinctTest(assert, roundCounts, expected);
        assert.end();
    });
};
distinct("basic", [
    [1, 1],
    [2, -1],
    [1, 1],
    [3, -1],
], {
    1: 1,
    3: -1
});
//------------------------------------------------------------
// Chris's section
//------------------------------------------------------------
distinct("basic 2", [
    [1, 1],
    [2, -1],
    [3, 1],
    [4, -1],
], {
    1: 1,
    2: -1,
    3: 1,
    4: -1,
});
distinct("basic 2 in reverse order", [
    [3, 1],
    [4, -1],
    [1, 1],
    [2, -1],
], {
    1: 1,
    2: -1,
    3: 1,
    4: -1,
});
distinct("basic 2 undone", [
    [1, 1],
    [2, -1],
    [3, 1],
    [4, -1],
    [1, -1],
    [2, 1],
], {
    3: 1,
    4: -1,
});
distinct("multiple counts", [
    [1, 1],
    [1, 1],
    [1, 1],
    [2, -1],
    [2, -1],
    [2, -1],
    [3, 1],
    [3, 1],
    [3, 1],
    [4, -1],
    [4, -1],
    [4, -1],
], {
    1: 1,
    2: -1,
    3: 1,
    4: -1,
});
distinct("multiple counts reversed", [
    [3, 1],
    [3, 1],
    [3, 1],
    [4, -1],
    [4, -1],
    [4, -1],
    [1, 1],
    [1, 1],
    [1, 1],
    [2, -1],
    [2, -1],
    [2, -1],
], {
    1: 1,
    2: -1,
    3: 1,
    4: -1,
});
distinct("multiple counts interleaved", [
    [3, 1],
    [4, -1],
    [3, 1],
    [4, -1],
    [3, 1],
    [4, -1],
    [1, 1],
    [2, -1],
    [1, 1],
    [2, -1],
    [1, 1],
    [2, -1],
], {
    1: 1,
    2: -1,
    3: 1,
    4: -1,
});
distinct("multiple counts negatives first", [
    [2, -1],
    [2, -1],
    [2, -1],
    [1, 1],
    [1, 1],
    [1, 1],
    [4, -1],
    [4, -1],
    [4, -1],
    [3, 1],
    [3, 1],
    [3, 1],
], {
    1: 1,
    2: -1,
    3: 1,
    4: -1,
});
distinct("multiple counts undone", [
    [1, 1],
    [1, 1],
    [1, 1],
    [2, -1],
    [2, -1],
    [2, -1],
    [3, 1],
    [3, 1],
    [3, 1],
    [4, -1],
    [4, -1],
    [4, -1],
    [1, -1],
    [1, -1],
    [1, -1],
    [2, 1],
    [2, 1],
    [2, 1],
], {
    3: 1,
    4: -1,
});
distinct("multiple counts undone interleaved", [
    [1, 1],
    [1, 1],
    [1, 1],
    [2, -1],
    [2, -1],
    [2, -1],
    [1, -1],
    [1, -1],
    [1, -1],
    [3, 1],
    [3, 1],
    [3, 1],
    [4, -1],
    [4, -1],
    [4, -1],
    [2, 1],
    [2, 1],
    [2, 1],
], {
    3: 1,
    4: -1,
});
distinct("multiple disparate counts", [
    [1, 1],
    [1, 1],
    [1, 1],
    [2, -1],
    [2, -1],
    [2, -1],
    [3, 1],
    [4, -1],
], {
    1: 1,
    2: -1,
    3: 1,
    4: -1,
});
distinct("multiple disparate counts with extra removes", [
    [1, 1],
    [1, 1],
    [1, 1],
    [2, -1],
    [2, -1],
    [2, -1],
    [1, -1],
    [1, -1],
    [1, -1],
    [2, 1],
    [2, 1],
    [2, 1],
    [3, 1],
    [4, -1],
], {
    3: 1,
    4: -1,
});
//------------------------------------------------------------
// Josh's section
//------------------------------------------------------------
distinct("simple round promotion", [
    [8, 1],
    [9, -1],
    [5, 1],
    [6, -1],
    [8, -1],
    [9, 1]
], {
    5: 1,
    6: -1,
    8: 0,
    9: 0
});
distinct("full promotion", [
    [9, 1],
    [9, 1],
    [10, -1],
    [10, -1],
    [9, 1],
    [9, 1],
    [10, -1],
    [10, -1],
    [9, -1],
    [10, 1],
    [9, -1],
    [10, 1],
    [9, -1],
    [10, 1],
    [9, -1],
    [10, 1]
], {
    9: 0,
    10: 0
});
distinct("positive full promotion", [
    [7, 1],
    [8, -1],
    [8, 1],
    [7, 1],
    [8, -1],
    [4, 1],
    [8, -1],
    [7, 1],
    [8, -1],
    [8, 1],
    [5, -1],
    [7, -3],
    [8, 1],
    [8, 3],
    [5, 1],
    [8, 1],
    [8, -2],
    [8, -1],
], {
    4: 1,
});
//# sourceMappingURL=distinct.js.map