"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var dsl2_1 = require("../src/runtime/dsl2");
var util_1 = require("./util");
var test = require("tape");
test("find a record and generate a record as a result", function (assert) {
    // -----------------------------------------------------
    // program
    // -----------------------------------------------------
    var prog = new dsl2_1.Program("test");
    prog.bind("simple block", function (_a) {
        var find = _a.find, record = _a.record, lib = _a.lib;
        find({ foo: "bar" });
        return [
            record({ zomg: "baz" })
        ];
    });
    // -----------------------------------------------------
    // verification
    // -----------------------------------------------------
    util_1.verify(assert, prog, [
        [1, "foo", "bar"]
    ], [
        [2, "zomg", "baz", 1]
    ]);
    assert.end();
});
test("> filters numbers", function (assert) {
    // -----------------------------------------------------
    // program
    // -----------------------------------------------------
    var prog = new dsl2_1.Program("test");
    prog.bind("simple block", function (_a) {
        var find = _a.find, record = _a.record, lib = _a.lib;
        var a = find();
        var b = find();
        a.age > b.age;
        return [
            record({ age1: a.age, age2: b.age })
        ];
    });
    // -----------------------------------------------------
    // verification
    // -----------------------------------------------------
    util_1.verify(assert, prog, [
        [1, "tag", "person"],
        [1, "age", 7],
        [2, "tag", "person"],
        [2, "age", 41],
        [3, "tag", "person"],
        [3, "age", 3],
    ], [
        [4, "age1", 41, 1],
        [4, "age2", 7, 1],
        [5, "age1", 41, 1],
        [5, "age2", 3, 1],
        [6, "age1", 7, 1],
        [6, "age2", 3, 1],
    ]);
    assert.end();
});
test("simple addition", function (assert) {
    // -----------------------------------------------------
    // program
    // -----------------------------------------------------
    var prog = new dsl2_1.Program("test");
    prog.bind("simple block", function (_a) {
        var find = _a.find, record = _a.record, lib = _a.lib;
        var a = find("person");
        var b = find("person");
        a.age > b.age;
        var result = a.age + b.age;
        return [
            record({ age1: a.age, age2: b.age, result: result })
        ];
    });
    // -----------------------------------------------------
    // verification
    // -----------------------------------------------------
    util_1.verify(assert, prog, [
        [1, "tag", "person"],
        [1, "age", 7],
        [2, "tag", "person"],
        [2, "age", 41],
        [3, "tag", "person"],
        [3, "age", 3],
    ], [
        [4, "age1", 41, 1],
        [4, "age2", 7, 1],
        [4, "result", 48, 1],
        [5, "age1", 41, 1],
        [5, "age2", 3, 1],
        [5, "result", 44, 1],
        [6, "age1", 7, 1],
        [6, "age2", 3, 1],
        [6, "result", 10, 1],
    ]);
    assert.end();
});
test("simple division", function (assert) {
    // -----------------------------------------------------
    // program
    // -----------------------------------------------------
    var prog = new dsl2_1.Program("test");
    prog.bind("simple block", function (_a) {
        var find = _a.find, record = _a.record, lib = _a.lib;
        var a = find("person");
        var b = find("person");
        a.age > b.age;
        var result = a.age / b.age;
        return [
            record({ age1: a.age, age2: b.age, result: result })
        ];
    });
    // -----------------------------------------------------
    // verification
    // -----------------------------------------------------
    util_1.verify(assert, prog, [
        [1, "tag", "person"],
        [1, "age", 7],
        [2, "tag", "person"],
        [2, "age", 35],
    ], [
        [4, "age1", 35, 1],
        [4, "age2", 7, 1],
        [4, "result", 5, 1],
    ]);
    assert.end();
});
test("static equality filters expressions", function (assert) {
    var prog = new dsl2_1.Program("Automatic Teacher's Assistant");
    prog.bind("Auto TA addition", function (_a) {
        var find = _a.find, record = _a.record, lib = _a.lib;
        var addition = find("addition");
        1 == addition.a + addition.b;
        return [record("success", { addition: addition })];
    });
    util_1.verify(assert, prog, [
        [1, "tag", "addition"],
        [1, "a", 7],
        [1, "b", 13],
        [2, "tag", "addition"],
        [2, "a", 3],
        [2, "b", -2],
    ], [
        ["A", "tag", "success", 1],
        ["A", "addition", 2, 1]
    ]);
    assert.end();
});
test("dynamic equality filters expressions", function (assert) {
    var prog = new dsl2_1.Program("Automatic Teacher's Assistant");
    prog.bind("Auto TA addition", function (_a) {
        var find = _a.find, record = _a.record, lib = _a.lib;
        var addition = find("addition");
        addition.c == addition.a + addition.b;
        return [record("success", { addition: addition })];
    });
    util_1.verify(assert, prog, [
        [1, "tag", "addition"],
        [1, "a", 7],
        [1, "b", 13],
        [1, "c", 1],
        [2, "tag", "addition"],
        [2, "a", 3],
        [2, "b", -2],
        [2, "c", 1],
    ], [
        ["Z", "tag", "success", 1],
        ["Z", "addition", 2, 1]
    ]);
    assert.end();
});
test("simple recursion", function (assert) {
    // -----------------------------------------------------
    // program
    // -----------------------------------------------------
    var prog = new dsl2_1.Program("test");
    prog.bind("simple block", function (_a) {
        var find = _a.find, record = _a.record, lib = _a.lib;
        var number = find().number;
        9 > number;
        var result = number + 1;
        return [
            record({ number: result })
        ];
    });
    // -----------------------------------------------------
    // verification
    // -----------------------------------------------------
    util_1.verify(assert, prog, [
        [1, "number", 1],
    ], [
        [2, "number", 2, 1],
        [3, "number", 3, 2],
        [4, "number", 4, 3],
        [5, "number", 5, 4],
        [6, "number", 6, 5],
        [7, "number", 7, 6],
        [8, "number", 8, 7],
        [9, "number", 9, 8],
    ]);
    assert.end();
});
test("test addition operator", function (assert) {
    // -----------------------------------------------------
    // program
    // -----------------------------------------------------
    var prog = new dsl2_1.Program("test");
    prog.bind("simple block", function (_a) {
        var find = _a.find, record = _a.record, lib = _a.lib;
        var joof = find({ foo: "bar" });
        return [
            joof.add("name", "JOOF")
        ];
    });
    // -----------------------------------------------------
    // verification
    // -----------------------------------------------------
    util_1.verify(assert, prog, [
        [1, "foo", "bar"]
    ], [
        [1, "name", "JOOF", 1]
    ]);
    assert.end();
});
test("transitive closure", function (assert) {
    // -----------------------------------------------------
    // program
    // -----------------------------------------------------
    var prog = new dsl2_1.Program("test");
    prog.bind("Every edge is the beginning of a path.", function (_a) {
        var find = _a.find, record = _a.record, lib = _a.lib;
        var from = find();
        return [
            from.add("path", from.edge)
        ];
    });
    prog.bind("Jump from node to node building the path.", function (_a) {
        var find = _a.find, record = _a.record, lib = _a.lib;
        var from = find();
        var intermediate = find();
        from.edge == intermediate;
        var to = intermediate.path;
        intermediate.path;
        return [
            from.add("path", to)
        ];
    });
    // -----------------------------------------------------
    // verification
    // -----------------------------------------------------
    util_1.verify(assert, prog, [
        [1, "edge", 2],
        [2, "edge", 1],
    ], [
        [1, "path", 2, 1],
        [2, "path", 1, 1],
        [1, "path", 1, 2],
        [2, "path", 2, 2],
    ]);
    util_1.verify(assert, prog, [
        [1, "edge", 2, 0, -1],
    ], [
        [1, "path", 2, 1, -1],
        [1, "path", 1, 2, -1],
        [2, "path", 2, 2, -1],
    ]);
    util_1.verify(assert, prog, [
        [1, "edge", 2, 0, 1],
    ], [
        [1, "path", 2, 1, 1],
        [1, "path", 1, 2, 1],
        [2, "path", 2, 2, 1],
    ]);
    // verify(assert, prog, [
    //   [1, "edge", 2],
    //   [2, "edge", 3],
    //   [3, "edge", 4],
    //   [4, "edge", 1],
    // ], [
    //   [1, "path", 2, 1],
    //   [2, "path", 3, 1],
    //   [3, "path", 4, 1],
    //   [4, "path", 1, 1],
    //   [1, "path", 3, 2],
    //   [2, "path", 4, 2],
    //   [3, "path", 1, 2],
    //   [4, "path", 2, 2],
    //   [1, "path", 4, 3],
    //   [2, "path", 1, 3],
    //   [3, "path", 2, 3],
    //   [4, "path", 3, 3],
    //   [1, "path", 1, 4],
    //   [2, "path", 2, 4],
    //   [3, "path", 3, 4],
    //   [4, "path", 4, 4],
    //   [1, "path", 2, 5],
    //   [2, "path", 3, 5],
    //   [3, "path", 4, 5],
    //   [4, "path", 1, 5]
    // ]);
    // // Kick the legs out from under the cycle.
    // verify(assert, prog, [
    //   [4, "edge", 1, 0, -1]
    // ], [
    //   [4, "path", 1, 1, -1],
    //   [4, "path", 2, 2, -1],
    //   [3, "path", 1, 2, -1],
    //   [2, "path", 1, 2, -1],
    //   [1, "path", 1, 2, -1],
    //   [4, "path", 3, 3, -1],
    //   [3, "path", 2, 3, -1],
    //   [2, "path", 2, 3, -1],
    //   [1, "path", 2, 3, -1],
    //   [4, "path", 4,  4, -1],
    //   [4, "path", 1,  5, -1],
    // ]);
    assert.end();
});
test("removal", function (assert) {
    // -----------------------------------------------------
    // program
    // -----------------------------------------------------
    var prog = new dsl2_1.Program("test");
    prog.bind("simple block", function (_a) {
        var find = _a.find, record = _a.record, lib = _a.lib;
        find({ foo: "bar" });
        return [
            record({ zomg: "baz" })
        ];
    });
    // -----------------------------------------------------
    // verification
    // -----------------------------------------------------
    // trust, but
    util_1.verify(assert, prog, [
        [1, "foo", "bar"]
    ], [
        [2, "zomg", "baz", 1]
    ]);
    util_1.verify(assert, prog, [
        [1, "foo", "bar", 0, -1]
    ], [
        [2, "zomg", "baz", 1, -1]
    ], 1);
    assert.end();
});
test.skip("not", function (assert) {
    // -----------------------------------------------------
    // program
    // -----------------------------------------------------
    var prog = new dsl2_1.Program("test");
    prog.bind("simple block", function (_a) {
        var find = _a.find, record = _a.record, lib = _a.lib, not = _a.not;
        var person = find({ tag: "person" });
        not(function () { return person.alive; });
        return [
            person.add("dead", "true")
        ];
    });
    // -----------------------------------------------------
    // verification
    // -----------------------------------------------------
    // trust, but
    util_1.verify(assert, prog, [
        [1, "tag", "person"]
    ], [
        [1, "dead", "true", 1]
    ]);
    assert.end();
});
test("Nested attribute lookup", function (assert) {
    // -----------------------------------------------------
    // program
    // -----------------------------------------------------
    var prog = new dsl2_1.Program("test");
    prog.bind("simple block", function (_a) {
        var find = _a.find, record = _a.record, lib = _a.lib;
        var jeff = find({ tag: "bar" });
        return [
            record({ zomg: jeff.dog.weight })
        ];
    });
    // -----------------------------------------------------
    // verification
    // -----------------------------------------------------
    util_1.verify(assert, prog, [
        [1, "tag", "bar"],
        [1, "dog", 2],
        [2, "weight", 13],
    ], [
        [3, "zomg", 13, 1]
    ]);
    assert.end();
});
test("Basic not", function (assert) {
    // -----------------------------------------------------
    // program
    // -----------------------------------------------------
    var prog = new dsl2_1.Program("test");
    prog.bind("simple block", function (_a) {
        var find = _a.find, record = _a.record, lib = _a.lib, not = _a.not;
        var person = find("person");
        not(function () {
            person.age;
        });
        return [
            person.add("tag", "old")
        ];
    });
    // -----------------------------------------------------
    // verification
    // -----------------------------------------------------
    util_1.verify(assert, prog, [
        [1, "tag", "person"],
    ], [
        [1, "tag", "old", 1]
    ]);
    util_1.verify(assert, prog, [
        [1, "age", 20],
    ], [
        [1, "tag", "old", 1, -1]
    ]);
    assert.end();
});
test("Basic aggregate", function (assert) {
    // -----------------------------------------------------
    // program
    // -----------------------------------------------------
    var prog = new dsl2_1.Program("test");
    prog.bind("simple block", function (_a) {
        var find = _a.find, record = _a.record, lib = _a.lib, gather = _a.gather;
        var person = find("person");
        var count = gather(person).count();
        return [
            record("info").add("total people", count)
        ];
    });
    // -----------------------------------------------------
    // verification
    // -----------------------------------------------------
    util_1.verify(assert, prog, [
        [1, "tag", "person"],
        [2, "tag", "person"],
    ], [
        [3, "tag", "info", 1],
        [3, "total people", 2, 1],
    ]);
    util_1.verify(assert, prog, [
        [1, "tag", "person", 0, -1],
    ], [
        [3, "total people", 2, 1, -1],
        [3, "total people", 1, 1],
    ]);
    util_1.verify(assert, prog, [
        [1, "tag", "person"],
        [4, "tag", "person"],
    ], [
        [3, "total people", 1, 1, -1],
        [3, "total people", 3, 1],
    ]);
    assert.end();
});
test("commit, remove, and recursion", function (assert) {
    // -----------------------------------------------------
    // program
    // -----------------------------------------------------
    var prog = new dsl2_1.Program("test");
    prog.commit("coolness", function (_a) {
        var find = _a.find, not = _a.not, record = _a.record, choose = _a.choose;
        var click = find("click", "direct-target");
        var count = find("count");
        var current = count.count;
        5 > current;
        return [
            count.add("count", current + 1)
        ];
    });
    prog.commit("foo", function (_a) {
        var find = _a.find;
        var click = find("click", "direct-target");
        return [
            click.remove("tag", "click"),
            click.remove("tag", "direct-target"),
        ];
    });
    // -----------------------------------------------------
    // verification
    // -----------------------------------------------------
    util_1.verify(assert, prog, [
        [1, "tag", "count"],
        [1, "count", 0]
    ], []);
    util_1.verify(assert, prog, [
        [2, "tag", "click"],
        [2, "tag", "direct-target"]
    ], [
        [2, "tag", "click", 0, -1],
        [2, "tag", "direct-target", 0, -1],
        [1, "count", 1, 0],
    ]);
    util_1.verify(assert, prog, [
        [3, "tag", "click"],
        [3, "tag", "direct-target"]
    ], [
        [3, "tag", "click", 0, -1],
        [3, "tag", "direct-target", 0, -1],
        [1, "count", 2, 0],
    ]);
    assert.end();
});
test("Remove: free AV", function (assert) {
    var prog = new dsl2_1.Program("test");
    prog.commit("coolness", function (_a) {
        var find = _a.find, not = _a.not, record = _a.record, choose = _a.choose;
        var person = find("person");
        return [
            person.remove()
        ];
    });
    util_1.verify(assert, prog, [
        [1, "tag", "person"],
        [1, "name", "chris"],
        [1, "age", 30],
    ], [
        [1, "tag", "person", 0, -1],
        [1, "name", "chris", 0, -1],
        [1, "age", 30, 0, -1],
    ]);
    assert.end();
});
test("Reference: arbitrary refs act like records", function (assert) {
    var prog = new dsl2_1.Program("test");
    prog.commit("coolness", function (_a) {
        var find = _a.find, not = _a.not, record = _a.record, union = _a.union;
        var person = find("person");
        var thing = union(function () {
            return find("person");
        }, function () {
            return "foo";
        })[0];
        return [
            thing.remove()
        ];
    });
    util_1.verify(assert, prog, [
        [1, "tag", "person"],
        [1, "name", "chris"],
        [1, "age", 30],
        ["foo", "tag", "person"],
        ["foo", "name", "chris"],
        ["foo", "age", 30],
    ], [
        [1, "tag", "person", 0, -1],
        [1, "name", "chris", 0, -1],
        [1, "age", 30, 0, -1],
        ["foo", "tag", "person", 0, -1],
        ["foo", "name", "chris", 0, -1],
        ["foo", "age", 30, 0, -1],
    ]);
    assert.end();
});
//# sourceMappingURL=foundation.js.map