"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var dsl2_1 = require("../src/runtime/dsl2");
var util_1 = require("./util");
var indexes_1 = require("../src/runtime/indexes");
var test = require("tape");
test("test single block performance with 10000 transactions", function (assert) {
    // -----------------------------------------------------
    // program
    // -----------------------------------------------------
    var prog = new dsl2_1.Program("test");
    prog.bind("simple block", function (_a) {
        var find = _a.find, record = _a.record, lib = _a.lib;
        var person = find("person");
        var text = "name: " + person.name;
        return [
            record("html/div", { person: person, text: text })
        ];
    });
    var _loop_1 = function (ix) {
        prog.index = new indexes_1.HashIndex();
        var size = 10000;
        var changes = [];
        for (var i = 0; i < size; i++) {
            changes.push(util_1.createChanges(i, [[i - 1, "name", i - 1], [i, "tag", "person"]]));
        }
        var start = util_1.time();
        for (var _i = 0, changes_1 = changes; _i < changes_1.length; _i++) {
            var change = changes_1[_i];
            prog.input(change);
        }
        var end = util_1.time(start);
        assert.test("updates finished in " + end, function (assert) {
            assert.true(end < 1000, "Took too long");
            assert.end();
        });
    };
    // -----------------------------------------------------
    // verification
    // -----------------------------------------------------
    for (var ix = 0; ix < 1; ix++) {
        _loop_1(ix);
    }
    assert.pass();
    assert.end();
});
//# sourceMappingURL=performance.js.map