"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var dsl2_1 = require("../../src/runtime/dsl2");
var util_1 = require("../util");
var test = require("tape");
test("stdlib:math:range: 1..5", function (assert) {
    var prog = new dsl2_1.Program("test");
    prog.bind("test range", function (_a) {
        var find = _a.find, math = _a.lib.math, record = _a.record;
        var ix = math.range(1, 5);
        return [
            record({ ix: ix })
        ];
    });
    util_1.verify(assert, prog, [
        ["A", "tag", "turtle"],
    ], [
        [11, "ix", 1, 1],
        [12, "ix", 2, 1],
        [13, "ix", 3, 1],
        [14, "ix", 4, 1],
        [15, "ix", 5, 1],
    ]);
    assert.end();
});
test("stdlib:math:range: 3..-1", function (assert) {
    var prog = new dsl2_1.Program("test");
    prog.bind("test range", function (_a) {
        var find = _a.find, math = _a.lib.math, record = _a.record;
        var ix = math.range(3, -1);
        return [
            record({ ix: ix })
        ];
    });
    util_1.verify(assert, prog, [
        ["A", "tag", "turtle"],
    ], [
        [11, "ix", 3, 1],
        [12, "ix", 2, 1],
        [13, "ix", 1, 1],
        [14, "ix", 0, 1],
        [15, "ix", -1, 1],
    ]);
    assert.end();
});
test("stdlib:math:range: x..y", function (assert) {
    var prog = new dsl2_1.Program("test");
    prog.bind("test range", function (_a) {
        var find = _a.find, math = _a.lib.math;
        var endpoints = find("endpoints");
        var x = endpoints.x, y = endpoints.y;
        var ix = math.range(x, y);
        return [
            endpoints.add({ ix: ix })
        ];
    });
    util_1.verify(assert, prog, [
        ["A", "tag", "endpoints"],
        ["A", "x", 1],
        ["A", "y", 3],
        ["B", "tag", "endpoints"],
        ["B", "x", 2],
        ["B", "y", 0],
    ], [
        ["A", "ix", 1, 1],
        ["A", "ix", 2, 1],
        ["A", "ix", 3, 1],
        ["B", "ix", 0, 1],
        ["B", "ix", 1, 1],
        ["B", "ix", 2, 1],
    ]);
    assert.end();
});
test("stdlib:math:range: bail on strings", function (assert) {
    var prog = new dsl2_1.Program("test");
    prog.bind("test range", function (_a) {
        var find = _a.find, math = _a.lib.math;
        var endpoints = find("endpoints");
        var x = endpoints.x, y = endpoints.y;
        var ix = math.range(x, y);
        return [
            endpoints.add({ ix: ix })
        ];
    });
    util_1.verify(assert, prog, [
        ["A", "tag", "endpoints"],
        ["A", "x", "1"],
        ["A", "y", 3],
        ["B", "tag", "endpoints"],
        ["B", "x", 2],
        ["B", "y", "0"],
    ], []);
    assert.end();
});
//# sourceMappingURL=math.js.map