"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var dsl2_1 = require("../src/runtime/dsl2");
var util_1 = require("./util");
var test = require("tape");
var programs = {
    "1 static": function () {
        var prog = new dsl2_1.Program("1 static branch");
        prog.bind("simple block", function (_a) {
            var find = _a.find, union = _a.union, record = _a.record;
            var foo = find("input");
            var branch = union(function () { return 1; })[0];
            return [
                record("result", { branch: branch })
            ];
        });
        return prog;
    },
    "2 static": function () {
        var prog = new dsl2_1.Program("2 static branches");
        prog.bind("simple block", function (_a) {
            var find = _a.find, union = _a.union, record = _a.record;
            var foo = find("input");
            var branch = union(function () { return 1; }, function () { return 2; })[0];
            return [
                record("result", { branch: branch })
            ];
        });
        return prog;
    },
    "1 dynamic": function () {
        var prog = new dsl2_1.Program("1 dynamic branch");
        prog.bind("simple block", function (_a) {
            var find = _a.find, union = _a.union, record = _a.record;
            var foo = find("input");
            var output = union(function () { return foo.arg0; })[0];
            return [
                record("result", { output: output })
            ];
        });
        return prog;
    },
    "2 dynamic": function () {
        var prog = new dsl2_1.Program("2 dynamic branches");
        prog.bind("simple block", function (_a) {
            var find = _a.find, union = _a.union, record = _a.record;
            var foo = find("input");
            var output = union(function () { foo.arg0 == 1; return ["one"]; }, function () { return foo.arg0; })[0];
            return [
                record("result", { output: output })
            ];
        });
        return prog;
    },
};
var verifyBranches = util_1.createVerifier(programs);
// -----------------------------------------------------
// 1 Static branch
// -----------------------------------------------------
test("Union: 1 static branch +A; -A; +A", function (assert) {
    verifyBranches(assert, "1 static", "+A; -A; +A", [
        [[2, "tag", "result", 1, +1], [2, "branch", 1, 1, +1]],
        [[2, "tag", "result", 1, -1], [2, "branch", 1, 1, -1]],
        [[2, "tag", "result", 1, +1], [2, "branch", 1, 1, +1]]
    ]);
});
test("Union: 1 static branch +A; -A; +B", function (assert) {
    verifyBranches(assert, "1 static", "+A; -A; +B", [
        [[2, "tag", "result", 1, +1], [2, "branch", 1, 1, +1]],
        [[2, "tag", "result", 1, -1], [2, "branch", 1, 1, -1]],
        [[2, "tag", "result", 1, +1], [2, "branch", 1, 1, +1]]
    ]);
});
// @NOTE: Broken due to verify being too simple.
test("Union: 1 static branch +A; +A; -A", function (assert) {
    verifyBranches(assert, "1 static", "+A; +A; -A", [
        [[2, "tag", "result", 1, +1], [2, "branch", 1, 1, +1]],
        [],
        []
    ]);
});
// @NOTE: Broken due to verify being too simple.
test("Union: 1 static branch +A; +A; -A; -A", function (assert) {
    verifyBranches(assert, "1 static", "+A; +A; -A; -A", [
        [[2, "tag", "result", 1, +1], [2, "branch", 1, 1, +1]],
        [],
        [],
        [[2, "tag", "result", 1, -1], [2, "branch", 1, 1, -1]],
    ]);
});
test("Union: 1 static branch +A +B; -A; +A", function (assert) {
    verifyBranches(assert, "1 static", "+A +B; -A; +A", [
        [[2, "tag", "result", 1, +1], [2, "branch", 1, 1, +1]],
        [],
        []
    ]);
});
test("Union: 1 static branch +A +B; -A -B", function (assert) {
    verifyBranches(assert, "1 static", "+A +B; -A -B", [
        [[2, "tag", "result", 1, +1], [2, "branch", 1, 1, +1]],
        [[2, "tag", "result", 1, -1], [2, "branch", 1, 1, -1]]
    ]);
});
test("Union: 1 static branch +A; -A +B", function (assert) {
    verifyBranches(assert, "1 static", "+A; -A +B", [
        [[2, "tag", "result", 1, +1], [2, "branch", 1, 1, +1]],
        []
    ]);
});
test("Union: 1 static branch +A, -A", function (assert) {
    verifyBranches(assert, "1 static", "+A, -A", [
        [[2, "tag", "result", 1, +1], [2, "branch", 1, 1, +1],
            [2, "tag", "result", 2, -1], [2, "branch", 1, 2, -1]]
    ]);
});
test("Union: 1 static branch +A, -A +B", function (assert) {
    verifyBranches(assert, "1 static", "+A, -A +B", [
        [[2, "tag", "result", 1, +1], [2, "branch", 1, 1, +1]]
    ]);
});
test("Union: 1 static branch +A, +B", function (assert) {
    verifyBranches(assert, "1 static", "+A, +B", [
        [[2, "tag", "result", 1, +1], [2, "branch", 1, 1, +1]]
    ]);
});
test("Union: 1 static branch +A, +B; -A", function (assert) {
    verifyBranches(assert, "1 static", "+A, +B; -A", [
        [[2, "tag", "result", 1, +1], [2, "branch", 1, 1, +1]],
        [[2, "tag", "result", 1, -1], [2, "branch", 1, 1, -1],
            [2, "tag", "result", 2, +1], [2, "branch", 1, 2, +1]]
    ]);
});
test("Union: 1 static branch +A; -A, +B", function (assert) {
    verifyBranches(assert, "1 static", "+A; -A, +B", [
        [[2, "tag", "result", 1, +1], [2, "branch", 1, 1, +1]],
        [[2, "tag", "result", 1, -1], [2, "branch", 1, 1, -1],
            [2, "tag", "result", 2, +1], [2, "branch", 1, 2, +1]]
    ]);
});
// -----------------------------------------------------
// 2 Static branches
// -----------------------------------------------------
test("Union: 2 static branches +A; -A; +A", function (assert) {
    verifyBranches(assert, "2 static", "+A; -A; +A", [
        [[1, "tag", "result", 1, +1], [1, "branch", 1, 1, +1],
            [2, "tag", "result", 1, +1], [2, "branch", 2, 1, +1]],
        [[1, "tag", "result", 1, -1], [1, "branch", 1, 1, -1],
            [2, "tag", "result", 1, -1], [2, "branch", 2, 1, -1]],
        [[1, "tag", "result", 1, +1], [1, "branch", 1, 1, +1],
            [2, "tag", "result", 1, +1], [2, "branch", 2, 1, +1]]
    ]);
});
test("Union: 2 static branches +A; +B", function (assert) {
    verifyBranches(assert, "2 static", "+A; +B", [
        [[1, "tag", "result", 1, +1], [1, "branch", 1, 1, +1],
            [2, "tag", "result", 1, +1], [2, "branch", 2, 1, +1]],
        []
    ]);
});
test("Union: 2 static branches +A; +B; -A", function (assert) {
    verifyBranches(assert, "2 static", "+A; +B; -A", [
        [[1, "tag", "result", 1, +1], [1, "branch", 1, 1, +1],
            [2, "tag", "result", 1, +1], [2, "branch", 2, 1, +1]],
        [],
        []
    ]);
});
test("Union: 2 static branches +A +B; -A; +A", function (assert) {
    verifyBranches(assert, "2 static", "+A +B; -A; +A", [
        [[1, "tag", "result", 1, +1], [1, "branch", 1, 1, +1],
            [2, "tag", "result", 1, +1], [2, "branch", 2, 1, +1]],
        [],
        []
    ]);
});
test("Union: 2 static branches +A; +B; -A; -B", function (assert) {
    verifyBranches(assert, "2 static", "+A; +B; -A; -B", [
        [[1, "tag", "result", 1, +1], [1, "branch", 1, 1, +1],
            [2, "tag", "result", 1, +1], [2, "branch", 2, 1, +1]],
        [],
        [],
        [[1, "tag", "result", 1, -1], [1, "branch", 1, 1, -1],
            [2, "tag", "result", 1, -1], [2, "branch", 2, 1, -1]],
    ]);
});
test("Union: 2 static branches +A; +B, -A; -B", function (assert) {
    verifyBranches(assert, "2 static", "+A; +B, -A; -B", [
        [[1, "tag", "result", 1, +1], [1, "branch", 1, 1, +1],
            [2, "tag", "result", 1, +1], [2, "branch", 2, 1, +1]],
        [],
        // These changes happen in round 2 because while the removal of B's results
        // happen in round 1, A's results exist in round 1 and then are removed
        // in round 2 as a result of A beind removed in round 1.
        [[1, "tag", "result", 2, -1], [1, "branch", 1, 2, -1],
            [2, "tag", "result", 2, -1], [2, "branch", 2, 2, -1]],
    ]);
});
// -----------------------------------------------------
// 1 dynamic branch
// -----------------------------------------------------
test("Union: 1 dynamic branch +A:1; -A:1", function (assert) {
    verifyBranches(assert, "1 dynamic", "+A:1; -A:1", [
        [[1, "tag", "result", 1, +1], [1, "output", 1, 1, +1]],
        [[1, "tag", "result", 1, -1], [1, "output", 1, 1, -1]],
    ]);
});
test("Union: 1 dynamic branch +A:1; -A:1; +A:1", function (assert) {
    verifyBranches(assert, "1 dynamic", "+A:1; -A:1; +A:1", [
        [[1, "tag", "result", 1, +1], [1, "output", 1, 1, +1]],
        [[1, "tag", "result", 1, -1], [1, "output", 1, 1, -1]],
        [[1, "tag", "result", 1, +1], [1, "output", 1, 1, +1]],
    ]);
});
test("Union: 1 dynamic branch +A:1; -A:1; +A:2", function (assert) {
    verifyBranches(assert, "1 dynamic", "+A:1; -A:1; +A:2", [
        [[1, "tag", "result", 1, +1], [1, "output", 1, 1, +1]],
        [[1, "tag", "result", 1, -1], [1, "output", 1, 1, -1]],
        [[1, "tag", "result", 1, +1], [1, "output", 2, 1, +1]],
    ]);
});
test("Union: 1 dynamic branch +A:1; +A:2; -A:1", function (assert) {
    verifyBranches(assert, "1 dynamic", "+A:1; +A:2; -A:1", [
        [[1, "tag", "result", 1, +1], [1, "output", 1, 1, +1]],
        [[1, "tag", "result", 1, +1], [1, "output", 2, 1, +1]],
        [[1, "tag", "result", 1, -1], [1, "output", 1, 1, -1]],
    ]);
});
test("Union: 1 dynamic branch +A:1; +B:1; -A:1", function (assert) {
    verifyBranches(assert, "1 dynamic", "+A:1; +B:1; -A:1", [
        [[1, "tag", "result", 1, +1], [1, "output", 1, 1, +1]],
        [],
        [],
    ]);
});
test("Union: 1 dynamic branch +A:1; +B:1; -A:1, -B:1", function (assert) {
    verifyBranches(assert, "1 dynamic", "+A:1; +B:1; -A:1, -B:1", [
        [[1, "tag", "result", 1, +1], [1, "output", 1, 1, +1]],
        [],
        [[1, "tag", "result", 2, -1], [1, "output", 1, 2, -1]],
    ]);
});
// -----------------------------------------------------
// 2 dynamic branches
// -----------------------------------------------------
test("Union: 2 dynamic branches +A:1; -A:1", function (assert) {
    verifyBranches(assert, "2 dynamic", "+A:1; -A:1", [
        [[1, "tag", "result", 1, +1], [1, "output", "one", 1, +1],
            [2, "tag", "result", 1, +1], [2, "output", 1, 1, +1]],
        [[1, "tag", "result", 1, -1], [1, "output", "one", 1, -1],
            [2, "tag", "result", 1, -1], [2, "output", 1, 1, -1]]
    ]);
});
test("Union: 2 dynamic branches +A:1; +B:1", function (assert) {
    verifyBranches(assert, "2 dynamic", "+A:1; +B:1", [
        [[1, "tag", "result", 1, +1], [1, "output", "one", 1, +1],
            [2, "tag", "result", 1, +1], [2, "output", 1, 1, +1]],
        []
    ]);
});
test("Union: 2 dynamic branches +A:1, +B:1", function (assert) {
    verifyBranches(assert, "2 dynamic", "+A:1, +B:1", [
        [[1, "tag", "result", 1, +1], [1, "output", "one", 1, +1],
            [2, "tag", "result", 1, +1], [2, "output", 1, 1, +1]]
    ]);
});
test("Union: 2 dynamic branches +A:1, +B:1; -A:1", function (assert) {
    verifyBranches(assert, "2 dynamic", "+A:1, +B:1; -A:1", [
        [[1, "tag", "result", 1, +1], [1, "output", "one", 1, +1],
            [2, "tag", "result", 1, +1], [2, "output", 1, 1, +1]],
        [[1, "tag", "result", 1, -1], [1, "output", "one", 1, -1],
            [2, "tag", "result", 1, -1], [2, "output", 1, 1, -1],
            [1, "tag", "result", 2, +1], [1, "output", "one", 2, +1],
            [2, "tag", "result", 2, +1], [2, "output", 1, 2, +1]]
    ]);
});
test("Union: 2 dynamic branches +A:1 +B:2, -A:1", function (assert) {
    verifyBranches(assert, "2 dynamic", "+A:1 +B:2, -A:1", [
        [[1, "tag", "result", 1, +1], [1, "output", "one", 1, +1],
            [2, "tag", "result", 1, +1], [2, "output", 1, 1, +1],
            [3, "tag", "result", 1, +1], [3, "output", 2, 1, +1],
            [1, "tag", "result", 2, -1], [1, "output", "one", 2, -1],
            [2, "tag", "result", 2, -1], [2, "output", 1, 2, -1]]
    ]);
});
test("Union: basic", function (assert) {
    var prog = new dsl2_1.Program("test");
    prog.bind("simple block", function (_a) {
        var find = _a.find, record = _a.record, lib = _a.lib, union = _a.union;
        var person = find("person");
        var info = union(function () {
            person.dog;
            return "cool";
        }, function () {
            return "not cool";
        })[0];
        return [
            record("coolness", { info: info })
        ];
    });
    util_1.verify(assert, prog, [
        [1, "tag", "person"],
    ], [
        [2, "tag", "coolness", 1],
        [2, "info", "not cool", 1],
    ]);
    util_1.verify(assert, prog, [
        [1, "dog", "spot"],
    ], [
        [3, "tag", "coolness", 1],
        [3, "info", "cool", 1],
    ]);
    assert.end();
});
test("Union: static moves", function (assert) {
    var prog = new dsl2_1.Program("test");
    prog.bind("simple block", function (_a) {
        var find = _a.find, record = _a.record, lib = _a.lib, union = _a.union;
        var person = find("person");
        var info = union(function () {
            return "cool";
        }, function () {
            return "not cool";
        })[0];
        return [
            record("coolness", { info: info })
        ];
    });
    util_1.verify(assert, prog, [
        [1, "tag", "person"],
    ], [
        [2, "tag", "coolness", 1],
        [2, "info", "not cool", 1],
        [3, "tag", "coolness", 1],
        [3, "info", "cool", 1],
    ]);
    assert.end();
});
//# sourceMappingURL=union.js.map