"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Runtime = require("../src/runtime/runtime");
var GlobalInterner = Runtime.GlobalInterner;
var TEST_INPUT_NODE = "test-input-node";
function pprint(obj) {
    if (typeof obj === "object" && obj instanceof Array) {
        return "[" + obj.map(function (v) { return pprint(v); }).join(", ") + "]";
    }
    else if (typeof obj === "string") {
        return "\"" + obj + "\"";
    }
    return "" + obj;
}
exports.pprint = pprint;
var EntityId = (function () {
    function EntityId(id) {
        this.id = id;
    }
    EntityId.prototype.toString = function () {
        return "$" + this.id;
    };
    return EntityId;
}());
exports.EntityId = EntityId;
function o_o(val) {
    var raw = GlobalInterner.reverse(val);
    if (typeof raw === "string" && raw.indexOf("|") !== -1) {
        return new EntityId(val);
    }
    return raw;
}
exports.o_o = o_o;
function createChanges(transaction, eavns) {
    var changes = [];
    for (var _i = 0, _a = eavns; _i < _a.length; _i++) {
        var _b = _a[_i], e = _b[0], a = _b[1], v = _b[2], _c = _b[3], round = _c === void 0 ? 0 : _c, _d = _b[4], count = _d === void 0 ? 1 : _d;
        changes.push(Runtime.Change.fromValues(e, a, v, TEST_INPUT_NODE, transaction, round, count));
    }
    return changes;
}
exports.createChanges = createChanges;
function verify(assert, program, input, output, transaction) {
    if (transaction === void 0) { transaction = 1; }
    var ins = createChanges(transaction, input);
    var outs = createChanges(transaction, output);
    var all = outs;
    var _a = program.input(ins), changes = _a.changes, context = _a.context;
    var inputNode = GlobalInterner.get(TEST_INPUT_NODE);
    changes = changes.filter(function (v) { return v.n !== inputNode; });
    var msg = "Fewer changes than expected";
    if (changes.length > all.length) {
        msg = "More changes than expected";
    }
    if (changes.length !== all.length) {
        assert.comment(".    Actual: " + pprint(changes.map(function (change) {
            var e = change.e, a = change.a, v = change.v, round = change.round, count = change.count;
            return [o_o(e), o_o(a), o_o(v), round, count];
        })));
    }
    assert.equal(changes.length, all.length, msg);
    try {
        context.distinctIndex.sanityCheck();
    }
    catch (e) {
        assert.fail("Distinct sanity check failed");
    }
    // Because the changes handed to us in expected aren't going to have the same
    // e's as what the program itself is going to generate, we're going to have to do
    // some fancy matching to map from generated e's to expected e's. We'll need to
    // store that mapping somewhere, so we have eMap:
    var eMap = {};
    var fullyResolved = {};
    if (changes.length === all.length) {
        // As we check all of the changes we got from running the input on the program,
        // we need to update our eMap based on the expected changes that *could* match.
        // Most of the time, the hope is that there's only one potential match, but when
        // you're looking at something like tag, it's easy for there to be many records
        // that get generated with the same tag, so we're going to do a decent amount of
        // work here.
        for (var _i = 0, changes_1 = changes; _i < changes_1.length; _i++) {
            var actual = changes_1[_i];
            var found = false;
            // console.log("\n\nACTUAL");
            // console.log("    ", actual.toString());
            // console.log("    ", actual);
            var expectedIx = 0;
            // check if we've found any potential matches for this e yet
            var matches = eMap[actual.e];
            // if we haven't found any matches yet, we need to collect some initial ones.
            if (!matches) {
                var potentials = [];
                for (var _b = 0, all_1 = all; _b < all_1.length; _b++) {
                    var expected = all_1[_b];
                    if (!expected) {
                        expectedIx++;
                        continue;
                    }
                    // if this expected *could* match ignoring e and n, then we'll store this as
                    // a potential mapping from the actual.e to the expected.e. We're also going
                    // to store this expected's index so that once we know for sure that this
                    // actual.e === expected.e we can clean out the expecteds that no one can claim
                    // anymore.
                    if (actual.equal(expected, true /*ignore the node*/, true /*ignore the e*/)) {
                        found = true;
                        potentials.push({ e: expected.e, relatedChanges: [expectedIx] });
                    }
                    expectedIx++;
                }
                // if there was only one match, no one can ever have this expected - we've claimed it.
                // As such, we need to remove it from the list;
                if (potentials.length === 1) {
                    // We need to check that we haven't already resolved this match to some other actual value.
                    var e = potentials[0].e;
                    if (fullyResolved[e] !== undefined) {
                        assert.fail("`" + GlobalInterner.reverse(e) + "` has already been resolved to `" + GlobalInterner.reverse(fullyResolved[e]) + "`," +
                            (" but we are trying to resolve it to `" + GlobalInterner.reverse(actual.e) + "`"));
                        break;
                    }
                    fullyResolved[e] = actual.e;
                    for (var _c = 0, _d = potentials[0].relatedChanges; _c < _d.length; _c++) {
                        var ix = _d[_c];
                        all[ix] = undefined;
                    }
                }
                eMap[actual.e] = potentials;
            }
            else if (matches.length === 1) {
                // in the case where we've mapped our actual.e to our expected.e, we just check this
                // current fact for a match where the expected.e is what we're looking for
                for (var _e = 0, all_2 = all; _e < all_2.length; _e++) {
                    var expected = all_2[_e];
                    if (!expected) {
                        expectedIx++;
                        continue;
                    }
                    if (expected.e === matches[0].e && actual.equal(expected, true, true)) {
                        found = true;
                        all[expectedIx] = undefined;
                    }
                    expectedIx++;
                }
            }
            else {
                // since we have multiple potential matches, we need to see if this actual might reduce
                // the set down for us. For each expected that's left, we'll check if expected.e matches
                // one of our potentials, and if this expected would equal our actual if we ignored the
                // e. If so, we keep this potential in the running. Any potentials the don't end up with
                // a match get removed on account of us recreating the potential array from scratch here.
                var potentials = [];
                for (var _f = 0, all_3 = all; _f < all_3.length; _f++) {
                    var expected = all_3[_f];
                    if (!expected) {
                        expectedIx++;
                        continue;
                    }
                    for (var _g = 0, matches_1 = matches; _g < matches_1.length; _g++) {
                        var match = matches_1[_g];
                        if (match.e === expected.e && !fullyResolved[match.e] && actual.equal(expected, true, true)) {
                            found = true;
                            potentials.push(match);
                            match.relatedChanges.push(expectedIx);
                        }
                    }
                    expectedIx++;
                }
                // If we only have one potential, we need to clean up after ourselves again. This time
                // however, we could have had many relatedChanges that we need to clean up, so we'll loop
                // through them and remove them from the list. They're our's now.
                if (potentials.length === 1) {
                    // We need to check that we haven't already resolved this match to some other actual value.
                    var e = potentials[0].e;
                    if (fullyResolved[e] !== undefined) {
                        assert.fail("`" + GlobalInterner.reverse(e) + "` has already been resolved to `" + GlobalInterner.reverse(fullyResolved[e]) + "`," +
                            (" but we are trying to resolve it to `" + GlobalInterner.reverse(actual.e) + "`"));
                        break;
                    }
                    fullyResolved[e] = actual.e;
                    var related = potentials[0].relatedChanges;
                    related.sort(function (a, b) { return b - a; });
                    for (var _h = 0, related_1 = related; _h < related_1.length; _h++) {
                        var relatedIx = related_1[_h];
                        all[relatedIx] = undefined;
                    }
                    potentials[0].relatedChanges = [];
                }
                eMap[actual.e] = potentials;
            }
            // console.log("    ", actual.e, ":", eMap[actual.e]);
            // console.log("    [")
            // for(let thing of all) {
            //   console.log("        ", thing);
            // }
            // console.log("    ]")
            if (!found)
                assert.fail("No match found for: " + actual.toString());
            else
                assert.pass("Found match for: " + actual.toString());
        }
    }
}
exports.verify = verify;
function time(start) {
    if (!start)
        return process.hrtime();
    var end = process.hrtime(start);
    return ((end[0] * 1000) + (end[1] / 1000000)).toFixed(3);
}
exports.time = time;
function createInputs(inputString) {
    var transactionInputs = [];
    var transactions = inputString.split(";");
    for (var _i = 0, transactions_1 = transactions; _i < transactions_1.length; _i++) {
        var transaction = transactions_1[_i];
        var eavrcs = [];
        var roundNumber = 0;
        for (var _a = 0, _b = transaction.split(","); _a < _b.length; _a++) {
            var round = _b[_a];
            for (var _c = 0, _d = round.split(" "); _c < _d.length; _c++) {
                var input = _d[_c];
                if (!input)
                    continue;
                var count = void 0;
                if (input[0] === "+")
                    count = 1;
                else if (input[0] === "-")
                    count = -1;
                else
                    throw new Error("Malformed input: " + input);
                var args = input.slice(1).split(":");
                var id = args.shift();
                if (!id)
                    throw new Error("Malformed input: '" + input + "'");
                eavrcs.push([id, "tag", "input", roundNumber, count]);
                var argIx = 0;
                for (var _e = 0, args_1 = args; _e < args_1.length; _e++) {
                    var arg = args_1[_e];
                    eavrcs.push([id, "arg" + argIx, (isNaN(arg) ? arg : +arg), roundNumber, count]);
                    argIx++;
                }
            }
            roundNumber += 1;
        }
        transactionInputs.push(eavrcs);
    }
    return transactionInputs;
}
exports.createInputs = createInputs;
function createVerifier(programs) {
    return function verifyInput(assert, progName, inputString, expecteds) {
        var prog = programs[progName]();
        var inputs = createInputs(inputString);
        if (expecteds.length !== inputs.length) {
            assert.fail("Malformed test case");
            throw new Error("Incorrect number of expecteds given the inputString Got " + expecteds.length + ", needed: " + inputs.length);
        }
        var transactionNumber = 0;
        for (var _i = 0, inputs_1 = inputs; _i < inputs_1.length; _i++) {
            var input = inputs_1[_i];
            var expected = expecteds[transactionNumber];
            assert.comment(".  Verifying: " + pprint(input) + " -> " + pprint(expected));
            verify(assert, prog, input, expected);
            transactionNumber++;
        }
        assert.end();
        return prog;
    };
}
exports.createVerifier = createVerifier;
//# sourceMappingURL=util.js.map