"use strict";
//@FIXME: This doesn't currently handle chooses/unions that rely on eachother.
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
//--------------------------------------------------------------------
// Javascript DSL for writing Eve programs
//--------------------------------------------------------------------
var runtime_1 = require("./runtime");
var Runtime = require("./runtime");
var indexes = require("./indexes");
var watcher_1 = require("../watchers/watcher");
var Parser = require("../parser/parser");
require("./stdlib");
var stdlib_1 = require("./stdlib");
var uuid_1 = require("uuid");
var falafel = require("falafel");
var UNASSIGNED = -1;
var operators = {
    "+": "math['+']",
    "-": "math['-']",
    "*": "math['*']",
    "/": "math['/']",
    ">": "compare['>']",
    ">=": "compare['>=']",
    "<": "compare['<']",
    "<=": "compare['<=']",
    "!=": "compare['!=']",
    "==": "compare['==']",
    "concat": "eve.internal.concat",
};
function isArray(v) {
    return v && v.constructor === Array;
}
function isASTString(thing) {
    return thing.value && typeof thing.value === "string";
}
function macro(func, transform) {
    var code = func.toString();
    // trim the function(...) { from the start and capture the arg names
    var name = "";
    code = code.replace(/function\s*(\w*)\s*/, function (str, funcName) {
        name = funcName;
        return "";
    });
    var functionArgs = [];
    code = code.replace(/\((.*)\)\s*\{/m, function (str, args) {
        functionArgs.push.apply(functionArgs, args.split(",").map(function (str) { return str.trim(); }));
        return "";
    });
    // trim the final } since we removed the function bit
    code = code.substring(0, code.length - 1);
    code = transform(code, functionArgs, name);
    var neueFunc = (new Function("\n    return function " + name + "(" + functionArgs.join(", ") + ") {\n      " + code + "\n    };\n  "))();
    return neueFunc;
}
function isRawValue(v) {
    return (typeof v === "string" || typeof v === "number");
}
function isReference(v) {
    return (v instanceof Reference);
}
var Reference = (function () {
    function Reference(__context, __owner) {
        this.__context = __context;
        this.__owner = __owner;
        this.__ID = Reference.ReferenceID++;
        this.__forceRegister = false;
        var proxied = this.__proxy();
        __context.register(proxied);
        this.__owner = __owner || null;
        return proxied;
    }
    Reference.create = function (context, value) {
        if (typeof value !== "object") {
            var neue = new Reference(context);
            if (value !== undefined)
                context.equality(neue, value);
            return neue;
        }
        return new Reference(context, value);
    };
    Reference.prototype.add = function (attrMapOrAttr, value) {
        if (!this.__owner) {
            this.__owner = new Record(this.__context, [], {}, this);
        }
        if (this.__owner instanceof Record) {
            // we only allow you to call add at the root context
            if (this.__context.parent)
                throw new Error("Add can't be called in a sub-block");
            if (isRawValue(attrMapOrAttr) || isReference(attrMapOrAttr)) {
                var attribute = attrMapOrAttr;
                if (value === undefined)
                    throw new Error("Can't call add without a value.");
                this.__owner.add(this.__context, attribute, value);
            }
            else {
                for (var _i = 0, _a = Object.keys(attrMapOrAttr); _i < _a.length; _i++) {
                    var attribute = _a[_i];
                    var value_1 = attrMapOrAttr[attribute];
                    this.__owner.add(this.__context, attribute, value_1);
                }
            }
            return this;
        }
        else {
            throw new Error("Can't call add on a non-record");
        }
    };
    // @TODO: allow free A's and V's here
    Reference.prototype.remove = function (attribute, value) {
        if (!this.__owner) {
            this.__owner = new Record(this.__context, [], {}, this);
        }
        if (this.__owner instanceof Record) {
            // we only allow you to call remove at the root context
            if (this.__context.parent)
                throw new Error("Add can't be called in a sub-block");
            this.__owner.remove(this.__context, attribute, value);
            return this;
        }
        else {
            throw new Error("Can't call add on a non-record");
        }
    };
    Reference.prototype.toString = function () {
        return this.__context.ID + "." + this.__ID;
    };
    Reference.prototype.__proxy = function () {
        var _this = this;
        return new Proxy(this, {
            get: function (obj, prop) {
                if (obj[prop] !== undefined)
                    return obj[prop];
                if (typeof prop === "symbol")
                    return function () {
                        return "uh oh";
                    };
                var active = _this.__context.getActive();
                if (!active) {
                    return;
                }
                if (!_this.__owner) {
                    _this.__owner = new Record(active, [], {}, _this);
                }
                return _this.__owner.access(_this.__context, active, prop);
            },
            set: function (obj, prop, value) {
                if (obj[prop] !== undefined) {
                    obj[prop] = value;
                    return true;
                }
                throw new Error("Cannot set a value on a reference");
            }
        });
    };
    Reference.ReferenceID = 0;
    return Reference;
}());
exports.Reference = Reference;
//--------------------------------------------------------------------
// ReferenceContext
//--------------------------------------------------------------------
var ReferenceContext = (function () {
    function ReferenceContext(parent, flow) {
        this.parent = parent;
        this.references = [];
        this.equalities = [];
        this.forcedMoves = [];
        this.referenceValues = [];
        this.totalRegisters = 0;
        this.maxRegisters = 0;
        this.ID = ReferenceContext.IDs++;
        this.flow = flow || new LinearFlow(function (x) { return []; });
    }
    ReferenceContext.push = function (context) {
        ReferenceContext.stack.push(context);
    };
    ReferenceContext.pop = function () {
        ReferenceContext.stack.pop();
    };
    ReferenceContext.prototype.register = function (ref) {
        // if this reference is not owned by this context, we have to walk up the context
        // stack and register this for our parents until we find a layer that *does* own it
        // so that it can be considered an input to that context.
        var parent = this.parent;
        if (!this.owns(ref) && parent) {
            while (parent && parent !== ref.__context) {
                parent.register(ref);
                parent = parent.parent;
            }
        }
        if (!this.owns(ref) && parent !== ref.__context) {
            console.error("Reference with no owner in the parent stack: ", ref);
            throw new Error("Reference with no owner in the parent stack");
        }
        if (!this.references[ref.__ID])
            this.references[ref.__ID] = ref;
    };
    ReferenceContext.prototype.equality = function (a, b) {
        if (a instanceof Reference) {
            this.register(a);
        }
        if (b instanceof Reference) {
            this.register(b);
        }
        this.equalities.push([a, b]);
    };
    ReferenceContext.prototype.getActive = function () {
        return ReferenceContext.stack[ReferenceContext.stack.length - 1];
    };
    ReferenceContext.prototype.owns = function (ref) {
        return ref.__context === this;
    };
    ReferenceContext.prototype.getValue = function (ref, orGenerateRegister) {
        if (isRawValue(ref))
            return ref;
        var val = this.referenceValues[ref.__ID];
        if (val === undefined) {
            if (!this.owns(ref) && this.parent)
                return this.parent.getValue(ref);
            if (orGenerateRegister) {
                val = new runtime_1.Register(UNASSIGNED);
                this.referenceValues[ref.__ID] = val;
            }
        }
        if (val === undefined)
            throw new Error("Unable to resolve reference: " + ref.__ID);
        return val;
    };
    ReferenceContext.prototype.interned = function (ref) {
        var value = this.getValue(ref);
        if (isRawValue(value))
            return runtime_1.GlobalInterner.intern(value);
        return value;
    };
    ReferenceContext.prototype.maybeInterned = function (ref) {
        if (ref === undefined)
            return;
        var value = this.getValue(ref);
        if (isRawValue(value))
            return runtime_1.GlobalInterner.intern(value);
        return value;
    };
    ReferenceContext.prototype.selectReference = function (refIds, ref, ref2) {
        var refID = refIds[ref.__ID] || ref.__ID;
        var ref2ID = refIds[ref2.__ID] || ref2.__ID;
        if (!this.owns(ref) && !this.owns(ref2)) {
            if (ref2ID < refID)
                return ref2;
            return ref;
        }
        if (!this.owns(ref))
            return ref;
        if (!this.owns(ref2))
            return ref2;
        if (ref2ID < refID)
            return ref2;
        return ref;
    };
    ReferenceContext.prototype.unify = function () {
        var equalities = this.equalities;
        var values = this.referenceValues;
        var forcedMoves = [];
        var changed = equalities.length > 0;
        var refIds = {};
        var round = 0;
        var maxRound = Math.pow(this.equalities.length + 1, 2);
        for (var _i = 0, _a = this.references; _i < _a.length; _i++) {
            var ref = _a[_i];
            if (!ref)
                continue;
            this.getValue(ref, true);
        }
        while (changed && round < maxRound) {
            round++;
            changed = false;
            for (var _b = 0, equalities_1 = equalities; _b < equalities_1.length; _b++) {
                var _c = equalities_1[_b], a = _c[0], b = _c[1];
                var aValue = isReference(a) ? this.getValue(a, true) : a;
                var bValue = isReference(b) ? this.getValue(b, true) : b;
                var neueA = aValue;
                var neueB = bValue;
                if (a.__forceRegister || b.__forceRegister) {
                    forcedMoves.push([a, b]);
                }
                else if (!runtime_1.isRegister(neueB) && !runtime_1.isRegister(neueA) && neueA != neueB) {
                    throw new Error("Attempting to unify two disparate static values: `" + neueA + "` and `" + neueB + "`");
                }
                else if (isReference(a) && !runtime_1.isRegister(neueB)) {
                    neueA = bValue;
                }
                else if (isReference(b) && !runtime_1.isRegister(neueA)) {
                    neueB = aValue;
                }
                else if (isReference(a) && isReference(b)) {
                    if (this.selectReference(refIds, a, b) === b) {
                        neueA = bValue;
                        refIds[a.__ID] = refIds[b.__ID] || b.__ID;
                    }
                    else {
                        neueB = aValue;
                        refIds[b.__ID] = refIds[a.__ID] || a.__ID;
                    }
                }
                else if (isReference(a)) {
                    neueA = bValue;
                }
                else if (isReference(b)) {
                    neueB = aValue;
                }
                else if (a !== b) {
                    throw new Error("Attempting to unify two disparate static values: `" + a + "` and `" + b + "`");
                }
                if (aValue !== neueA) {
                    // console.log("Unifying A", a.toString(), b.toString(), neueA, neueB);
                    values[a.__ID] = neueA;
                    changed = true;
                }
                if (bValue !== neueB) {
                    // console.log("Unifying B", a.toString(), b.toString(), neueA, neueB);
                    values[b.__ID] = neueB;
                    changed = true;
                }
            }
        }
        if (round >= maxRound) {
            throw new Error("Unable to unify variables. This is almost certainly an implementation error.");
        }
        this.assignRegisters();
        this.forcedMoves = forcedMoves;
    };
    ReferenceContext.prototype.getMoves = function () {
        var moves = [];
        for (var _i = 0, _a = this.forcedMoves; _i < _a.length; _i++) {
            var _b = _a[_i], a = _b[0], b = _b[1];
            var to = isReference(a) ? a : b;
            var from = to === a ? b : a;
            moves.push(new Move(this, from, to));
        }
        for (var _c = 0, _d = this.references; _c < _d.length; _c++) {
            var ref = _d[_c];
            if (ref === undefined || this.owns(ref))
                continue;
            var local = this.getValue(ref);
            var parent_1 = ref.__context.getValue(ref);
            if (local !== parent_1) {
                moves.push(new Move(this, ref));
            }
        }
        return moves;
    };
    ReferenceContext.prototype.getInputReferences = function () {
        var refs = [];
        for (var _i = 0, _a = this.references; _i < _a.length; _i++) {
            var reference = _a[_i];
            if (!reference || this.owns(reference))
                continue;
            refs.push(reference);
        }
        return refs;
    };
    ReferenceContext.prototype.getInputRegisters = function () {
        var _this = this;
        return this.getInputReferences().map(function (v) { return _this.getValue(v); }).filter(runtime_1.isRegister);
    };
    ReferenceContext.prototype.updateMaxRegisters = function (maybeMax) {
        var parent = this;
        while (parent) {
            parent.maxRegisters = Math.max(parent.maxRegisters, maybeMax);
            parent = parent.parent;
        }
    };
    ReferenceContext.prototype.assignRegisters = function () {
        var startIx = this.parent ? this.parent.totalRegisters : 0;
        for (var _i = 0, _a = this.references; _i < _a.length; _i++) {
            var ref = _a[_i];
            if (ref === undefined)
                continue;
            var local = this.getValue(ref);
            if (runtime_1.isRegister(local)) {
                if (local.offset >= startIx)
                    throw new Error("Trying to assign a register that already has a higher offset");
                if (local.offset === UNASSIGNED) {
                    local.offset = startIx++;
                }
            }
        }
        this.totalRegisters = startIx;
        this.updateMaxRegisters(startIx);
    };
    ReferenceContext.IDs = 0;
    ReferenceContext.stack = [];
    return ReferenceContext;
}());
exports.ReferenceContext = ReferenceContext;
var FlowLevel = (function () {
    function FlowLevel() {
        this.records = [];
        this.lookups = [];
        this.functions = [];
        this.aggregates = [];
        this.inserts = [];
        this.nots = [];
        this.chooses = [];
        this.unions = [];
        this.moves = [];
    }
    FlowLevel.prototype.collect = function (node) {
        if (node instanceof Insert) {
            this.inserts.push(node);
        }
        else if (node instanceof Record) {
            this.records.push(node);
        }
        else if (node instanceof Lookup) {
            this.lookups.push(node);
        }
        else if (node instanceof Fn) {
            this.functions.push(node);
        }
        else if (node instanceof Aggregate) {
            this.aggregates.push(node);
        }
        else if (node instanceof Not) {
            this.nots.push(node);
        }
        else if (node instanceof Choose) {
            this.chooses.push(node);
        }
        else if (node instanceof Union) {
            this.unions.push(node);
        }
        else if (node instanceof Move) {
            if (!this.moves.some(function (v) { return v.toKey() == node.toKey(); })) {
                this.moves.push(node);
            }
        }
        else {
            console.error("Don't know how to collect this type of node: ", node);
            throw new Error("Unknown node type sent to collect");
        }
    };
    FlowLevel.prototype.findReference = function (node) {
        var ref = node.reference();
        var items;
        if (node instanceof Record) {
            items = this.records;
        }
        else {
            console.error("Don't know how to lookup a: ", node);
            throw new Error("Unknown node type sent to findReference");
        }
        for (var _i = 0, items_1 = items; _i < items_1.length; _i++) {
            var item = items_1[_i];
            if (item.reference() === ref)
                return item;
        }
    };
    FlowLevel.prototype.toConstraints = function (injections) {
        var items = [];
        runtime_1.concatArray(items, injections);
        runtime_1.concatArray(items, this.functions);
        runtime_1.concatArray(items, this.records);
        runtime_1.concatArray(items, this.lookups);
        runtime_1.concatArray(items, this.moves);
        return items;
    };
    FlowLevel.prototype.compile = function (nodes, injections, toPass) {
        var items = this.toConstraints(injections);
        var seen = {};
        var constraints = [];
        for (var _i = 0, items_2 = items; _i < items_2.length; _i++) {
            var toCompile = items_2[_i];
            var compiled = toCompile.compile();
            for (var _a = 0, compiled_1 = compiled; _a < compiled_1.length; _a++) {
                var item = compiled_1[_a];
                if (!(item instanceof Runtime.Scan)) {
                    constraints.push(item);
                }
                else {
                    var key = item.toKey();
                    if (!seen[key]) {
                        seen[key] = true;
                        constraints.push(item);
                    }
                }
            }
        }
        var join;
        if (!nodes.length && constraints.length) {
            join = new Runtime.JoinNode(constraints);
            if (!this.records.length && !this.lookups.length && !this.moves.length) {
                join.isStatic = true;
            }
        }
        else if (constraints.length) {
            join = new Runtime.DownstreamJoinNode(constraints);
        }
        else if (nodes.length) {
            join = nodes.pop();
        }
        else {
            join = new Runtime.NoopJoinNode([]);
        }
        // @NOTE: We need to unify all of our sub-blocks along with ourselves
        //        before the root node can allocate registers.
        for (var _b = 0, _c = this.nots; _b < _c.length; _b++) {
            var not = _c[_b];
            // All sub blocks take their parents' items and embed them into
            // the sub block. This is to make sure that the sub only computes the
            // results that might actually join with the parent instead of the possibly
            // very large set of unjoined results. This isn't guaranteed to be optimal
            // and may very well cause us to do more work than necessary. For example if
            // the results of the inner join with many outers, we'll still enumerate the
            // whole set. This *may* be necessary for getting the correct multiplicities
            // anyways, so this is what we're doing.
            var notNodes = not.compile(toPass.concat(items));
            // @TODO: once we have multiple nodes in a not (e.g. aggs, or recursive not/choose/union)
            // this won't be sufficient.
            var notJoinNode = notNodes[0];
            join = new Runtime.AntiJoin(join, notJoinNode, not.getInputRegisters());
        }
        for (var _d = 0, _e = this.chooses; _d < _e.length; _d++) {
            var choose = _e[_d];
            // For why we pass items down, see the comment about not
            join = choose.compile(join);
        }
        for (var _f = 0, _g = this.unions; _f < _g.length; _f++) {
            var union = _g[_f];
            // For why we pass items down, see the comment about not
            join = union.compile(join);
        }
        for (var _h = 0, _j = this.aggregates; _h < _j.length; _h++) {
            var aggregate = _j[_h];
            var aggregateNode = aggregate.compile();
            join = new Runtime.MergeAggregateFlow(join, aggregateNode, aggregate.getJoinRegisters(), aggregate.getOutputRegisters());
        }
        nodes.push(join);
        return nodes;
    };
    FlowLevel.prototype.split = function (context) {
        var maxLevel = 0;
        // if a register can be filled from the database, it doesn't need to be up-leveled,
        // since we always have a value for it from the beginning. Let's find all of those
        // registers so we can ignore them in our functions
        var databaseSupported = runtime_1.concatArray([], this.records);
        runtime_1.concatArray(databaseSupported, this.lookups);
        var supported = [];
        for (var _i = 0, _a = this.records; _i < _a.length; _i++) {
            var item = _a[_i];
            var registers = item.getRegisters();
            for (var _b = 0, registers_1 = registers; _b < registers_1.length; _b++) {
                var register = registers_1[_b];
                supported[register.offset] = true;
            }
        }
        // Every register that's an input to this level is supported by the outer context,
        // so we don't need to level its output.
        for (var _c = 0, _d = context.getInputRegisters(); _c < _d.length; _c++) {
            var register = _d[_c];
            supported[register.offset] = true;
        }
        // Any move whose `from` is supported, has their `to` supported as well
        for (var _e = 0, _f = this.moves; _e < _f.length; _e++) {
            var move = _f[_e];
            var from = move.getInputRegisters()[0];
            var to = move.getOutputRegisters()[0];
            if (from && supported[from.offset]) {
                supported[to.offset] = true;
            }
        }
        // choose, union, and aggregates can cause us to need multiple levels
        // if there's something that relies on an output from one of those, it
        // has to come in a level after that thing is computed.
        var changed = false;
        var leveledRegisters = {};
        var providerToLevel = {};
        var items = runtime_1.concatArray([], this.chooses);
        runtime_1.concatArray(items, this.unions);
        runtime_1.concatArray(items, this.aggregates);
        for (var _g = 0, items_3 = items; _g < items_3.length; _g++) {
            var item = items_3[_g];
            for (var _h = 0, _j = item.getOutputRegisters(); _h < _j.length; _h++) {
                var result = _j[_h];
                var offset = result.offset;
                if (!supported[offset]) {
                    var found = leveledRegisters[offset];
                    if (!found) {
                        found = leveledRegisters[offset] = { level: 1, providers: [] };
                    }
                    leveledRegisters[offset].providers.push(item);
                    providerToLevel[item.ID] = 0;
                    changed = true;
                    maxLevel = 1;
                }
            }
        }
        // go through all the functions, nots, chooses, and unions to see if they rely on
        // a register that has been leveled, if so, they need to move to a level after
        // the provider's heighest
        runtime_1.concatArray(items, this.functions);
        runtime_1.concatArray(items, this.nots);
        runtime_1.concatArray(items, this.moves);
        var remaining = items.length * items.length;
        while (changed && remaining > -1) {
            changed = false;
            for (var _k = 0, items_4 = items; _k < items_4.length; _k++) {
                var item = items_4[_k];
                remaining--;
                if (!item)
                    continue;
                var changedProvider = false;
                var providerLevel = providerToLevel[item.ID] || 0;
                var regs = item.getInputRegisters();
                for (var _l = 0, regs_1 = regs; _l < regs_1.length; _l++) {
                    var input = regs_1[_l];
                    var inputInfo = leveledRegisters[input.offset];
                    if (inputInfo && inputInfo.level > providerLevel) {
                        changedProvider = true;
                        providerLevel = inputInfo.level;
                    }
                }
                if (changedProvider) {
                    providerToLevel[item.ID] = providerLevel;
                    // level my outputs
                    if (item.getOutputRegisters) {
                        for (var _m = 0, _o = item.getOutputRegisters(); _m < _o.length; _m++) {
                            var output = _o[_m];
                            if (supported[output.offset])
                                continue;
                            var outputInfo = leveledRegisters[output.offset];
                            if (!outputInfo) {
                                outputInfo = leveledRegisters[output.offset] = { level: 0, providers: [] };
                            }
                            if (outputInfo.providers.indexOf(item) === -1) {
                                outputInfo.providers.push(item);
                            }
                            if (outputInfo.level <= providerLevel) {
                                outputInfo.level = providerLevel + 1;
                            }
                        }
                    }
                    maxLevel = Math.max(maxLevel, providerLevel);
                    changed = true;
                }
            }
        }
        if (remaining === -1) {
            // we couldn't stratify
            throw new Error("Unstratifiable program: cyclic dependency");
        }
        // now we put all our children into a series of objects that
        // represent each level
        var levels = [];
        for (var ix = 0; ix <= maxLevel; ix++) {
            levels[ix] = new FlowLevel();
        }
        // @FIXME: An implementation issue in 0.3.0 causes stratified subblocks (nots, unions, and chooses) to
        // break in several edge cases. To prevent foot-shootiness until we've resolved the problem, we disallow
        // any orderings which are potentially dangerous.
        var dangerousOrdering = false;
        // all database scans are at the first level
        for (var _p = 0, _q = this.records; _p < _q.length; _p++) {
            var record = _q[_p];
            levels[0].records.push(record);
        }
        for (var _r = 0, _s = this.lookups; _r < _s.length; _r++) {
            var lookup = _s[_r];
            levels[0].lookups.push(lookup);
        }
        // functions/nots/chooses/unions can all be in different levels
        for (var _t = 0, _u = this.functions; _t < _u.length; _t++) {
            var func = _u[_t];
            if (!func)
                continue;
            var level = providerToLevel[func.ID] || 0;
            levels[level].functions.push(func);
        }
        for (var _v = 0, _w = this.nots; _v < _w.length; _v++) {
            var not = _w[_v];
            var level = providerToLevel[not.ID] || 0;
            if (level > 0)
                dangerousOrdering = true;
            levels[level].nots.push(not);
        }
        for (var _x = 0, _y = this.chooses; _x < _y.length; _x++) {
            var choose = _y[_x];
            var level = providerToLevel[choose.ID] || 0;
            if (level > 0)
                dangerousOrdering = true;
            levels[level].chooses.push(choose);
        }
        for (var _z = 0, _0 = this.unions; _z < _0.length; _z++) {
            var union = _0[_z];
            var level = providerToLevel[union.ID] || 0;
            if (level > 0)
                dangerousOrdering = true;
            levels[level].unions.push(union);
        }
        for (var _1 = 0, _2 = this.aggregates; _1 < _2.length; _1++) {
            var aggregate = _2[_1];
            var level = providerToLevel[aggregate.ID] || 0;
            levels[level].aggregates.push(aggregate);
        }
        for (var _3 = 0, _4 = this.moves; _3 < _4.length; _3++) {
            var move = _4[_3];
            var level = providerToLevel[move.ID] || 0;
            levels[level].moves.push(move);
        }
        if (dangerousOrdering)
            throw new Error("Refusing to compile potentially dangerous ordering with stratified subblock(s). This is avoids an implementation issue in the 0.3 runtime. Until it's lifted, you can work around it by splitting any subblocks (nots, unions, or chooses) that depend directly on other subblocks into separate blocks. Please contact the Eve team on the mailing list for further assistance.");
        return levels;
    };
    return FlowLevel;
}());
exports.FlowLevel = FlowLevel;
var DSLBase = (function () {
    function DSLBase() {
        this.ID = DSLBase.CurrentID++;
    }
    DSLBase.CurrentID = 1;
    return DSLBase;
}());
exports.DSLBase = DSLBase;
var LinearFlow = (function (_super) {
    __extends(LinearFlow, _super);
    function LinearFlow(func, parent) {
        var _this = _super.call(this) || this;
        _this.collector = new FlowLevel();
        _this.levels = [];
        //------------------------------------------------------------------
        // End user API
        //------------------------------------------------------------------
        _this.find = function () {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            var tags = args;
            var attributes = tags.pop();
            if (typeof attributes === "string") {
                tags.push(attributes);
                attributes = undefined;
            }
            var active = _this.context.getActive();
            var record = new Record(active, tags, attributes);
            return record.reference();
        };
        _this.lookup = function (record) {
            var active = _this.context.getActive();
            var lookup = new Lookup(active, record);
            return lookup.output();
        };
        _this.record = function () {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            var tags = args;
            var attributes = tags.pop();
            if (typeof attributes === "string") {
                tags.push(attributes);
                attributes = undefined;
            }
            var active = _this.context.getActive();
            var insert = new Insert(active, tags, attributes);
            return insert.reference();
        };
        _this.not = function (func) {
            var active = _this.context.getActive();
            var not = new Not(func, active.flow.parent || _this);
            return;
        };
        _this.union = function () {
            var branches = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                branches[_i] = arguments[_i];
            }
            var active = _this.context.getActive();
            var union = new Union(active, branches, active.flow.parent || _this);
            return union.results.slice();
        };
        _this.choose = function () {
            var branches = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                branches[_i] = arguments[_i];
            }
            var active = _this.context.getActive();
            var choose = new Choose(active, branches, active.flow.parent || _this);
            return choose.results.slice();
        };
        _this.gather = function () {
            var projection = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                projection[_i] = arguments[_i];
            }
            var active = _this.context.getActive();
            return new AggregateBuilder(active, projection);
        };
        _this.transformCode = function (code, functionArgs) {
            if (!functionArgs[0])
                throw new Error("Trying to create a block that has no args with code:\n\n```\n" + code + "\n```");
            var output = falafel("function f() { var $___eve_block$ = " + functionArgs[0] + "; " + code + " }", function (node) {
                if (node.type === 'BinaryExpression') {
                    var func = operators[node.operator];
                    if (node.operator === "+" && (isASTString(node.left) || isASTString(node.right))) {
                        func = operators["concat"];
                    }
                    if (func) {
                        node.update("$___eve_block$.lib." + func + "(" + node.left.source() + ", " + node.right.source() + ")");
                    }
                }
            });
            var updated = output.toString();
            updated = updated.replace("function f() {", "");
            updated = updated.substring(0, updated.length - 1);
            return updated;
        };
        var parentContext = parent ? parent.context : undefined;
        _this.parent = parent;
        _this.createLib();
        _this.context = new ReferenceContext(parentContext, _this);
        var transformed = func;
        if (!parent) {
            transformed = _this.transform(func);
        }
        ReferenceContext.push(_this.context);
        var results = transformed.call(func, _this);
        if (isArray(results))
            _this.results = results;
        else if (results === undefined)
            _this.results = [];
        else
            _this.results = [results];
        for (var _i = 0, _a = _this.results; _i < _a.length; _i++) {
            var result = _a[_i];
            if (isReference(result)) {
                _this.context.register(result);
            }
        }
        ReferenceContext.pop();
        return _this;
    }
    LinearFlow.prototype.createLib = function () {
        var _this = this;
        var lib = {};
        var registered = Runtime.FunctionConstraint.registered;
        var _loop_1 = function (name_1) {
            var parts = name_1.replace(/\/\//gi, "/slash").split("/").map(function (v) { return v === "slash" ? "/" : v; });
            var final = parts.pop();
            var found = lib;
            for (var _i = 0, parts_1 = parts; _i < parts_1.length; _i++) {
                var part = parts_1[_i];
                var next = found[part];
                if (!next)
                    next = found[part] = {};
                found = next;
            }
            found[final] = function () {
                var args = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    args[_i] = arguments[_i];
                }
                var fn = new Fn(_this.context.getActive(), name_1, args);
                return fn.reference();
            };
        };
        for (var name_1 in registered) {
            _loop_1(name_1);
        }
        lib.compare["=="] = function (a, b) {
            _this.context.getActive().equality(a, b);
            return b;
        };
        this.lib = lib;
    };
    //------------------------------------------------------------------
    // Collector interactions
    //------------------------------------------------------------------
    LinearFlow.prototype.collect = function (node) {
        this.collector.collect(node);
    };
    LinearFlow.prototype.findReference = function (node) {
        this.collector.findReference(node);
    };
    //------------------------------------------------------------------
    // Inputs/outputs
    //------------------------------------------------------------------
    LinearFlow.prototype.getInputRegisters = function () {
        return this.context.getInputRegisters();
    };
    //------------------------------------------------------------------
    // Compile
    //------------------------------------------------------------------
    LinearFlow.prototype.unify = function () {
        this.context.unify();
    };
    LinearFlow.prototype.compile = function (_) {
        if (_ === void 0) { _ = []; }
        var items = [];
        this.unify();
        var nodes = [];
        for (var _i = 0, _a = this.context.getMoves(); _i < _a.length; _i++) {
            var move = _a[_i];
            this.collect(move);
        }
        // Split our collector into levels
        var levels = this.collector.split(this.context);
        var localItems = items.slice();
        for (var _b = 0, levels_1 = levels; _b < levels_1.length; _b++) {
            var level = levels_1[_b];
            nodes = level.compile(nodes, items, localItems);
            runtime_1.concatArray(localItems, level.toConstraints([]));
        }
        // all the inputs end up at the end
        var outputs = [];
        for (var _c = 0, _d = this.collector.inserts; _c < _d.length; _c++) {
            var record = _d[_c];
            var compiled = record.compile();
            for (var _e = 0, compiled_2 = compiled; _e < compiled_2.length; _e++) {
                var node = compiled_2[_e];
                if (node instanceof Runtime.WatchNode) {
                    nodes.push(node);
                }
                else {
                    outputs.push(node); // @FIXME: types
                }
            }
        }
        if (outputs.length) {
            nodes.push(new Runtime.OutputWrapperNode(outputs));
        }
        this.levels = levels;
        return nodes;
    };
    //------------------------------------------------------------------
    // Function transformation
    //------------------------------------------------------------------
    LinearFlow.prototype.transform = function (func) {
        return macro(func, this.transformCode);
    };
    return LinearFlow;
}(DSLBase));
exports.LinearFlow = LinearFlow;
//--------------------------------------------------------------------
// WatchFlow
//--------------------------------------------------------------------
var WatchFlow = (function (_super) {
    __extends(WatchFlow, _super);
    function WatchFlow() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    WatchFlow.prototype.collect = function (node) {
        if (!(node instanceof Watch) && node instanceof Insert) {
            node = node.toWatch();
            return;
        }
        _super.prototype.collect.call(this, node);
    };
    return WatchFlow;
}(LinearFlow));
exports.WatchFlow = WatchFlow;
//--------------------------------------------------------------------
// CommitFlow
//--------------------------------------------------------------------
var CommitFlow = (function (_super) {
    __extends(CommitFlow, _super);
    function CommitFlow() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    CommitFlow.prototype.collect = function (node) {
        if (!(node instanceof CommitInsert) && !(node instanceof CommitRemove) && node instanceof Insert) {
            node = node.toCommit();
            return;
        }
        _super.prototype.collect.call(this, node);
    };
    return CommitFlow;
}(LinearFlow));
exports.CommitFlow = CommitFlow;
//--------------------------------------------------------------------
// DSL runtime types
//--------------------------------------------------------------------
//--------------------------------------------------------------------
// Record
//--------------------------------------------------------------------
var Record = (function (_super) {
    __extends(Record, _super);
    function Record(context, tags, attributes, record) {
        if (tags === void 0) { tags = []; }
        if (attributes === void 0) { attributes = {}; }
        var _this = _super.call(this) || this;
        _this.context = context;
        _this.record = record;
        if (!record) {
            _this.record = _this.createReference();
        }
        else {
            context.register(record);
        }
        var attrs = [];
        for (var _i = 0, tags_1 = tags; _i < tags_1.length; _i++) {
            var tag = tags_1[_i];
            attrs.push("tag", tag);
        }
        var keys = Object.keys(attributes).sort();
        for (var _a = 0, keys_1 = keys; _a < keys_1.length; _a++) {
            var attr = keys_1[_a];
            var value = attributes[attr];
            if (isArray(value)) {
                for (var _b = 0, value_2 = value; _b < value_2.length; _b++) {
                    var current = value_2[_b];
                    if (isReference(current))
                        context.register(current);
                    attrs.push(attr, current);
                }
            }
            else {
                if (isReference(value))
                    context.register(value);
                attrs.push(attr, value);
            }
        }
        _this.attributes = attrs;
        context.flow.collect(_this);
        return _this;
    }
    Record.prototype.createReference = function () {
        return Reference.create(this.context, this);
    };
    Record.prototype.createSub = function (context, record) {
        return new Record(context, undefined, undefined, record);
    };
    Record.prototype.reference = function () {
        return this.record;
    };
    Record.prototype.add = function (context, attribute, value) {
        var insert = new Insert(context, [], {}, this.reference());
        if (!isArray(value)) {
            insert.add(context, attribute, value);
        }
        else {
            for (var _i = 0, value_3 = value; _i < value_3.length; _i++) {
                var v = value_3[_i];
                insert.add(context, attribute, v);
            }
        }
    };
    Record.prototype.remove = function (context, attribute, value) {
        var insert = new Insert(context, [], {}, this.reference());
        if (!isArray(value)) {
            insert.remove(context, attribute, value);
        }
        else {
            for (var _i = 0, value_4 = value; _i < value_4.length; _i++) {
                var v = value_4[_i];
                insert.remove(context, attribute, v);
            }
        }
    };
    Record.prototype.copyToContext = function (activeContext) {
        var found = activeContext.flow.findReference(this);
        if (found)
            return found;
        var neue = this.createSub(activeContext, this.record);
        activeContext.register(this.record);
        return neue;
    };
    Record.prototype.findAttribute = function (name) {
        var attributes = this.attributes;
        for (var ix = 0, len = attributes.length; ix < len; ix += 2) {
            var attrName = attributes[ix];
            var value = attributes[ix + 1];
            if (attrName === name && isReference(value))
                return value;
        }
    };
    Record.prototype.access = function (refContext, activeContext, prop) {
        var record = this;
        if (refContext !== activeContext) {
            record = this.copyToContext(activeContext);
        }
        var found = record.findAttribute(prop);
        if (found)
            return found;
        // we need to add this attribute to us and return that
        var attrRecord = this.createSub(activeContext);
        var attrRef = attrRecord.reference();
        record.attributes.push(prop, attrRef);
        return attrRef;
    };
    Record.prototype.getRegisters = function () {
        var _this = this;
        var values = [this.record];
        var attributes = this.attributes;
        for (var ix = 0, len = attributes.length; ix < len; ix += 2) {
            var a = attributes[ix];
            var v = attributes[ix + 1];
            values.push(a, v);
        }
        return values.map(function (v) { return _this.context.getValue(v); }).filter(runtime_1.isRegister);
    };
    Record.prototype.compile = function () {
        var _a = this, attributes = _a.attributes, context = _a.context;
        var constraints = [];
        var e = context.interned(this.record);
        for (var ix = 0, len = attributes.length; ix < len; ix += 2) {
            var a = attributes[ix];
            var v = attributes[ix + 1];
            // @TODO: get a real node id
            constraints.push(new Runtime.Scan(e, context.interned(a), context.interned(v), Runtime.IGNORE_REG));
        }
        return constraints;
    };
    return Record;
}(DSLBase));
exports.Record = Record;
//--------------------------------------------------------------------
// Lookup
//--------------------------------------------------------------------
var Lookup = (function (_super) {
    __extends(Lookup, _super);
    function Lookup(context, record) {
        var _this = _super.call(this) || this;
        _this.context = context;
        _this.record = record;
        if (isReference(record)) {
            context.register(record);
        }
        var attribute = new Record(context);
        var value = new Record(context);
        _this.attribute = attribute.reference();
        _this.value = value.reference();
        context.flow.collect(_this);
        return _this;
    }
    Lookup.prototype.reference = function () {
        return this.record;
    };
    Lookup.prototype.output = function () {
        return { attribute: this.attribute, value: this.value };
    };
    Lookup.prototype.compile = function () {
        var scans = [];
        var context = this.context;
        scans.push(new Runtime.Scan(context.interned(this.record), context.interned(this.attribute), context.interned(this.value), Runtime.IGNORE_REG));
        return scans;
    };
    return Lookup;
}(DSLBase));
exports.Lookup = Lookup;
//--------------------------------------------------------------------
// Move
//--------------------------------------------------------------------
var Move = (function (_super) {
    __extends(Move, _super);
    function Move(context, from, to) {
        var _this = _super.call(this) || this;
        _this.context = context;
        _this.from = from;
        if (isReference(from)) {
            context.register(from);
        }
        if (!to) {
            if (!isReference(from))
                throw new Error("Move where the to is not a reference");
            _this.to = from;
        }
        else {
            _this.to = to;
        }
        return _this;
    }
    Move.prototype.toKey = function () {
        var from = isReference(this.from) ? "[" + this.from.__ID + "]" : this.from;
        var to = isReference(this.to) ? "[" + this.to.__ID + "]" : this.to;
        return from + "|" + to;
    };
    Move.prototype.getInputRegisters = function () {
        var value = this.context.getValue(this.from);
        if (runtime_1.isRegister(value)) {
            return [value];
        }
        return [];
    };
    Move.prototype.getOutputRegisters = function () {
        var to = this.to;
        var parent = to.__context.getValue(to);
        return [parent];
    };
    Move.prototype.compile = function () {
        var _a = this, from = _a.from, to = _a.to;
        var local = this.context.interned(from);
        var parent = to.__context.getValue(to);
        return [new Runtime.MoveConstraint(local, parent)];
    };
    return Move;
}(DSLBase));
exports.Move = Move;
//--------------------------------------------------------------------
// Insert
//--------------------------------------------------------------------
var Insert = (function (_super) {
    __extends(Insert, _super);
    function Insert(context, tags, attributes, record) {
        if (tags === void 0) { tags = []; }
        if (attributes === void 0) { attributes = {}; }
        var _this = _super.call(this, context, tags, attributes, record) || this;
        _this.context = context;
        if (!record) {
            // we have to make our ID generation function
            var args = [];
            for (var ix = 0, len = _this.attributes.length; ix < len; ix += 2) {
                var a = _this.attributes[ix];
                var v = _this.attributes[ix + 1];
                args.push(a, v);
            }
            var genId = new Fn(context, "eve/internal/gen-id", args, _this.reference());
        }
        return _this;
    }
    Insert.prototype.createReference = function () {
        // @TODO: create an InsertReference type and return that here
        return Reference.create(this.context, this);
    };
    Insert.prototype.createSub = function (context, record) {
        return new Insert(context, undefined, undefined, record);
    };
    Insert.prototype.add = function (context, attribute, value) {
        if (!isArray(value)) {
            this.attributes.push(attribute, value);
        }
        else {
            for (var _i = 0, value_5 = value; _i < value_5.length; _i++) {
                var v = value_5[_i];
                this.attributes.push(attribute, v);
            }
        }
        return this.reference();
    };
    Insert.prototype.remove = function (context, attribute, value) {
        var remove = new Remove(context, [], {}, this.record);
        if (!isArray(value)) {
            remove.attributes.push(attribute, value);
        }
        else {
            for (var _i = 0, value_6 = value; _i < value_6.length; _i++) {
                var v = value_6[_i];
                remove.attributes.push(attribute, v);
            }
        }
        return this.reference();
    };
    Insert.prototype.toWatch = function () {
        var watch = new Watch(this.context, [], {}, this.record);
        watch.attributes = this.attributes;
        return watch;
    };
    Insert.prototype.toCommit = function () {
        var commit = new CommitInsert(this.context, [], {}, this.record);
        commit.attributes = this.attributes;
        return commit;
    };
    Insert.prototype.compile = function () {
        var _a = this, attributes = _a.attributes, context = _a.context;
        var nodes = [];
        var e = context.interned(this.record);
        for (var ix = 0, len = attributes.length; ix < len; ix += 2) {
            var a = attributes[ix];
            var v = attributes[ix + 1];
            // @TODO: get a real node id
            var n = uuid_1.v4();
            nodes.push(new Runtime.InsertNode(e, context.interned(a), context.interned(v), context.interned(n)));
        }
        return nodes;
    };
    return Insert;
}(Record));
exports.Insert = Insert;
//--------------------------------------------------------------------
// Remove
//--------------------------------------------------------------------
var Remove = (function (_super) {
    __extends(Remove, _super);
    function Remove() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Remove.prototype.toCommit = function () {
        var commit = new CommitRemove(this.context, [], {}, this.record);
        commit.attributes = this.attributes;
        return commit;
    };
    Remove.prototype.compile = function () {
        var _a = this, attributes = _a.attributes, context = _a.context;
        var nodes = [];
        var e = context.interned(this.record);
        for (var ix = 0, len = attributes.length; ix < len; ix += 2) {
            var a = attributes[ix];
            var v = attributes[ix + 1];
            // @TODO: get a real node id
            var n = uuid_1.v4();
            var internedV = context.maybeInterned(v); // @FIXME
            internedV = internedV !== undefined ? internedV : Runtime.IGNORE_REG;
            var internedA = context.maybeInterned(a); // @FIXME
            internedA = internedA !== undefined ? internedA : Runtime.IGNORE_REG;
            nodes.push(new Runtime.RemoveNode(e, internedA, internedV, context.interned(n)));
        }
        return nodes;
    };
    return Remove;
}(Insert));
exports.Remove = Remove;
//--------------------------------------------------------------------
// Watch
//--------------------------------------------------------------------
var Watch = (function (_super) {
    __extends(Watch, _super);
    function Watch() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Watch.prototype.compile = function () {
        var _a = this, attributes = _a.attributes, context = _a.context;
        var nodes = [];
        var e = context.interned(this.record);
        for (var ix = 0, len = attributes.length; ix < len; ix += 2) {
            var a = attributes[ix];
            var v = attributes[ix + 1];
            // @TODO: get a real node id
            var n = uuid_1.v4();
            nodes.push(new Runtime.WatchNode(e, context.interned(a), context.interned(v), context.interned(n), context.flow.ID));
        }
        return nodes;
    };
    return Watch;
}(Insert));
exports.Watch = Watch;
//--------------------------------------------------------------------
// CommitInsert
//--------------------------------------------------------------------
var CommitInsert = (function (_super) {
    __extends(CommitInsert, _super);
    function CommitInsert() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    CommitInsert.prototype.compile = function () {
        var _a = this, attributes = _a.attributes, context = _a.context;
        var nodes = [];
        var e = context.interned(this.record);
        for (var ix = 0, len = attributes.length; ix < len; ix += 2) {
            var a = attributes[ix];
            var v = attributes[ix + 1];
            // @TODO: get a real node id
            var n = uuid_1.v4();
            nodes.push(new Runtime.CommitInsertNode(e, context.interned(a), context.interned(v), context.interned(n)));
        }
        return nodes;
    };
    return CommitInsert;
}(Insert));
exports.CommitInsert = CommitInsert;
//--------------------------------------------------------------------
// CommitRemove
//--------------------------------------------------------------------
var CommitRemove = (function (_super) {
    __extends(CommitRemove, _super);
    function CommitRemove() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    CommitRemove.prototype.compile = function () {
        var _a = this, attributes = _a.attributes, context = _a.context;
        var nodes = [];
        var e = context.interned(this.record);
        for (var ix = 0, len = attributes.length; ix < len; ix += 2) {
            var a = attributes[ix];
            var v = attributes[ix + 1];
            // @TODO: get a real node id
            var n = uuid_1.v4();
            var internedV = context.maybeInterned(v); // @FIXME
            internedV = internedV !== undefined ? internedV : Runtime.IGNORE_REG;
            var internedA = context.maybeInterned(a); // @FIXME
            internedA = internedA !== undefined ? internedA : Runtime.IGNORE_REG;
            nodes.push(new Runtime.CommitRemoveNode(e, internedA, internedV, context.interned(n)));
        }
        return nodes;
    };
    return CommitRemove;
}(Remove));
exports.CommitRemove = CommitRemove;
//--------------------------------------------------------------------
// Fn
//--------------------------------------------------------------------
var Fn = (function (_super) {
    __extends(Fn, _super);
    function Fn(context, name, args, output) {
        var _this = _super.call(this) || this;
        _this.context = context;
        _this.name = name;
        _this.args = args;
        var filter = Runtime.FunctionConstraint.fetchInfo(name).filter;
        if (output) {
            _this.output = output;
        }
        else if (filter) {
            _this.output = args[args.length - 1];
        }
        else {
            _this.output = Reference.create(context, _this);
        }
        for (var _i = 0, args_1 = args; _i < args_1.length; _i++) {
            var arg = args_1[_i];
            if (isReference(arg)) {
                context.register(arg);
            }
        }
        context.flow.collect(_this);
        return _this;
    }
    Fn.prototype.reference = function () {
        return this.output;
    };
    Fn.prototype.access = function (refContext, activeContext, prop) {
        throw new Error("Implement me!");
    };
    Fn.prototype.getInputRegisters = function () {
        var context = this.context;
        return this.args.map(function (v) { return context.getValue(v); }).filter(runtime_1.isRegister);
    };
    Fn.prototype.getOutputRegisters = function () {
        var registers = [];
        var value = this.context.getValue(this.output);
        if (runtime_1.isRegister(value)) {
            registers.push(value);
        }
        return registers;
    };
    Fn.prototype.compile = function () {
        var constraints = [];
        var _a = this, context = _a.context, name = _a.name;
        var values = this.args.map(function (v) { return context.interned(v); });
        var _b = Runtime.FunctionConstraint.fetchInfo(name), variadic = _b.variadic, filter = _b.filter;
        var returns = {};
        if (!filter) {
            returns.result = context.interned(this.output);
        }
        var constraint;
        if (variadic) {
            constraint = Runtime.FunctionConstraint.create(name, returns, values);
        }
        else {
            constraint = Runtime.FunctionConstraint.create(name, returns, []);
            var ix = 0;
            for (var _i = 0, _c = constraint.argNames; _i < _c.length; _i++) {
                var arg = _c[_i];
                constraint.fields[arg] = values[ix];
                ix++;
            }
        }
        constraints.push(constraint);
        return constraints;
    };
    return Fn;
}(DSLBase));
exports.Fn = Fn;
//--------------------------------------------------------------------
// Aggregate
//--------------------------------------------------------------------
var Aggregate = (function (_super) {
    __extends(Aggregate, _super);
    function Aggregate(context, aggregate, projection, group, args, output) {
        var _this = _super.call(this) || this;
        _this.context = context;
        _this.aggregate = aggregate;
        _this.projection = projection;
        _this.group = group;
        _this.args = args;
        if (output) {
            _this.output = output;
        }
        else {
            _this.output = Reference.create(context, _this);
        }
        _this.output.__forceRegister = true;
        // add all of our args to our projection
        for (var _i = 0, args_2 = args; _i < args_2.length; _i++) {
            var arg = args_2[_i];
            if (isReference(arg)) {
                projection.push(arg);
            }
        }
        context.flow.collect(_this);
        return _this;
    }
    Aggregate.prototype.getJoinRegisters = function () {
        var context = this.context;
        var items = []; //concatArray([], this.args);
        if (this.aggregate === Runtime.SortNode) {
            runtime_1.concatArray(items, this.projection);
        }
        runtime_1.concatArray(items, this.group);
        return items.map(function (v) { return context.getValue(v); }).filter(runtime_1.isRegister);
    };
    Aggregate.prototype.getInputRegisters = function () {
        var context = this.context;
        var items = runtime_1.concatArray([], this.args);
        runtime_1.concatArray(items, this.projection);
        runtime_1.concatArray(items, this.group);
        return items.map(function (v) { return context.getValue(v); }).filter(runtime_1.isRegister);
    };
    Aggregate.prototype.getOutputRegisters = function () {
        // @TODO: should this blow up if it doesn't resolve to a register?
        var value = this.context.getValue(this.output);
        return [value];
    };
    Aggregate.prototype.compile = function () {
        var context = this.context;
        var groupRegisters = this.group.map(function (v) { return context.getValue(v); }).filter(runtime_1.isRegister);
        var projectRegisters = this.projection.map(function (v) { return context.getValue(v); }).filter(runtime_1.isRegister);
        var inputs = this.args.map(function (v) { return context.interned(v); });
        var agg = new this.aggregate(groupRegisters, projectRegisters, inputs, this.getOutputRegisters());
        return agg;
    };
    Aggregate.prototype.reference = function () {
        return this.output;
    };
    Aggregate.prototype.per = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        for (var _a = 0, args_3 = args; _a < args_3.length; _a++) {
            var arg = args_3[_a];
            this.group.push(arg);
        }
    };
    return Aggregate;
}(DSLBase));
exports.Aggregate = Aggregate;
var AggregateBuilder = (function () {
    function AggregateBuilder(context, projection) {
        this.context = context;
        this.projection = projection;
        this.group = [];
    }
    AggregateBuilder.prototype.per = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        for (var _a = 0, args_4 = args; _a < args_4.length; _a++) {
            var arg = args_4[_a];
            this.group.push(arg);
        }
        return this;
    };
    AggregateBuilder.prototype.checkBlock = function () {
        var active = this.context.getActive();
        if (active !== this.context)
            throw new Error("Cannot gather in one scope and aggregate in another");
    };
    AggregateBuilder.prototype.sum = function (value) {
        this.checkBlock();
        var agg = new Aggregate(this.context, stdlib_1.SumAggregate, this.projection, this.group, [value]);
        return agg.reference();
    };
    AggregateBuilder.prototype.count = function () {
        this.checkBlock();
        var agg = new Aggregate(this.context, stdlib_1.SumAggregate, this.projection, this.group, [1]);
        return agg.reference();
    };
    AggregateBuilder.prototype.sort = function () {
        var directions = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            directions[_i] = arguments[_i];
        }
        this.checkBlock();
        var agg = new Aggregate(this.context, Runtime.SortNode, this.projection, this.group, directions);
        return agg.reference();
    };
    return AggregateBuilder;
}());
exports.AggregateBuilder = AggregateBuilder;
//--------------------------------------------------------------------
// Not
//--------------------------------------------------------------------
var Not = (function (_super) {
    __extends(Not, _super);
    function Not(func, parent) {
        var _this = _super.call(this, func, parent) || this;
        parent.collect(_this);
        return _this;
    }
    return Not;
}(LinearFlow));
exports.Not = Not;
//--------------------------------------------------------------------
// Union
//--------------------------------------------------------------------
var Union = (function (_super) {
    __extends(Union, _super);
    function Union(context, branchFunctions, parent, existingResults) {
        var _this = _super.call(this) || this;
        _this.context = context;
        _this.branches = [];
        _this.results = [];
        _this.inputs = [];
        _this.branchInputs = [];
        var _a = _this, branches = _a.branches, results = _a.results;
        var resultCount;
        if (existingResults) {
            resultCount = existingResults.length;
            _this.results = existingResults.slice();
            results = _this.results;
            for (var _i = 0, results_1 = results; _i < results_1.length; _i++) {
                var result = results_1[_i];
                if (isReference(result)) {
                    result.__forceRegister = true;
                }
                else {
                    throw new Error("Non-reference choose/union result");
                }
            }
        }
        var ix = 0;
        for (var _b = 0, branchFunctions_1 = branchFunctions; _b < branchFunctions_1.length; _b++) {
            var branch = branchFunctions_1[_b];
            var flow = new LinearFlow(branch, parent);
            var branchResultCount = _this.resultCount(flow.results);
            if (resultCount === undefined) {
                resultCount = branchResultCount;
                for (var resultIx_1 = 0; resultIx_1 < resultCount; resultIx_1++) {
                    var ref = Reference.create(context);
                    ref.__forceRegister = true;
                    results.push(ref);
                }
            }
            else if (resultCount !== branchResultCount) {
                throw new Error("Choose branch " + ix + " doesn't have the right number of returns. I expected " + resultCount + ", but got " + branchResultCount);
            }
            _this.setBranchInputs(ix, flow.context.getInputReferences());
            var resultIx = 0;
            for (var _c = 0, results_2 = results; _c < results_2.length; _c++) {
                var result = results_2[_c];
                flow.collect(new Move(flow.context, flow.results[resultIx], result));
                resultIx++;
            }
            branches.push(flow);
            ix++;
        }
        context.flow.collect(_this);
        return _this;
    }
    Union.prototype.setBranchInputs = function (branchIx, inputs) {
        var branchInputs = this.branchInputs[branchIx] = [];
        for (var _i = 0, inputs_1 = inputs; _i < inputs_1.length; _i++) {
            var ref = inputs_1[_i];
            if (this.inputs.indexOf(ref) === -1) {
                this.inputs.push(ref);
            }
            branchInputs.push(ref);
        }
    };
    Union.prototype.getInputRegisters = function () {
        var context = this.context;
        var inputs = this.inputs.map(function (v) { return context.getValue(v); }).filter(runtime_1.isRegister);
        return inputs;
    };
    Union.prototype.getOutputRegisters = function () {
        var context = this.context;
        return this.results.map(function (v) { return context.getValue(v); }).filter(runtime_1.isRegister);
    };
    Union.prototype.resultCount = function (result) {
        if (result && result.constructor === Array) {
            return result.length;
        }
        else if (result) {
            return 1;
        }
        return 0;
    };
    Union.prototype.build = function (left, nodes, inputs, outputs, extraOuterJoins) {
        return new Runtime.UnionFlow(left, nodes, inputs, outputs, extraOuterJoins);
    };
    Union.prototype.compile = function (join) {
        var _a = this, context = _a.context, branchInputs = _a.branchInputs;
        var nodes = [];
        var inputs = [];
        var outputs = this.getOutputRegisters();
        var ix = 0;
        var _loop_2 = function (flow) {
            if (flow.collector.chooses.length > 0 || flow.collector.unions.length > 0) {
                throw new Error("Nested chooses and unions are not currently supported");
            }
            var compiled = flow.compile();
            // @NOTE: Not sure why TS isn't correctly pegging this as filtered to only Registers already.
            var flowInputs = branchInputs[ix].map(function (v) { return context.getValue(v); }).filter(runtime_1.isRegister);
            if (compiled.length > 1) {
                // if this branch has an aggregate in it, we need to do some surgery to make sure that
                // the JoinNode gets wrapped in an AggregateOuterLookup. For the motivation see the
                // comment about the definition for AggregateOuterLookup.
                if (flow.collector.aggregates.length > 0) {
                    var aggMerge_1 = compiled[0];
                    // we have to make sure that the outer lookup fields are actually used in the join node for
                    // the aggregate merge otherwise they will never join. For example if you have:
                    //
                    // board = [#board size: N, not(winner)]
                    // winner = if N = gather/count[for: cell, per: col]
                    //             cell = [#bar board col player]
                    //             then player
                    //
                    // N is both the input to the branch as well as the output of the aggregate, but it won't
                    // be available when doing the join *before* the aggregate. If it's included in the outer
                    // lookup we'll join the board size with undefined every time and always fail.
                    var outerLookupInputs = flowInputs.filter(function (v) {
                        var next = aggMerge_1.left;
                        if (next instanceof Runtime.JoinNode) {
                            return next.registerLookup[v.offset];
                        }
                        else {
                            throw new Error("Don't know how to handle multi-node aggregates that don't start with a join");
                        }
                    });
                    aggMerge_1.left = new Runtime.AggregateOuterLookup(join, aggMerge_1.left, outerLookupInputs);
                }
                nodes.push(new Runtime.LinearFlow(compiled));
            }
            else {
                nodes.push(compiled[0]);
            }
            inputs.push(flowInputs);
            ix++;
        };
        for (var _i = 0, _b = this.branches; _i < _b.length; _i++) {
            var flow = _b[_i];
            _loop_2(flow);
        }
        var extraJoins = [];
        for (var _c = 0, _d = this.results; _c < _d.length; _c++) {
            var result = _d[_c];
            if (result.__owner && result.__owner instanceof Record && result.__owner.attributes.length) {
                var maybeReg = context.getValue(result);
                if (runtime_1.isRegister(maybeReg)) {
                    extraJoins.push(maybeReg);
                }
            }
        }
        return this.build(join, nodes, inputs, this.getOutputRegisters(), extraJoins);
    };
    return Union;
}(DSLBase));
exports.Union = Union;
//--------------------------------------------------------------------
// Choose
//--------------------------------------------------------------------
var Choose = (function (_super) {
    __extends(Choose, _super);
    function Choose() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Choose.prototype.build = function (left, nodes, inputs, outputs, extraOuterJoins) {
        return new Runtime.ChooseFlow(left, nodes, inputs, outputs, extraOuterJoins);
    };
    return Choose;
}(Union));
exports.Choose = Choose;
var Program = (function () {
    function Program(name) {
        this.name = name;
        this.blocks = [];
        this.flows = [];
        this.nodeCount = 0;
        this.nextTransactionId = 0;
        this.exporter = new watcher_1.Exporter();
        this.watchers = {};
        this.index = new indexes.HashIndex();
        this.context = new Runtime.EvaluationContext(this.index);
    }
    Program.prototype.constants = function (obj) {
        if (!this._constants)
            this._constants = {};
        for (var constant in obj) {
            if (this._constants[constant] && this._constants[constant] !== obj[constant]) {
                // throw new Error("Unable to rebind existing constant");
            }
            this._constants[constant] = obj[constant];
        }
        return this;
    };
    Program.prototype.injectConstants = function (func) {
        var _this = this;
        if (!this._constants)
            return func;
        return macro(func, function (code) {
            var constants = _this._constants;
            for (var constant in constants) {
                code = code.replace(new RegExp("{{" + constant + "}}", "gm"), "" + constants[constant]);
            }
            return code;
        });
    };
    Program.prototype.clear = function () {
        this.index = new indexes.HashIndex();
        this.context = new Runtime.EvaluationContext(this.index);
    };
    Program.prototype._bind = function (name, flow) {
        var nodes = flow.compile();
        var block = new Runtime.Block(name, nodes, flow.context.maxRegisters);
        this.flows.push(flow);
        this.blocks.push(block);
        return block;
    };
    Program.prototype.bind = function (name, func) {
        var flow = new LinearFlow(this.injectConstants(func));
        this._bind(name, flow);
        return this;
    };
    Program.prototype.blockChangeTransaction = function (added, removed) {
        for (var _i = 0, removed_1 = removed; _i < removed_1.length; _i++) {
            var remove = removed_1[_i];
            var ix = this.blocks.indexOf(remove);
            this.blocks.splice(ix, 1);
        }
        // console.time("input");
        var trans = new Runtime.BlockChangeTransaction(this.context, this.nextTransactionId++, added, removed, this.blocks, this.lastWatch ? this.exporter.handle : undefined);
        trans.exec(this.context);
        // console.timeEnd("input");
        // console.info(trans.changes.map((change, ix) => `    <- ${change}`).join("\n"));
        return trans;
    };
    Program.prototype.input = function (changes) {
        // console.time("input");
        if (changes[0].transaction >= this.nextTransactionId)
            this.nextTransactionId = changes[0].transaction + 1;
        var trans = new Runtime.Transaction(this.context, changes[0].transaction, this.blocks, this.lastWatch ? this.exporter.handle : undefined);
        for (var _i = 0, changes_1 = changes; _i < changes_1.length; _i++) {
            var change = changes_1[_i];
            trans.output(this.context, change);
        }
        trans.exec(this.context);
        // console.timeEnd("input");
        // console.info(trans.changes.map((change, ix) => `    <- ${change}`).join("\n"));
        // let g:any = global;
        // let filterPrefix = "eve/compiler/";
        // let filteredIds = g.filteredIds = g.filteredIds || [];
        // for(let change of trans.changes) {
        //   if(change.a == GlobalInterner.get("tag") && (""+GlobalInterner.reverse(change.v)).indexOf(filterPrefix) == 0) {
        //     filteredIds.push(change.e);
        //   }
        // }
        // let filtered = trans.changes.filter((c) => filteredIds.indexOf(c.e) !== -1);
        // if(filtered.length) {
        //   console.log("---------------INPUT-----------")
        //   console.log(filtered.map((change, ix) => `    <- ${change}`).join("\n"));
        // }
        return trans;
    };
    Program.prototype.inputEAVs = function (eavcs) {
        var changes = [];
        var transactionId = this.nextTransactionId++;
        for (var _i = 0, _a = eavcs; _i < _a.length; _i++) {
            var _b = _a[_i], e = _b[0], a = _b[1], v = _b[2], _c = _b[3], c = _c === void 0 ? 1 : _c;
            changes.push(runtime_1.Change.fromValues(e, a, v, "input", transactionId, 0, c));
        }
        return this.input(changes);
    };
    Program.prototype.test = function (transaction, eavns) {
        if ("group" in console)
            console.group(this.name + " test " + transaction);
        if (transaction >= this.nextTransactionId)
            this.nextTransactionId = transaction + 1;
        var trans = new Runtime.Transaction(this.context, transaction, this.blocks, this.lastWatch ? this.exporter.handle : undefined);
        for (var _i = 0, _a = eavns; _i < _a.length; _i++) {
            var _b = _a[_i], e = _b[0], a = _b[1], v = _b[2], _c = _b[3], round = _c === void 0 ? 0 : _c, _d = _b[4], count = _d === void 0 ? 1 : _d;
            var change = Runtime.Change.fromValues(e, a, v, "input", transaction, round, count);
            trans.output(this.context, change);
        }
        trans.exec(this.context);
        console.info(trans.changes.map(function (change, ix) { return "    <- " + change; }).join("\n"));
        if ("group" in console)
            console.groupEnd();
        return this;
    };
    Program.prototype._commit = function (name, flow) {
        var nodes = flow.compile();
        var block = new Runtime.Block(name, nodes, flow.context.maxRegisters);
        this.flows.push(flow);
        this.blocks.push(block);
        return block;
    };
    Program.prototype.commit = function (name, func) {
        var flow = new CommitFlow(this.injectConstants(func));
        this._commit(name, flow);
        return this;
    };
    Program.prototype.attach = function (id) {
        var WatcherConstructor = watcher_1.Watcher.get(id);
        if (!WatcherConstructor)
            throw new Error("Unable to attach unknown watcher '" + id + "'.");
        if (this.watchers[id])
            return this.watchers[id];
        var watcher = new WatcherConstructor(this);
        this.watchers[id] = watcher;
        return watcher;
    };
    Program.prototype._watch = function (name, flow) {
        var nodes = flow.compile();
        var block = new Runtime.Block(name, nodes, flow.context.maxRegisters);
        this.lastWatch = flow.ID;
        this.flows.push(flow);
        this.blocks.push(block);
        return block;
    };
    Program.prototype.watch = function (name, func) {
        var flow = new WatchFlow(this.injectConstants(func));
        this._watch(name, flow);
        return this;
    };
    Program.prototype.asDiffs = function (handler) {
        if (!this.lastWatch)
            throw new Error("Must have at least one watch block to export as diffs.");
        this.exporter.triggerOnDiffs(this.lastWatch, handler);
        return this;
    };
    Program.prototype.asObjects = function (handler) {
        if (!this.exporter || !this.lastWatch)
            throw new Error("Must have at least one watch block to export as diffs.");
        this.exporter.triggerOnObjects(this.lastWatch, handler);
        return this;
    };
    Program.prototype.load = function (str) {
        var results = Parser.parseDoc(str);
        this.attach("ui");
        this.attach("html");
        this.attach("compiler");
        if (results.results.eavs.length) {
            this.inputEAVs(results.results.eavs);
        }
    };
    return Program;
}());
exports.Program = Program;
//# sourceMappingURL=dsl2.js.map