"use strict";
//--------------------------------------------------------------------
// The Eve compiler as a watcher
//--------------------------------------------------------------------
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var watcher_1 = require("./watcher");
var runtime_1 = require("../runtime/runtime");
var dsl2_1 = require("../runtime/dsl2");
var stdlib_1 = require("../runtime/stdlib");
var Runtime = require("../runtime/runtime");
require("setimmediate");
var CompilerWatcher = (function (_super) {
    __extends(CompilerWatcher, _super);
    function CompilerWatcher() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.blocksToCompile = {};
        _this.blocksToRemove = {};
        _this.blocks = {};
        _this.items = {};
        _this.watcherFunctions = {};
        _this.programToInjectInto = _this.program;
        //------------------------------------------------------------------
        // Compile queue
        //------------------------------------------------------------------
        _this.queued = false;
        _this.runQueue = function () {
            var adds = [];
            var removes = [];
            for (var ID in _this.blocksToRemove) {
                if (!_this.blocks[ID])
                    continue;
                removes.push(_this.blocks[ID]);
                delete _this.blocks[ID];
            }
            for (var ID in _this.blocksToCompile) {
                if (!_this.items[ID])
                    continue;
                var neue = _this.compileBlock(ID);
                if (neue) {
                    adds.push(neue);
                    _this.blocks[ID] = neue;
                }
            }
            _this.programToInjectInto.blockChangeTransaction(adds, removes);
            _this.queued = false;
            _this.blocksToCompile = {};
            _this.blocksToRemove = {};
        };
        _this.compileValue = function (compile, context, value) {
            if (value === undefined)
                return undefined;
            var items = _this.items;
            if (items[value] && items[value].type === "variable") {
                var found = void 0;
                var cur = context;
                while (!found && cur) {
                    found = compile.variables[cur.ID] && compile.variables[cur.ID][value];
                    cur = cur.parent;
                }
                if (!found) {
                    if (!compile.variables[context.ID]) {
                        compile.variables[context.ID] = {};
                    }
                    found = compile.variables[context.ID][value] = new dsl2_1.Reference(context);
                }
                return found;
            }
            return value;
        };
        return _this;
    }
    CompilerWatcher.prototype.queue = function (blockID, isAdd) {
        if (isAdd === void 0) { isAdd = true; }
        if (isAdd)
            this.blocksToCompile[blockID] = true;
        this.blocksToRemove[blockID] = true;
        if (!this.queued) {
            this.queued = true;
            setImmediate(this.runQueue);
        }
    };
    //------------------------------------------------------------------
    // Program to inject into
    //------------------------------------------------------------------
    CompilerWatcher.prototype.injectInto = function (prog) {
        this.programToInjectInto = prog;
    };
    //------------------------------------------------------------------
    // Watch functions
    //------------------------------------------------------------------
    CompilerWatcher.prototype.registerWatcherFunction = function (name, consumer) {
        this.watcherFunctions[name] = consumer;
    };
    //------------------------------------------------------------------
    // Compiler
    //------------------------------------------------------------------
    CompilerWatcher.prototype.inContext = function (flow, func) {
        dsl2_1.ReferenceContext.push(flow.context);
        func();
        dsl2_1.ReferenceContext.pop();
    };
    CompilerWatcher.prototype.compileFlow = function (compile, flow, constraints) {
        var _this = this;
        var _a = this, inContext = _a.inContext, items = _a.items, compileValue = _a.compileValue;
        var context = flow.context;
        var subBlocks = [];
        var _loop_1 = function (constraintID) {
            var constraint = items[constraintID];
            if (!constraint)
                return "continue";
            if (constraint.type === "record") {
                inContext(flow, function () {
                    var attrs = {};
                    for (var _i = 0, _a = constraint.attributes; _i < _a.length; _i++) {
                        var _b = _a[_i], attribute = _b[0], value = _b[1];
                        var safeValue = compileValue(compile, context, value);
                        var found = attrs[attribute];
                        if (!found) {
                            found = attrs[attribute] = [];
                        }
                        found.push(safeValue);
                    }
                    var recordVar = compileValue(compile, context, constraint.record);
                    var record = new dsl2_1.Record(flow.context, [], attrs, recordVar);
                    recordVar.__owner = record;
                });
            }
            if (constraint.type === "output") {
                inContext(flow, function () {
                    var attrs = {};
                    for (var _i = 0, _a = constraint.attributes; _i < _a.length; _i++) {
                        var _b = _a[_i], attribute = _b[0], value = _b[1];
                        var safeValue = compileValue(compile, context, value);
                        var found = attrs[attribute];
                        if (!found) {
                            found = attrs[attribute] = [];
                        }
                        found.push(safeValue);
                    }
                    var outputType = dsl2_1.Insert;
                    var outputOp = "add";
                    if (constraint.outputType === "remove") {
                        outputType = dsl2_1.Remove;
                        outputOp = "remove";
                    }
                    var outputVar = compileValue(compile, context, constraint.record);
                    var output;
                    if (constraint.attributes.length > 0) {
                        output = new outputType(flow.context, [], attrs);
                        context.equality(output.reference(), outputVar);
                    }
                    else {
                        output = new outputType(flow.context, [], attrs, outputVar);
                    }
                    var record = output.reference();
                    for (var _c = 0, _d = constraint.nonIdentityAttribute; _c < _d.length; _c++) {
                        var _e = _d[_c], attribute = _e[0], value = _e[1];
                        record[outputOp](compileValue(compile, context, attribute), compileValue(compile, context, value));
                    }
                });
            }
            if (constraint.type === "removeRecord") {
                inContext(flow, function () {
                    var outputVar = compileValue(compile, context, constraint.record);
                    var output;
                    if (!outputVar.__owner) {
                        throw new Error("Trying to fully remove a record that doesn't exist");
                    }
                    outputVar.remove();
                });
            }
            if (constraint.type === "lookup") {
                inContext(flow, function () {
                    var lookup = flow.lookup(compileValue(compile, context, constraint.record));
                    context.equality(lookup.attribute, compileValue(compile, context, constraint.attribute));
                    context.equality(lookup.value, compileValue(compile, context, constraint.value));
                });
            }
            if (constraint.type === "expression") {
                inContext(flow, function () {
                    var args = constraint.args.map(function (v) { return compileValue(compile, context, v); });
                    var returns = constraint.returns.map(function (v) { return compileValue(compile, context, v); })[0];
                    var fn = new dsl2_1.Fn(flow.context, constraint.op, args, returns);
                });
            }
            if (constraint.type === "aggregate") {
                inContext(flow, function () {
                    var projection = [];
                    var group = [];
                    var args = [];
                    for (var arg in constraint.namedArgs) {
                        var values = constraint.namedArgs[arg].map(function (v) { return compileValue(compile, context, v); });
                        if (arg === "for" || arg === "given") {
                            projection = values;
                        }
                        else if (arg === "per") {
                            group = values;
                        }
                        else if (arg === "direction" || arg === "value") {
                            args = args.concat(values);
                        }
                    }
                    var aggOp = stdlib_1.SumAggregate;
                    if (constraint.op === "gather/sort") {
                        aggOp = Runtime.SortNode;
                    }
                    else if (constraint.op === "gather/count" && args.length === 0) {
                        args.push(1);
                    }
                    var returns = constraint.returns.map(function (v) { return compileValue(compile, context, v); })[0];
                    var agg = new dsl2_1.Aggregate(flow.context, aggOp, projection, group, args, returns);
                });
            }
            if (constraint.type === "equality") {
                inContext(flow, function () {
                    context.equality(compileValue(compile, context, constraint.left), compileValue(compile, context, constraint.right));
                });
            }
            if (constraint.type === "not" || constraint.type === "choose" || constraint.type === "union") {
                subBlocks.push(constraint);
            }
        };
        for (var _i = 0, constraints_1 = constraints; _i < constraints_1.length; _i++) {
            var constraintID = constraints_1[_i];
            _loop_1(constraintID);
        }
        var _loop_2 = function (constraint) {
            if (constraint.type === "not") {
                inContext(flow, function () {
                    var not = new dsl2_1.Not(function (a) { return []; }, flow);
                    _this.compileFlow(compile, not, constraint.constraints);
                });
            }
            if (constraint.type === "choose" || constraint.type == "union") {
                var builder_1 = constraint.type == "choose" ? dsl2_1.Choose : dsl2_1.Union;
                var branchFuncs_1 = [];
                var _loop_3 = function (branchId) {
                    var branch = items[branchId];
                    branchFuncs_1.push(function (self) {
                        return branch.outputs.map(function (v) {
                            return compileValue(compile, self.context, v);
                        });
                    });
                };
                for (var _i = 0, _a = constraint.branches; _i < _a.length; _i++) {
                    var branchId = _a[_i];
                    _loop_3(branchId);
                }
                inContext(flow, function () {
                    var outputs = constraint.outputs.map(function (v) {
                        return compileValue(compile, context, v);
                    });
                    var choose = new builder_1(flow.context, branchFuncs_1, flow, outputs);
                    var branchIx = 0;
                    var _loop_4 = function (branchId) {
                        var branch = items[branchId];
                        var compiled = choose.branches[branchIx];
                        inContext(compiled, function () {
                            _this.compileFlow(compile, compiled, branch.constraints);
                            choose.setBranchInputs(branchIx, compiled.context.getInputReferences());
                        });
                        branchIx++;
                    };
                    for (var _i = 0, _a = constraint.branches; _i < _a.length; _i++) {
                        var branchId = _a[_i];
                        _loop_4(branchId);
                    }
                });
            }
        };
        for (var _b = 0, subBlocks_1 = subBlocks; _b < subBlocks_1.length; _b++) {
            var constraint = subBlocks_1[_b];
            _loop_2(constraint);
        }
    };
    CompilerWatcher.prototype.compileBlock = function (blockID) {
        var _a = this, inContext = _a.inContext, items = _a.items, compileValue = _a.compileValue;
        var item = items[blockID];
        var name = item.name, constraints = item.constraints, type = item.type;
        var compile = { variables: {} };
        var flow;
        if (type === "commit") {
            flow = new dsl2_1.CommitFlow(function (a) { return []; });
        }
        else if (type === "watch") {
            flow = new dsl2_1.WatchFlow(function (a) { return []; });
        }
        else if (type === "bind") {
            flow = new dsl2_1.LinearFlow(function (a) { return []; });
        }
        else {
            return;
        }
        this.compileFlow(compile, flow, constraints);
        var block = this.programToInjectInto["_" + type](name, flow);
        if (type === "watch" && item.watcher) {
            var func = this.watcherFunctions[item.watcher];
            if (!func) {
                console.error("No such watcher function registered: " + item.watcher);
            }
            else {
                this.programToInjectInto.asDiffs(func);
            }
        }
        // if(typeof process === "undefined" && console.groupCollapsed) {
        //   console.groupCollapsed("Compiled: " + block.name);
        //   console.log(block, flow);
        //   console.log(printBlock(block));
        //   console.groupEnd();
        // }
        return block;
    };
    //------------------------------------------------------------------
    // Compile item extraction via watch blocks
    //------------------------------------------------------------------
    CompilerWatcher.prototype.setup = function () {
        var _this = this;
        var me = this.program;
        me.bind("show errors", function (_a) {
            var find = _a.find, record = _a.record;
            var err = find("eve/compiler/error");
            return [
                record("ui/column", { err: err, class: "eve-compiler-error" }).add("children", [
                    record("ui/row", { err: err, class: "eve-compiler-error-message-container" }).add("children", [
                        record("ui/column", { err: err, class: "eve-compiler-line-info", sort: 0 }).add("children", [
                            record("ui/text", { err: err, text: "Line", sort: 0 }),
                            record("ui/text", { err: err, class: "eve-compiler-error-message-line", text: err.start.line, sort: 1 }),
                        ]),
                        record("ui/column", { err: err, class: "eve-compiler-error-content", sort: 1 }).add("children", [
                            record("ui/text", { err: err, class: "eve-compiler-error-message", text: err.message, sort: 0 }),
                            record("ui/text", { err: err, class: "eve-compiler-error-sample", text: err.sample, sort: 1 })
                        ]),
                    ]),
                ])
            ];
        });
        me.watch("get blocks", function (_a) {
            var find = _a.find, record = _a.record;
            var block = find("eve/compiler/rule");
            var constraint = block.constraint, name = block.name, type = block.type;
            return [
                record({ block: block, constraint: constraint, name: name, type: type })
            ];
        });
        me.asObjects(function (_a) {
            var adds = _a.adds, removes = _a.removes;
            var items = _this.items;
            for (var key in adds) {
                var _b = adds[key], block = _b.block, name_1 = _b.name, constraint = _b.constraint, type = _b.type;
                var found = items[block];
                if (!found) {
                    found = items[block] = { type: type, name: name_1, constraints: [] };
                }
                found.name = name_1;
                if (found.constraints.indexOf(constraint) === -1) {
                    found.constraints.push(constraint);
                }
                _this.queue(block);
            }
            for (var key in removes) {
                var _c = removes[key], block = _c.block, name_2 = _c.name, constraint = _c.constraint;
                var found = items[block];
                if (!found) {
                    continue;
                }
                var ix = found.constraints.indexOf(constraint);
                if (ix > -1) {
                    found.constraints.splice(ix, 1);
                }
                if (found.constraints.length === 0) {
                    delete items[block];
                }
                _this.queue(block);
            }
        });
        me.watch("get nots", function (_a) {
            var find = _a.find, record = _a.record;
            var not = find("eve/compiler/not");
            var block = find("eve/compiler/block", { constraint: not });
            return [
                record({ block: block, not: not, constraint: not.constraint })
            ];
        });
        me.asObjects(function (_a) {
            var adds = _a.adds, removes = _a.removes;
            var items = _this.items;
            for (var key in adds) {
                var _b = adds[key], block = _b.block, not = _b.not, constraint = _b.constraint;
                var found = items[not];
                if (!found) {
                    found = items[not] = { type: "not", constraints: [] };
                }
                if (found.constraints.indexOf(constraint) === -1) {
                    found.constraints.push(constraint);
                }
                _this.queue(block);
            }
            for (var key in removes) {
                var _c = removes[key], block = _c.block, not = _c.not, constraint = _c.constraint;
                var found = items[not];
                if (!found) {
                    continue;
                }
                var ix = found.constraints.indexOf(constraint);
                if (ix > -1) {
                    found.constraints.splice(ix, 1);
                }
                if (found.constraints.length === 0) {
                    delete items[not];
                }
                _this.queue(block);
            }
        });
        me.watch("get choose branches", function (_a) {
            var find = _a.find, record = _a.record, choose = _a.choose;
            var item = find("eve/compiler/branch-set");
            var itemType = choose(function () {
                item.tag == "eve/compiler/choose";
                return ["choose"];
            }, function () {
                item.tag == "eve/compiler/union";
                return ["union"];
            })[0];
            var block = find("eve/compiler/rule", { constraint: item });
            return [
                record({ block: block, item: item, itemType: itemType, branch: item.branch })
            ];
        });
        me.asObjects(function (_a) {
            var adds = _a.adds, removes = _a.removes;
            var items = _this.items;
            for (var key in adds) {
                var _b = adds[key], block = _b.block, item = _b.item, branch = _b.branch, itemType = _b.itemType;
                var found = items[item];
                if (!found) {
                    found = items[item] = { type: itemType, branches: [], outputs: [] };
                }
                if (!items[branch]) {
                    items[branch] = { type: "branch", constraints: [], outputs: [] };
                }
                if (found.branches.indexOf(branch) === -1) {
                    found.branches.push(branch);
                }
                _this.queue(block);
            }
            for (var key in removes) {
                var _c = removes[key], block = _c.block, item = _c.item, itemType = _c.itemType, branch = _c.branch;
                var found = items[item];
                if (!found) {
                    continue;
                }
                var ix = found.branches.indexOf(branch);
                if (ix > -1) {
                    found.branches.splice(ix, 1);
                }
                if (found.branches.length === 0) {
                    delete items[item];
                }
                _this.queue(block);
            }
        });
        me.watch("get choose outputs", function (_a) {
            var find = _a.find, record = _a.record;
            var choose = find("eve/compiler/branch-set");
            var block = find("eve/compiler/block", { constraint: choose });
            var _b = choose.output, value = _b.value, index = _b.index;
            return [
                record({ block: block, choose: choose, value: value, index: index })
            ];
        });
        me.asObjects(function (_a) {
            var adds = _a.adds, removes = _a.removes;
            var items = _this.items;
            for (var key in adds) {
                var _b = adds[key], block = _b.block, choose = _b.choose, value = _b.value, index = _b.index;
                var found = items[choose];
                if (!found) {
                    console.error("adding output for a branch that doesn't exist");
                    return;
                }
                found.outputs[index - 1] = value;
                _this.queue(block);
            }
            for (var key in removes) {
                var _c = removes[key], block = _c.block, choose = _c.choose, value = _c.value, index = _c.index;
                var found = items[choose];
                if (!found) {
                    continue;
                }
                var cur = found.outputs[index];
                if (cur === value) {
                    found.outputs[index - 1] = undefined;
                }
                _this.queue(block);
            }
        });
        me.watch("get choose branch constraints", function (_a) {
            var find = _a.find, record = _a.record;
            var choose = find("eve/compiler/branch-set");
            var block = find("eve/compiler/block", { constraint: choose });
            var branch = choose.branch;
            return [
                record({ block: block, branch: branch, constraint: branch.constraint })
            ];
        });
        me.asObjects(function (_a) {
            var adds = _a.adds, removes = _a.removes;
            var items = _this.items;
            for (var key in adds) {
                var _b = adds[key], block = _b.block, constraint = _b.constraint, branch = _b.branch;
                var found = items[branch];
                if (!found) {
                    console.error("adding constraint for a branch that doesn't exist");
                    return;
                }
                if (found.constraints.indexOf(constraint) === -1) {
                    found.constraints.push(constraint);
                }
                _this.queue(block);
            }
            for (var key in removes) {
                var _c = removes[key], block = _c.block, constraint = _c.constraint, branch = _c.branch;
                var found = items[branch];
                if (!found) {
                    continue;
                }
                var ix = found.constraints.indexOf(constraint);
                if (ix > -1) {
                    found.constraints.splice(ix, 1);
                }
                if (found.constraints.length === 0) {
                    delete items[branch];
                }
                _this.queue(block);
            }
        });
        me.watch("get choose branch outputs", function (_a) {
            var find = _a.find, record = _a.record;
            var choose = find("eve/compiler/branch-set");
            var block = find("eve/compiler/block", { constraint: choose });
            var branch = choose.branch;
            var _b = branch.output, value = _b.value, index = _b.index;
            return [
                record({ block: block, branch: branch, value: value, index: index })
            ];
        });
        me.asObjects(function (_a) {
            var adds = _a.adds, removes = _a.removes;
            var items = _this.items;
            for (var key in adds) {
                var _b = adds[key], block = _b.block, branch = _b.branch, value = _b.value, index = _b.index;
                var found = items[branch];
                if (!found) {
                    console.error("adding output for a branch that doesn't exist");
                    return;
                }
                found.outputs[index - 1] = value;
                _this.queue(block);
            }
            for (var key in removes) {
                var _c = removes[key], block = _c.block, branch = _c.branch, value = _c.value, index = _c.index;
                var found = items[branch];
                if (!found) {
                    continue;
                }
                var cur = found.outputs[index];
                if (cur === value) {
                    found.outputs[index - 1] = undefined;
                }
                _this.queue(block);
            }
        });
        me.watch("get watcher property", function (_a) {
            var find = _a.find, record = _a.record;
            var block = find("eve/compiler/rule");
            var constraint = block.constraint, name = block.name, type = block.type, watcher = block.watcher;
            return [
                record({ block: block, watcher: watcher })
            ];
        });
        me.asObjects(function (_a) {
            var adds = _a.adds, removes = _a.removes;
            var items = _this.items;
            for (var key in adds) {
                var _b = adds[key], block = _b.block, watcher = _b.watcher;
                var found = items[block];
                found.watcher = watcher;
                _this.queue(block);
            }
            for (var key in removes) {
                var _c = removes[key], block = _c.block, watcher = _c.watcher;
                var found = items[block];
                if (!found) {
                    continue;
                }
                found.watcher = undefined;
                _this.queue(block);
            }
        });
        me.watch("get variables", function (_a) {
            var find = _a.find, record = _a.record;
            var variable = find("eve/compiler/var");
            return [
                record({ variable: variable })
            ];
        });
        me.asObjects(function (_a) {
            var adds = _a.adds, removes = _a.removes;
            var items = _this.items;
            for (var key in adds) {
                var variable = adds[key].variable;
                items[variable] = { type: "variable" };
            }
        });
        me.watch("get equalities", function (_a) {
            var find = _a.find, record = _a.record;
            var eq = find("eve/compiler/equality");
            return [
                record({ eq: eq, left: eq.left, right: eq.right })
            ];
        });
        me.asObjects(function (_a) {
            var adds = _a.adds, removes = _a.removes;
            var items = _this.items;
            for (var key in adds) {
                var _b = adds[key], eq = _b.eq, left = _b.left, right = _b.right;
                items[eq] = { type: "equality", left: left, right: right };
            }
            for (var key in removes) {
                var eq = removes[key].eq;
                items[eq] = undefined;
            }
        });
        me.watch("get lookups", function (_a) {
            var find = _a.find, record = _a.record;
            var lookup = find("eve/compiler/lookup");
            var block = find("eve/compiler/block", { constraint: lookup });
            var rec = lookup.record, attribute = lookup.attribute, value = lookup.value;
            return [
                record({ block: block, id: lookup, record: rec, attribute: attribute, value: value })
            ];
        });
        me.asObjects(function (_a) {
            var adds = _a.adds, removes = _a.removes;
            var items = _this.items;
            for (var key in adds) {
                var _b = adds[key], block = _b.block, id = _b.id, record = _b.record, attribute = _b.attribute, value = _b.value;
                items[id] = { type: "lookup", record: record, attribute: attribute, value: value };
                _this.queue(block);
            }
            for (var key in removes) {
                var _c = removes[key], block = _c.block, id = _c.id, record = _c.record, attribute = _c.attribute, value = _c.value;
                delete items[id];
                _this.queue(block);
            }
        });
        me.watch("get expressions", function (_a) {
            var find = _a.find, record = _a.record, choose = _a.choose;
            var expr = find("eve/compiler/expression");
            var type = choose(function () {
                expr.tag == "eve/compiler/aggregate";
                return ["aggregate"];
            }, function () {
                return ["expression"];
            })[0];
            var block = find("eve/compiler/block", { constraint: expr });
            return [
                record({ block: block, id: expr, op: expr.op, type: type })
            ];
        });
        me.asObjects(function (_a) {
            var adds = _a.adds, removes = _a.removes;
            var items = _this.items;
            for (var key in adds) {
                var _b = adds[key], block = _b.block, id = _b.id, op = _b.op, type = _b.type;
                var found = items[id];
                if (!found) {
                    found = items[id] = { type: type, op: op, args: [], returns: [], namedArgs: {} };
                }
                found.op = op;
                _this.queue(block);
            }
            for (var key in removes) {
                var _c = removes[key], block = _c.block, id = _c.id, op = _c.op;
                var found = items[id];
                if (!found) {
                    continue;
                }
                delete items[id];
                _this.queue(block);
            }
        });
        me.watch("get expression args", function (_a) {
            var find = _a.find, record = _a.record;
            var expr = find("eve/compiler/expression");
            var block = find("eve/compiler/block", { constraint: expr });
            var arg = expr.arg;
            return [
                record({ block: block, id: expr, index: arg.index, value: arg.value })
            ];
        });
        me.asObjects(function (_a) {
            var adds = _a.adds, removes = _a.removes;
            var items = _this.items;
            for (var key in adds) {
                var _b = adds[key], block = _b.block, id = _b.id, index = _b.index, value = _b.value;
                var found = items[id];
                if (!found) {
                    throw new Error("args for a non existent expression");
                }
                found.args[index - 1] = value;
                _this.queue(block);
            }
            for (var key in removes) {
                var _c = removes[key], block = _c.block, id = _c.id, index = _c.index, value = _c.value;
                var found = items[id];
                if (!found) {
                    continue;
                }
                if (found.args[index - 1] == value) {
                    found.args[index - 1] = undefined;
                }
                _this.queue(block);
            }
        });
        me.watch("get expression named args", function (_a) {
            var find = _a.find, record = _a.record, not = _a.not;
            var expr = find("eve/compiler/expression");
            not(function () {
                expr.tag == "eve/compiler/aggregate";
            });
            var block = find("eve/compiler/block", { constraint: expr });
            var arg = expr.arg;
            return [
                record({ block: block, id: expr, name: arg.name, value: arg.value, index: arg.index })
            ];
        });
        me.asObjects(function (_a) {
            var adds = _a.adds, removes = _a.removes;
            var items = _this.items;
            for (var key in adds) {
                var _b = adds[key], block = _b.block, id = _b.id, name_3 = _b.name, value = _b.value, index = _b.index;
                var found = items[id];
                if (!found) {
                    throw new Error("args for a non existent expression");
                }
                var _c = runtime_1.FunctionConstraint.fetchInfo(found.op), argNames = _c.argNames, returnNames = _c.returnNames;
                var argIx = argNames.indexOf(name_3);
                var retIx = returnNames.indexOf(name_3);
                if (argIx > -1) {
                    found.args[argIx] = value;
                }
                else if (retIx > -1) {
                    found.returns[retIx] = value;
                }
                else {
                    console.error("Unknown arg for expression: " + found.op + "[" + name_3 + "]");
                }
                _this.queue(block);
            }
            for (var key in removes) {
                var _d = removes[key], block = _d.block, id = _d.id, name_4 = _d.name, value = _d.value;
                var found = items[id];
                if (!found) {
                    continue;
                }
                var _e = runtime_1.FunctionConstraint.fetchInfo(found.op), argNames = _e.argNames, returnNames = _e.returnNames;
                var argIx = argNames.indexOf(name_4);
                var retIx = returnNames.indexOf(name_4);
                if (argIx > -1) {
                    found.args[argIx] = undefined;
                }
                else if (retIx > -1) {
                    found.returns[retIx] = undefined;
                }
                else {
                    console.error("Unknown arg for expression: " + found.op + "[" + name_4 + "]");
                }
                _this.queue(block);
            }
        });
        me.watch("get aggregate named args", function (_a) {
            var find = _a.find, record = _a.record, not = _a.not;
            var expr = find("eve/compiler/aggregate");
            var block = find("eve/compiler/block", { constraint: expr });
            var arg = expr.arg;
            return [
                record({ block: block, id: expr, name: arg.name, value: arg.value, index: arg.index })
            ];
        });
        me.asObjects(function (_a) {
            var adds = _a.adds, removes = _a.removes;
            var items = _this.items;
            for (var key in adds) {
                var _b = adds[key], block = _b.block, id = _b.id, name_5 = _b.name, value = _b.value, index = _b.index;
                var found = items[id];
                if (!found) {
                    throw new Error("args for a non existent expression");
                }
                var args = found.namedArgs[name_5] || [];
                args[index - 1] = value;
                found.namedArgs[name_5] = args;
                _this.queue(block);
            }
            for (var key in removes) {
                var _c = removes[key], block = _c.block, id = _c.id, name_6 = _c.name, value = _c.value;
                var found = items[id];
                if (!found) {
                    continue;
                }
                var args = found.namedArgs[name_6];
                if (args) {
                    var ix = args.indexOf(value);
                    args.splice(ix, 1);
                }
                _this.queue(block);
            }
        });
        me.watch("get expression returns", function (_a) {
            var find = _a.find, record = _a.record;
            var expr = find("eve/compiler/expression");
            var block = find("eve/compiler/block", { constraint: expr });
            var ret = expr.return;
            return [
                record({ block: block, id: expr, index: ret.index, value: ret.value })
            ];
        });
        me.asObjects(function (_a) {
            var adds = _a.adds, removes = _a.removes;
            var items = _this.items;
            for (var key in adds) {
                var _b = adds[key], block = _b.block, id = _b.id, index = _b.index, value = _b.value;
                var found = items[id];
                if (!found) {
                    throw new Error("returns for a non existent expression");
                }
                found.returns[index - 1] = value;
                _this.queue(block);
            }
            for (var key in removes) {
                var _c = removes[key], block = _c.block, id = _c.id, index = _c.index, value = _c.value;
                var found = items[id];
                if (!found) {
                    continue;
                }
                if (found.returns[index - 1] == value) {
                    found.returns[index - 1] = undefined;
                }
                _this.queue(block);
            }
        });
        me.watch("get records", function (_a) {
            var find = _a.find, record = _a.record;
            var compilerRecord = find("eve/compiler/record");
            var block = find("eve/compiler/block", { constraint: compilerRecord });
            var id = compilerRecord.record, attribute = compilerRecord.attribute;
            return [
                record({ block: block, id: compilerRecord, record: id, attribute: attribute.attribute, value: attribute.value })
            ];
        });
        me.asObjects(function (_a) {
            var adds = _a.adds, removes = _a.removes;
            var items = _this.items;
            for (var key in adds) {
                var _b = adds[key], block = _b.block, id = _b.id, record = _b.record, attribute = _b.attribute, value = _b.value;
                var found = items[id];
                if (!found) {
                    found = items[id] = { type: "record", attributes: [], record: record };
                }
                found.attributes.push([attribute, value]);
                _this.queue(block);
            }
            var _loop_5 = function (key) {
                var _a = removes[key], block = _a.block, id = _a.id, record = _a.record, attribute = _a.attribute, value = _a.value;
                var found = items[id];
                if (!found) {
                    return "continue";
                }
                found.attributes = found.attributes.filter(function (_a) {
                    var a = _a[0], v = _a[1];
                    return a !== attribute || v !== value;
                });
                if (found.attributes.length === 0) {
                    delete items[id];
                }
                _this.queue(block);
            };
            for (var key in removes) {
                _loop_5(key);
            }
        });
        me.watch("get outputs", function (_a) {
            var find = _a.find, record = _a.record, choose = _a.choose;
            var compilerRecord = find("eve/compiler/output");
            var block = find("eve/compiler/block", { constraint: compilerRecord });
            var id = compilerRecord.record, attribute = compilerRecord.attribute;
            var attributeType = choose(function () {
                attribute.tag == "eve/compiler/attribute/non-identity";
                return "non-identity";
            }, function () {
                return "identity";
            })[0];
            var outputType = choose(function () {
                compilerRecord.tag == "eve/compiler/remove";
                return "remove";
            }, function () {
                return "add";
            })[0];
            return [
                record({ block: block, id: compilerRecord, record: id, attribute: attribute.attribute, value: attribute.value, attributeType: attributeType, outputType: outputType })
            ];
        });
        me.asObjects(function (_a) {
            var adds = _a.adds, removes = _a.removes;
            var items = _this.items;
            for (var key in adds) {
                var _b = adds[key], block = _b.block, id = _b.id, record = _b.record, attribute = _b.attribute, value = _b.value, attributeType = _b.attributeType, outputType = _b.outputType;
                var found = items[id];
                if (!found) {
                    found = items[id] = { type: "output", attributes: [], nonIdentityAttribute: [], record: record, outputType: outputType };
                }
                if (attributeType === "identity") {
                    found.attributes.push([attribute, value]);
                }
                else {
                    found.nonIdentityAttribute.push([attribute, value]);
                }
                _this.queue(block);
            }
            var _loop_6 = function (key) {
                var _a = removes[key], block = _a.block, id = _a.id, record = _a.record, attribute = _a.attribute, value = _a.value;
                var found = items[id];
                if (!found) {
                    return "continue";
                }
                found.attributes = found.attributes.filter(function (_a) {
                    var a = _a[0], v = _a[1];
                    return a !== attribute || v !== value;
                });
                found.nonIdentityAttribute = found.nonIdentityAttribute.filter(function (_a) {
                    var a = _a[0], v = _a[1];
                    return a !== attribute || v !== value;
                });
                if (found.attributes.length === 0) {
                    delete items[id];
                }
                _this.queue(block);
            };
            for (var key in removes) {
                _loop_6(key);
            }
        });
        me.watch("get valueless outputs", function (_a) {
            var find = _a.find, record = _a.record, choose = _a.choose, not = _a.not;
            var compilerRecord = find("eve/compiler/output");
            var block = find("eve/compiler/block", { constraint: compilerRecord });
            var attribute = compilerRecord.attribute;
            not(function () { return attribute.value; });
            return [
                record({ block: block, id: compilerRecord, record: compilerRecord.record, attribute: attribute.attribute })
            ];
        });
        me.asObjects(function (_a) {
            var adds = _a.adds, removes = _a.removes;
            var items = _this.items;
            for (var key in adds) {
                var _b = adds[key], block = _b.block, id = _b.id, record = _b.record, attribute = _b.attribute;
                var found = items[id];
                if (!found) {
                    found = items[id] = { type: "output", attributes: [], nonIdentityAttribute: [], record: record, outputType: "remove" };
                }
                found.nonIdentityAttribute.push([attribute, undefined]);
                _this.queue(block);
            }
            var _loop_7 = function (key) {
                var _a = removes[key], block = _a.block, id = _a.id, attribute = _a.attribute;
                var found = items[id];
                if (!found) {
                    return "continue";
                }
                found.nonIdentityAttribute = found.nonIdentityAttribute.filter(function (_a) {
                    var a = _a[0], v = _a[1];
                    return a !== attribute || v !== undefined;
                });
                if (found.nonIdentityAttribute.length === 0) {
                    delete items[id];
                }
                _this.queue(block);
            };
            for (var key in removes) {
                _loop_7(key);
            }
        });
        me.watch("get full remove", function (_a) {
            var find = _a.find, record = _a.record, choose = _a.choose, not = _a.not;
            var compilerRecord = find("eve/compiler/remove");
            var block = find("eve/compiler/block", { constraint: compilerRecord });
            not(function () { return compilerRecord.attribute; });
            return [
                record({ block: block, id: compilerRecord, record: compilerRecord.record })
            ];
        });
        me.asObjects(function (_a) {
            var adds = _a.adds, removes = _a.removes;
            var items = _this.items;
            for (var key in adds) {
                var _b = adds[key], block = _b.block, id = _b.id, record = _b.record;
                var found = items[id];
                if (!found) {
                    found = items[id] = { type: "removeRecord", record: record };
                }
                _this.queue(block);
            }
            for (var key in removes) {
                var _c = removes[key], block = _c.block, id = _c.id;
                var found = items[id];
                if (!found) {
                    continue;
                }
                delete items[id];
                _this.queue(block);
            }
        });
    };
    return CompilerWatcher;
}(watcher_1.Watcher));
exports.CompilerWatcher = CompilerWatcher;
watcher_1.Watcher.register("compiler", CompilerWatcher);
//# sourceMappingURL=compiler.js.map