"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var watcher_1 = require("./watcher");
var naturalSort = require("javascript-natural-sort");
var DOMWatcher = (function (_super) {
    __extends(DOMWatcher, _super);
    function DOMWatcher() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.styles = Object.create(null);
        _this.roots = Object.create(null);
        _this.instances = Object.create(null);
        _this.elementToInstances = Object.create(null);
        _this.styleToInstances = Object.create(null);
        return _this;
    }
    DOMWatcher.prototype._sendEvent = function (eavs) {
        this.program.inputEAVs(eavs);
    };
    DOMWatcher.prototype.getStyle = function (id) {
        return this.styles[id] = this.styles[id] || { __size: 0 };
    };
    DOMWatcher.prototype.isInstance = function (elem) {
        if (!elem || !(elem instanceof Element))
            return false;
        var instance = elem;
        return instance && !!instance["__element"];
    };
    DOMWatcher.prototype.addInstance = function (id, element, tagname) {
        var instance = this.instances[id] = this.createInstance(id, element, tagname);
        if (!this.elementToInstances[element])
            this.elementToInstances[element] = [];
        this.elementToInstances[element].push(id);
        return instance;
    };
    DOMWatcher.prototype.clearInstance = function (id) {
        var instance = this.instances[id];
        if (instance && instance.parentElement) {
            instance.parentElement.removeChild(instance);
        }
        this.instances[id] = undefined;
        var instances = instance && this.elementToInstances[instance.__element];
        if (instances)
            instances.splice(instances.indexOf(id), 1);
    };
    DOMWatcher.prototype.getRoot = function (id, tagname) {
        if (tagname === void 0) { tagname = "div"; }
        return this.roots[id] = this.roots[id];
    };
    DOMWatcher.prototype.clearRoot = function (id) {
        this.clearInstance(id);
        this.roots[id] = undefined;
    };
    DOMWatcher.prototype.insertChild = function (parent, child, at) {
        if (at === void 0) { at = child.__sort; }
        child.__sort = at;
        if (at !== undefined)
            child.setAttribute("sort", "" + at);
        if (!parent)
            return;
        var current;
        for (var curIx = 0; curIx < parent.childNodes.length; curIx++) {
            var cur = parent.childNodes[curIx];
            if (cur === child)
                continue;
            if (cur.__sort !== undefined && at !== undefined && naturalSort(cur.__sort, at) > 0) {
                current = cur;
                break;
            }
        }
        if (current) {
            parent.insertBefore(child, current);
        }
        else {
            parent.appendChild(child);
        }
    };
    // @NOTE: This requires styles to have disjoint attribute sets or it'll do bad things.
    // @NOTE: Styles may only have a single value for each attribute due to our inability
    //        to express an ordering of non-record values.
    DOMWatcher.prototype.setStyleAttribute = function (styleId, attribute, value, count) {
        var style = this.getStyle(styleId);
        if (count === -1) {
            //if(!style[attribute]) throw new Error(`Cannot remove non-existent attribute '${attribute}'`);
            //if(style[attribute] !== value) throw new Error(`Cannot remove mismatched AV ${attribute}: ${value} (current: ${style[attribute]})`);
            style[attribute] = undefined;
        }
        else {
            if (style[attribute])
                throw new Error("Cannot add already present attribute '" + attribute + "'");
            style[attribute] = value;
        }
        style.__size += count;
        // Update all existing instances with this style.
        var instances = this.styleToInstances[styleId];
        if (instances) {
            for (var _i = 0, instances_1 = instances; _i < instances_1.length; _i++) {
                var instanceId = instances_1[_i];
                var instance = this.getInstance(instanceId);
                if (!instance) {
                    // We may have removed one instance of multiple subscribed to this style.
                    continue;
                }
                instance.style[attribute] = style[attribute];
            }
        }
    };
    DOMWatcher.prototype.addStyleInstance = function (styleId, instanceId) {
        var instance = this.getInstance(instanceId);
        if (!instance)
            throw new Error("Orphaned instance '" + instanceId + "'");
        var style = this.getStyle(styleId);
        for (var prop in style) {
            if (prop === "__size")
                continue;
            instance.style[prop] = style[prop];
        }
        if (this.styleToInstances[styleId])
            this.styleToInstances[styleId].push(instanceId);
        else
            this.styleToInstances[styleId] = [instanceId];
        if (!instance.__styles)
            instance.__styles = [];
        if (instance.__styles.indexOf(styleId) === -1)
            instance.__styles.push(styleId);
    };
    DOMWatcher.prototype.removeStyleInstance = function (styleId, instanceId) {
        var instance = this.getInstance(instanceId);
        if (!instance)
            return;
        instance.removeAttribute("style");
        var ix = instance.__styles.indexOf(styleId);
        instance.__styles.splice(ix, 1);
        for (var _i = 0, _a = instance.__styles; _i < _a.length; _i++) {
            var otherStyleId = _a[_i];
            var style = this.getStyle(otherStyleId);
            for (var prop in style) {
                if (prop === "__size")
                    continue;
                instance.style[prop] = style[prop];
            }
        }
    };
    DOMWatcher.prototype.setup = function () {
        var _this = this;
        if (typeof document === "undefined")
            return;
        this.program
            .constants({ tagPrefix: this.tagPrefix })
            .commit("Remove click events!", function (_a) {
            var find = _a.find;
            var click = find("{{tagPrefix}}/event/click");
            return [click.remove()];
        })
            .bind("Create instances for each root.", function (_a) {
            var find = _a.find, record = _a.record, lib = _a.lib;
            var elem = find("{{tagPrefix}}/root");
            return [
                record("{{tagPrefix}}/instance", { element: elem, tagname: elem.tagname })
            ];
        })
            .bind("Create an instance for each child of a rooted parent.", function (_a) {
            var find = _a.find, record = _a.record, lib = _a.lib;
            var elem = find("{{tagPrefix}}/element");
            var parentElem = find("{{tagPrefix}}/element", { children: elem });
            var parent = find("{{tagPrefix}}/instance", { element: parentElem });
            return [
                record("{{tagPrefix}}/instance", { element: elem, tagname: elem.tagname, parent: parent })
            ];
        })
            .watch("Export all instances.", function (_a) {
            var find = _a.find, record = _a.record;
            var instance = find("{{tagPrefix}}/instance");
            return [
                record({ tagname: instance.tagname, element: instance.element, instance: instance })
            ];
        })
            .asObjects(function (diff) {
            for (var _i = 0, _a = Object.keys(diff.removes); _i < _a.length; _i++) {
                var e = _a[_i];
                var instanceId = diff.removes[e].instance;
                _this.clearInstance(instanceId);
            }
            for (var _b = 0, _c = Object.keys(diff.adds); _b < _c.length; _b++) {
                var e = _c[_b];
                var _d = diff.adds[e], instanceId = _d.instance, tagname = _d.tagname, element = _d.element;
                _this.addInstance(instanceId, element, tagname);
            }
        })
            .watch("Export roots.", function (_a) {
            var find = _a.find, record = _a.record;
            var root = find("{{tagPrefix}}/root");
            var instance = find("{{tagPrefix}}/instance", { element: root });
            return [
                record({ instance: instance })
            ];
        })
            .asDiffs(function (diff) {
            for (var _i = 0, _a = diff.removes; _i < _a.length; _i++) {
                var _b = _a[_i], e = _b[0], a = _b[1], rootId = _b[2];
                _this.clearRoot(rootId);
            }
            for (var _c = 0, _d = diff.adds; _c < _d.length; _c++) {
                var _e = _d[_c], e = _e[0], a = _e[1], rootId = _e[2];
                _this.roots[rootId] = _this.createRoot(rootId);
            }
        })
            .watch("Export instance parents.", function (_a) {
            var find = _a.find, record = _a.record;
            var instance = find("{{tagPrefix}}/instance");
            return [
                record({ instance: instance, parent: instance.parent })
            ];
        })
            .asObjects(function (diff) {
            for (var _i = 0, _a = Object.keys(diff.removes); _i < _a.length; _i++) {
                var e = _a[_i];
                var _b = diff.removes[e], instanceId = _b.instance, parentId = _b.parent;
                var instance = _this.getInstance(instanceId);
                var parent_1 = _this.getInstance(parentId);
                if (!instance || !parent_1)
                    continue;
                if (instance && instance.parentElement) {
                    instance.parentElement.removeChild(instance);
                }
            }
            for (var _c = 0, _d = Object.keys(diff.adds); _c < _d.length; _c++) {
                var e = _d[_c];
                var _e = diff.adds[e], instanceId = _e.instance, parentId = _e.parent;
                var instance = _this.getInstance(instanceId);
                if (!instance)
                    throw new Error("Orphaned instance '" + instanceId + "'");
                var parent_2 = _this.getInstance(parentId);
                if (!parent_2)
                    throw new Error("Missing parent instance '" + parentId + "', " + instanceId);
                _this.insertChild(parent_2, instance);
            }
        })
            .watch("Export element styles.", function (_a) {
            var find = _a.find, record = _a.record, lib = _a.lib, lookup = _a.lookup;
            var elem = find("{{tagPrefix}}/element");
            var style = elem.style;
            var _b = lookup(style), attribute = _b.attribute, value = _b.value;
            return [
                style.add(attribute, value)
            ];
        })
            .asDiffs(function (diff) {
            var maybeGC = [];
            for (var _i = 0, _a = diff.removes; _i < _a.length; _i++) {
                var _b = _a[_i], styleId = _b[0], a = _b[1], v = _b[2];
                maybeGC.push(styleId);
                _this.setStyleAttribute(styleId, a, v, -1);
            }
            for (var _c = 0, _d = diff.adds; _c < _d.length; _c++) {
                var _e = _d[_c], styleId = _e[0], a = _e[1], v = _e[2];
                _this.setStyleAttribute(styleId, a, v, 1);
            }
            for (var _f = 0, maybeGC_1 = maybeGC; _f < maybeGC_1.length; _f++) {
                var styleId = maybeGC_1[_f];
                var style = _this.getStyle(styleId);
                if (style.__size === 0) {
                    _this.styles[styleId] = undefined;
                }
            }
        })
            .watch("Export element attributes.", function (_a) {
            var find = _a.find, record = _a.record, lookup = _a.lookup;
            var instance = find("{{tagPrefix}}/instance");
            var elem = instance.element;
            var _b = lookup(elem), attribute = _b.attribute, value = _b.value;
            attribute != "class";
            return [
                instance.add(attribute, value)
            ];
        })
            .asDiffs(function (diff) {
            for (var _i = 0, _a = diff.removes; _i < _a.length; _i++) {
                var _b = _a[_i], e = _b[0], a = _b[1], v = _b[2];
                var instance = _this.getInstance(e);
                if (!instance)
                    continue;
                if (a === "tagname")
                    continue;
                else if (a === "children")
                    continue;
                else if (a === "sort")
                    continue; // I guess..?
                else if (a === "eve-auto-index")
                    continue; // I guess..?
                else if (a === "text")
                    instance.textContent = null;
                else if (a === "style")
                    _this.removeStyleInstance(v, e);
                else
                    _this.removeAttribute(instance, a, watcher_1.asJS(v));
            }
            for (var _c = 0, _d = diff.adds; _c < _d.length; _c++) {
                var _e = _d[_c], e = _e[0], a = _e[1], v = _e[2];
                var instance = _this.getInstance(e);
                if (!instance)
                    throw new Error("Orphaned instance '" + e + "'");
                else if ((a === "tagname"))
                    continue;
                else if (a === "children")
                    continue;
                else if (a === "sort")
                    _this.insertChild(instance.parentElement, instance, v);
                else if (a === "eve-auto-index")
                    _this.insertChild(instance.parentElement, instance, v);
                else if (a === "text")
                    instance.textContent = "" + v;
                else if (a === "style")
                    _this.addStyleInstance(v, e);
                else
                    _this.addAttribute(instance, a, watcher_1.asJS(v));
            }
        })
            .watch("Export static classes.", function (_a) {
            var find = _a.find, not = _a.not, lookup = _a.lookup;
            var instance = find("{{tagPrefix}}/instance");
            var elem = instance.element;
            var klass = elem.class;
            not(function () { return lookup(klass); });
            return [instance.add("class", klass)];
        })
            .asDiffs(function (diff) {
            for (var _i = 0, _a = diff.removes; _i < _a.length; _i++) {
                var _b = _a[_i], e = _b[0], a = _b[1], v = _b[2];
                var instance = _this.getInstance(e);
                if (!instance)
                    continue;
                for (var _c = 0, _d = ("" + v).split(" "); _c < _d.length; _c++) {
                    var klass = _d[_c];
                    if (!klass)
                        continue;
                    instance.classList.remove(klass);
                }
            }
            for (var _e = 0, _f = diff.adds; _e < _f.length; _e++) {
                var _g = _f[_e], e = _g[0], a = _g[1], v = _g[2];
                var instance = _this.getInstance(e);
                if (!instance)
                    throw new Error("Orphaned instance '" + e + "'");
                for (var _h = 0, _j = ("" + v).split(" "); _h < _j.length; _h++) {
                    var klass = _j[_h];
                    if (!klass)
                        continue;
                    instance.classList.add(klass);
                }
            }
        })
            .bind("Elements with a dynamic class record apply classes for each true attribute.", function (_a) {
            var find = _a.find, lookup = _a.lookup, record = _a.record;
            var element = find("{{tagPrefix}}/element");
            var _b = lookup(element.class), attribute = _b.attribute, value = _b.value;
            value == "true";
            return [
                element.add("class", attribute)
            ];
        });
    };
    return DOMWatcher;
}(watcher_1.Watcher));
exports.DOMWatcher = DOMWatcher;
//# sourceMappingURL=dom.js.map