"use strict";
//--------------------------------------------------------------------
// Editor
//--------------------------------------------------------------------
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var watcher_1 = require("../watchers/watcher");
var EditorWatcher = (function (_super) {
    __extends(EditorWatcher, _super);
    function EditorWatcher() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    EditorWatcher.prototype.setup = function () {
        this.editor = this.createEditor();
        var _a = this, editor = _a.editor, program = _a.program;
        editor
            .bind("Draw the root editor view.", function (_a) {
            var find = _a.find, record = _a.record;
            var editor = find("editor/root");
            return [
                record("editor/view", "ui/row", { editor: editor }).add("children", [
                    record("editor/nav", "ui/column", { editor: editor, sort: 0 }),
                    record("editor/main", "ui/column", { editor: editor, sort: 1 }).add("children", [
                        record("ui/row", { editor: editor, sort: 0, class: "editor-block-header" }).add("children", [
                            record("editor/block/description", "ui/column", { editor: editor }),
                            record("editor/block/storyboard", "ui/row", { editor: editor })
                        ]),
                        record("ui/row", "editor/block/content", { editor: editor, sort: 1 })
                    ])
                ])
            ];
        })
            .bind("Attach the current frame type to the editor content window.", function (_a) {
            var find = _a.find;
            var editor = find("editor/root");
            var active_frame = editor.active_frame;
            var content = find("editor/block/content", { editor: editor });
            return [content.add("type", active_frame.type)];
        })
            .bind("A block's next node sort is it's max node sort + 1 (or 1).", function (_a) {
            var find = _a.find, choose = _a.choose, gather = _a.gather;
            var block = find("block");
            // @NOTE: We can only reliably use an aggregate in a choose if the choose inputs are *only* used in the aggregate grouping.
            var sort = choose(function () {
                var node = block.node;
                1 == gather(node.sort, node).per(block).sort("down");
                return node.sort + 1;
            }, function () { return 1; })[0];
            return [block.add("next_node_sort", sort)];
        })
            .bind("A node is another node's parent if it has an AV who's V is the other node's entity", function (_a) {
            var find = _a.find;
            var parent = find("node");
            var node = find("node");
            node != parent;
            var attribute = parent.attribute;
            attribute.value == node.entity;
            return [node.add({ parent: parent, parent_field: attribute.attribute })];
        })
            .bind("Mark nodes without parents as root nodes.", function (_a) {
            var find = _a.find, not = _a.not;
            var node = find("node");
            not(function () { return node.parent; });
            return [node.add("tag", "root-node")];
        })
            .commit("A node with no entity creates one.", function (_a) {
            var find = _a.find, not = _a.not, record = _a.record;
            var node = find("node");
            not(function () { return node.entity; });
            return [node.add("entity", record("entity", { node: node, sort: node.sort }))];
        })
            .commit("If a node's attribute is a record and it's not already a subnode, fix that.", function (_a) {
            var find = _a.find, not = _a.not, record = _a.record;
            var editor = find("editor/root");
            var active_block = editor.active_block;
            var node = active_block.node;
            var attribute = node.attribute;
            find("editor/existing-node-attribute", { node: node, text: attribute.attribute, is_record: "true" });
            not(function () { return attribute.value == find("node").entity; });
            var sort = active_block.next_node_sort;
            var subnode;
            return [
                active_block.add("node", [
                    subnode = record("node", "derived-subnode", { sort: sort, entity: record("entity", { sort: sort, _node: node, _attr: attribute.attribute }) })
                ]),
                attribute.remove("value").add("value", subnode.entity)
            ];
        })
            .commit("A node is only derived once.", function (_a) {
            var find = _a.find, record = _a.record;
            var node = find("node", "derived-subnode");
            return [node.remove("tag", "derived-subnode")];
        })
            .commit("Deriving a subnode closes it's parent and opens it.", function (_a) {
            var find = _a.find, record = _a.record;
            var node = find("node", "derived-subnode");
            var tree_node = find("editor/node-tree/node", { node: node });
            var parent_tree_node = find("editor/node-tree/node", { node: node.parent });
            parent_tree_node.open;
            return [
                tree_node.add("open", "true"),
                parent_tree_node.remove("open")
            ];
        })
            .commit("A node with an empty value has no value at all.", function (_a) {
            var find = _a.find;
            var node = find("node");
            var attribute = node.attribute;
            attribute.value == "";
            return [attribute.remove("value", "")];
        })
            .bind("A node's name is it's parent_field if it has one, or it's tag attribute.", function (_a) {
            var find = _a.find, choose = _a.choose;
            var node = find("node");
            var name = choose(function () { return node.parent_field; }, function () {
                var attribute = node.attribute;
                attribute.attribute == "tag";
                return attribute.value;
            }, function () { return "???"; })[0];
            return [node.add("name", name)];
        })
            .bind("A node's label is the uppercased first character of it's name.", function (_a) {
            var find = _a.find, string = _a.lib.string;
            var node = find("node");
            var name = node.name;
            var label = string.uppercase(string.get(name, 1));
            return [node.add("label", label)];
        })
            .bind("A node's color is derived from it's sort.", function (_a) {
            var find = _a.find, math = _a.lib.math;
            var node = find("node");
            var color = find("node-color", { sort: math.mod(node.sort - 1, 5) + 1 }).color;
            return [node.add("color", color)];
        });
        // this.navigation();
        this.header();
        this.nodeTree();
        this.queryEditor();
        this.moleculeGenerator();
        this.moleculeLayout();
        this.infobox();
        this.completionGenerator();
        this.fixtures();
        this.initEditor();
    };
    EditorWatcher.prototype.initEditor = function () {
        var EDITOR_ID = watcher_1.createId();
        var STYLE_ID = watcher_1.createId();
        var TAG_MARINA_ID = watcher_1.createId();
        var TAG_MARINARA_ID = watcher_1.createId();
        var BLOCK_PPL_W_BOATS_ID = watcher_1.createId();
        var BLOCK_BOAT_TYPES_ID = watcher_1.createId();
        var FRAME_PPL_W_BOATS_QUERY_ID = watcher_1.createId();
        var fixture = [
            [EDITOR_ID, "tag", "editor/root"],
            [STYLE_ID, "tag", "html/element"],
            [STYLE_ID, "tagname", "link"],
            [STYLE_ID, "rel", "stylesheet"],
            [STYLE_ID, "href", "/assets/css/editor.css"],
            ["|init", "tag", "editor/init"]
        ];
        // @NOTE: To get successive layers, multiply offsets by magnitude = ceil(mod(ix - 2, 6) + 2) and connect the dots
        // @NOTE: Take special care about the 0,0, in ix: 1.
        watcher_1.appendAsEAVs(fixture, { tag: "spiral", row: 1, sort: 1, x: 0, y: 0 });
        watcher_1.appendAsEAVs(fixture, { tag: "spiral", row: 1, sort: 2, x: 1, y: 0 });
        watcher_1.appendAsEAVs(fixture, { tag: "spiral", row: 1, sort: 3, x: 1, y: 1 });
        watcher_1.appendAsEAVs(fixture, { tag: "spiral", row: 1, sort: 4, x: 0, y: 1 });
        watcher_1.appendAsEAVs(fixture, { tag: "spiral", row: 1, sort: 5, x: -1, y: 0 });
        watcher_1.appendAsEAVs(fixture, { tag: "spiral", row: 1, sort: 6, x: 0, y: -1 });
        watcher_1.appendAsEAVs(fixture, { tag: "spiral", row: 1, sort: 7, x: 1, y: -1 });
        watcher_1.appendAsEAVs(fixture, { tag: "spiral", row: 8, sort: 8, x: 2, y: -1 });
        watcher_1.appendAsEAVs(fixture, { tag: "spiral", row: 0, sort: 1, x: 0, y: 0 });
        watcher_1.appendAsEAVs(fixture, { tag: "spiral", row: 0, sort: 2, x: 1, y: 0 });
        watcher_1.appendAsEAVs(fixture, { tag: "spiral", row: 0, sort: 3, x: 0, y: 1 });
        watcher_1.appendAsEAVs(fixture, { tag: "spiral", row: 0, sort: 4, x: -1, y: 1 });
        watcher_1.appendAsEAVs(fixture, { tag: "spiral", row: 0, sort: 5, x: -1, y: 0 });
        watcher_1.appendAsEAVs(fixture, { tag: "spiral", row: 0, sort: 6, x: -1, y: -1 });
        watcher_1.appendAsEAVs(fixture, { tag: "spiral", row: 0, sort: 7, x: 0, y: -1 });
        watcher_1.appendAsEAVs(fixture, { tag: "spiral", row: 0, sort: 8, x: 1, y: -1 });
        watcher_1.appendAsEAVs(fixture, { tag: "node-color", sort: 1, color: "#9926ea" });
        watcher_1.appendAsEAVs(fixture, { tag: "node-color", sort: 2, color: "#6c86ff" });
        watcher_1.appendAsEAVs(fixture, { tag: "node-color", sort: 3, color: "red" });
        watcher_1.appendAsEAVs(fixture, { tag: "node-color", sort: 4, color: "orange" });
        watcher_1.appendAsEAVs(fixture, { tag: "node-color", sort: 5, color: "green" });
        watcher_1.appendAsEAVs(fixture, { tag: "node-color", sort: 6, color: "indigo" });
        this.editor.inputEAVs(fixture);
    };
    EditorWatcher.prototype.fixtures = function () {
        this.editor
            .commit("When the init tag is added, preload the system with sample data.", function (_a) {
            var find = _a.find, record = _a.record;
            var init = find("editor/init");
            var editor = find("editor/root");
            var block1, frame1, person_node, boat_node, dock_node;
            return [
                editor.add("block", [
                    block1 = record("block", { sort: 1 }).add({
                        nav_tag: record("nav/tag", { name: "Marina" }),
                        name: "People with boats",
                        description: "Add a description...",
                        storyboard: [
                            frame1 = record("frame", { type: "query", sort: 1 }),
                            record("frame", { type: "output", sort: 2 }),
                        ],
                        node: []
                    })
                ]),
                record("node", { sort: 0 }),
                editor.add({ active_block: block1, active_frame: frame1 }),
                init.remove()
            ];
        })
            .commit("DEBUG: Add a spiral range to iterate over when expanding the spiral.", function (_a) {
            var find = _a.find, record = _a.record;
            find("editor/root");
            return [
                record("spiral-range", { ix: 9 }),
                record("spiral-range", { ix: 10 }),
                record("spiral-range", { ix: 11 }),
                record("spiral-range", { ix: 12 }),
                record("spiral-range", { ix: 13 }),
                record("spiral-range", { ix: 14 }),
                record("spiral-range", { ix: 15 }),
                record("spiral-range", { ix: 16 }),
                record("range", { ix: 1 }),
                record("range", { ix: 2 }),
                record("range", { ix: 3 }),
                record("range", { ix: 4 }),
                record("range", { ix: 5 }),
                record("range", { ix: 6 }),
                record("range", { ix: 7 }),
                record("range", { ix: 8 }),
                record("range", { ix: 8 }),
                record("range", { ix: 9 }),
                record("range", { ix: 10 }),
                record("range", { ix: 11 }),
                record("range", { ix: 12 }),
                record("range", { ix: 13 }),
                record("range", { ix: 14 }),
                record("range", { ix: 15 }),
                record("range", { ix: 16 }),
            ];
        });
    };
    EditorWatcher.prototype.createEditor = function () {
        var editor = new watcher_1.Program("Editor");
        editor.attach("compiler");
        editor.attach("ui");
        editor.attach("shape");
        var compiler = editor.attach("compiler");
        compiler.injectInto(this.program);
        compiler.registerWatcherFunction("send-to-editor", watcher_1.forwardDiffs(editor, "send-to-editor"));
        return editor;
    };
    //--------------------------------------------------------------------
    // Navigation
    //--------------------------------------------------------------------
    EditorWatcher.prototype.navigation = function () {
        this.editor
            .bind("Populate the nav bar with the program's block tags.", function (_a) {
            var find = _a.find, record = _a.record;
            var nav = find("editor/nav");
            var tag = nav.editor.block.nav_tag;
            return [
                nav.add("children", [
                    record("editor/nav/tag", "ui/column", { editor: nav.editor, sort: tag.name, nav_tag: tag }).add("children", [
                        record("ui/text", { sort: 0, text: tag.name })
                    ])
                ])
            ];
        })
            .bind("Populate nav tags with the blocks that have them.", function (_a) {
            var find = _a.find, choose = _a.choose, record = _a.record;
            var tag = find("editor/nav/tag");
            var block = tag.editor.block;
            block.nav_tag == tag.nav_tag;
            var name = choose(function () { return block.name; }, function () { return "Untitled Block"; })[0];
            return [
                tag.add("children", [
                    record("editor/nav/block", "ui/text", { editor: tag.editor, nav_tag: tag.nav_tag, block: block, text: name, sort: name })
                ])
            ];
        });
    };
    //--------------------------------------------------------------------
    // Header
    //--------------------------------------------------------------------
    EditorWatcher.prototype.header = function () {
        this.editor
            .bind("Populate the block description for the active block.", function (_a) {
            var find = _a.find, choose = _a.choose, record = _a.record;
            var description = find("editor/block/description");
            var active_block = description.editor.active_block;
            var name = choose(function () { return active_block.name; }, function () { return "Untitled Block"; })[0];
            var text = choose(function () { return active_block.description; }, function () { return ""; })[0];
            return [
                description.add("children", [
                    record("ui/text", { sort: 0, text: name, class: "editor-block-title" }),
                    record("ui/text", { sort: 1, text: text })
                ])
            ];
        })
            .bind("Populate the block storyboard for the active block.", function (_a) {
            var find = _a.find, record = _a.record;
            var storyboard = find("editor/block/storyboard");
            var editor = storyboard.editor;
            var active_block = editor.active_block;
            var frame = active_block.storyboard;
            return [
                storyboard.add("children", [
                    record("editor/block/frame", "ui/column", { editor: editor, sort: frame.sort, frame: frame }).add("children", [
                        record("ui/text", { text: frame.type })
                    ])
                ])
            ];
        })
            .bind("Mark the active frame.", function (_a) {
            var find = _a.find;
            var editor = find("editor/root");
            var frame = editor.active_frame;
            var frame_elem = find("editor/block/frame", { frame: frame });
            return [frame_elem.add("class", "active")];
        })
            .commit("Clicking a frame activates it.", function (_a) {
            var find = _a.find;
            var frame_elem = find("editor/block/frame");
            find("html/event/click", { element: frame_elem });
            var frame = frame_elem.frame, editor = frame_elem.editor;
            return [editor.remove("active_frame").add("active_frame", frame)];
        })
            .bind("Add new frame button to the storyboard.", function (_a) {
            var find = _a.find, record = _a.record;
            var storyboard = find("editor/block/storyboard");
            var editor = storyboard.editor;
            var active_block = editor.active_block;
            return [
                storyboard.add("children", [
                    record("editor/new-frame", "editor/block/frame", "ui/column", { editor: editor, sort: Infinity })
                ])
            ];
        })
            .commit("Clicking the new frame button opens it.", function (_a) {
            var find = _a.find;
            var new_frame = find("editor/new-frame");
            find("html/event/click", "html/direct-target", { element: new_frame });
            return [
                new_frame.add("open", "true")
            ];
        })
            .bind("When the new frame is open, display a list of editor types to choose from.", function (_a) {
            var find = _a.find, record = _a.record;
            var new_frame = find("editor/new-frame", { open: "true" });
            var editor = new_frame.editor;
            return [
                new_frame.add("children", [
                    record("editor/new-frame/type", "ui/button", { editor: editor, text: "Query", type: "query", class: "flat" }),
                    record("editor/new-frame/type", "ui/button", { editor: editor, text: "Output", type: "output", class: "flat" }),
                ])
            ];
        })
            .commit("Clicking a new frame type adds a frame of that type and closes the new frame button.", function (_a) {
            var find = _a.find, gather = _a.gather, choose = _a.choose, record = _a.record;
            var new_frame_type = find("editor/new-frame/type");
            find("html/event/click", "html/direct-target", { element: new_frame_type });
            var type = new_frame_type.type, editor = new_frame_type.editor;
            var new_frame = find("editor/new-frame", { editor: editor });
            var block = editor.active_block;
            var ix = choose(function () { return gather(block.storyboard).per(block).count() + 1; }, function () { return 1; })[0];
            return [
                new_frame.remove("open"),
                block.add("storyboard", [
                    record("frame", { block: block, type: type, sort: ix })
                ])
            ];
        });
    };
    //--------------------------------------------------------------------
    // Node Tree
    //--------------------------------------------------------------------
    EditorWatcher.prototype.nodeTree = function () {
        this.editor
            .bind("Decorate the node tree as a column.", function (_a) {
            var find = _a.find, record = _a.record;
            var tree = find("editor/node-tree");
            var side = 21, lineWidth = 1, strokeStyle = "#AAA";
            return [tree.add({ tag: "ui/column" }).add("children", [
                    record("editor/node-tree/node", "editor/node-tree/node/new", "ui/row", { sort: Infinity, tree: tree }).add("children", [
                        record("editor/node-tree/node/hex", "shape/hexagon", {
                            sort: 0, tree: tree, side: side, lineWidth: lineWidth, strokeStyle: strokeStyle
                        }).add("content", [
                            record("ui/button", { icon: "android-add" })
                        ])
                    ])
                ])];
        })
            .bind("When the new node is open, it has an input for specifying the tag.", function (_a) {
            var find = _a.find, record = _a.record;
            var new_node = find("editor/node-tree/node/new", { open: "true" });
            return [
                new_node.add("children", [
                    record("editor/node-tree/node/new/tag", "ui/autocomplete", "html/trigger-focus", { sort: 2, new_node: new_node, placeholder: "tag..." })
                ])
            ];
        })
            .bind("Each root node is an element in the tree.", function (_a) {
            var find = _a.find, record = _a.record;
            var tree = find("editor/node-tree");
            var node = tree.node;
            node.tag == "root-node";
            return [
                tree.add("children", [
                    record("editor/node-tree/node", { tree: tree, node: node, sort: node.sort })
                ])
            ];
        })
            .bind("A node consists of a hex, and a pattern.", function (_a) {
            var find = _a.find, record = _a.record;
            var tree_node = find("editor/node-tree/node");
            var tree = tree_node.tree, node = tree_node.node;
            var color = node.color, label = node.label;
            var side = 21, lineWidth = 1, strokeStyle = "#AAA";
            return [
                tree_node.add({ tag: "ui/row" }).add("children", [
                    record("editor/node-tree/node/hex", "shape/hexagon", { sort: 0, tree_node: tree_node, side: side, lineWidth: lineWidth, strokeStyle: strokeStyle }).add("content", [
                        record("ui/text", { text: label, style: record({ color: color }) })
                    ]),
                    record("editor/node-tree/node/pattern", { sort: 1, tree_node: tree_node })
                ])
            ];
        })
            .bind("A node pattern is a column of fields on the node.", function (_a) {
            var find = _a.find, record = _a.record;
            var node_pattern = find("editor/node-tree/node/pattern");
            var tree_node = node_pattern.tree_node;
            var name = tree_node.node.name;
            return [
                node_pattern.add({ tag: "ui/column" }).add("children", [
                    record("ui/row", { sort: 0, node_pattern: node_pattern }).add("children", [
                        record("editor/node-tree/node/pattern/name", "ui/text", { tree_node: tree_node, text: name })
                    ])
                ])
            ];
        })
            .bind("If a node has attributes, display them in it's pattern.", function (_a) {
            var find = _a.find, not = _a.not, choose = _a.choose, record = _a.record;
            var node_pattern = find("editor/node-tree/node/pattern");
            var tree_node = node_pattern.tree_node;
            var node = tree_node.node;
            var attribute = node.attribute;
            not(function () { attribute.attribute == "tag"; attribute.value == node.name; });
            not(function () { return attribute.value == find("entity"); });
            var sort = choose(function () { return "z" + attribute.sort; }, function () { return attribute.attribute; }, function () { return 999; })[0];
            return [
                node_pattern.add("children", [
                    record("editor/node-tree/fields", "ui/column", { sort: 1, tree_node: tree_node, attribute: attribute }).add("children", [
                        record("editor/node-tree/node/pattern/field", "ui/row", { sort: sort, tree_node: tree_node, attribute: attribute })
                    ])
                ])
            ];
        })
            .bind("A node displays attributes as text", function (_a) {
            var find = _a.find, record = _a.record;
            var pattern_field = find("editor/node-tree/node/pattern/field");
            var tree_node = pattern_field.tree_node, attribute = pattern_field.attribute;
            var field = attribute.attribute;
            return [
                pattern_field.add("children", [
                    record("ui/text", { sort: 1, text: field })
                ])
            ];
        })
            .bind("If a node's attribute has a value, display them in it's field.", function (_a) {
            var find = _a.find, not = _a.not, record = _a.record;
            var field = find("editor/node-tree/node/pattern/field");
            var tree_node = field.tree_node, attribute = field.attribute;
            var node = tree_node.node;
            not(function () { return field.open; });
            not(function () { attribute.attribute == "tag"; attribute.value == node.name; });
            return [
                field.add("children", [
                    record("editor/node-tree/node/pattern/value", "ui/text", { sort: 2, tree_node: tree_node, text: attribute.value })
                ])
            ];
        })
            .bind("An open field has a value cell even if it's attribute lacks one.", function (_a) {
            var find = _a.find, choose = _a.choose, record = _a.record;
            var field = find("editor/node-tree/node/pattern/field", { open: "true" });
            var tree_node = field.tree_node, attribute = field.attribute;
            var value = choose(function () { return attribute.value; }, function () { return ""; })[0];
            return [
                field.add("children", [
                    record("editor/node-tree/node/pattern/value", "ui/input", "html/trigger-focus", "html/autosize-input", { sort: 2, tree_node: tree_node, attribute: attribute, initial: value })
                ])
            ];
        })
            .bind("An open node displays controls beneath itself.", function (_a) {
            var find = _a.find, record = _a.record;
            var tree_node = find("editor/node-tree/node", { open: "true" });
            var hex = find("editor/node-tree/node/hex", { tree_node: tree_node });
            return [
                hex.add("children", [
                    record("editor/node-tree/node/controls", "ui/row", { tree_node: tree_node }).add("children", [
                        //record("editor/node-tree/node/add-field", "ui/button", {sort: 1, tree_node, icon: "android-add"}),
                        record("editor/node-tree/node/delete", "ui/button", { sort: 2, tree_node: tree_node, icon: "android-close" })
                    ])
                ])
            ];
        })
            .bind("An open node displays delete buttons on its fields.", function (_a) {
            var find = _a.find, choose = _a.choose, record = _a.record;
            var field = find("editor/node-tree/node/pattern/field");
            var tree_node = field.tree_node, attribute = field.attribute;
            tree_node.open;
            return [
                field.add("children", [
                    record("editor/node-tree/node/field/delete", "ui/button", { sort: 0, tree_node: tree_node, attribute: attribute, icon: "android-close" })
                ])
            ];
        })
            .bind("An open node displays a plus field button in its pattern.", function (_a) {
            var find = _a.find, record = _a.record;
            var tree_node = find("editor/node-tree/node", { open: "true" });
            var node_pattern = find("editor/node-tree/node/pattern", { tree_node: tree_node });
            return [
                node_pattern.add("children", [
                    record("ui/column", { sort: 2, node_pattern: node_pattern, class: "editor-node-tree-new-field" }).add("children", [
                        record("editor/node-tree/node/pattern/new-field", "ui/row", { tree_node: tree_node }).add("children", [
                            record("editor/node-tree/node/field/new", "ui/button", { sort: 1, tree_node: tree_node, icon: "android-add" }),
                            record("editor/node-tree/node/field/new/attribute", "ui/autocomplete", { sort: 2, tree_node: tree_node, placeholder: "field..." }),
                        ])
                    ])
                ])
            ];
        })
            .bind("Non root nodes are children of their parent's pattern.", function (_a) {
            var find = _a.find, record = _a.record;
            var node = find("node");
            var parent = node.parent;
            var tree_node = find("editor/node-tree/node", { node: parent });
            var node_pattern = find("editor/node-tree/node/pattern", { tree_node: tree_node });
            var tree = tree_node.tree;
            tree.node == node;
            return [
                node_pattern.add("children", [
                    record("ui/column", { sort: 3, node_pattern: node_pattern, class: "editor-node-tree-subnodes" }).add("children", [
                        record("editor/node-tree/node", { tree: tree, node: node, sort: node.sort })
                    ])
                ])
            ];
        })
            .bind("Fill tag completions.", function (_a) {
            var find = _a.find, record = _a.record;
            var new_tag = find("editor/node-tree/node/new/tag");
            var completion = find("editor/existing-tag");
            return [new_tag.add("completion", completion)];
        })
            .bind("Fill attribute completions.", function (_a) {
            var find = _a.find, record = _a.record;
            var new_attribute = find("editor/node-tree/node/field/new/attribute");
            var tree_node = new_attribute.tree_node;
            var completion = find("editor/existing-node-attribute", { node: tree_node.node });
            return [new_attribute.add("completion", completion)];
        })
            .bind("An open tree node require completions.", function (_a) {
            var find = _a.find;
            var tree_node = find("editor/node-tree/node", { open: "true" });
            var node = tree_node.node;
            return [node.add("completing", "true")];
        });
        //--------------------------------------------------------------------
        // Node Tree Interaction
        //--------------------------------------------------------------------
        this.editor
            .commit("Clicking a node's hex opens it.", function (_a) {
            var find = _a.find, not = _a.not;
            var hex = find("editor/node-tree/node/hex");
            find("html/event/click", { element: hex });
            var tree_node = hex.tree_node;
            not(function () { return tree_node.open; });
            return [tree_node.add("open", "true")];
        })
            .commit("Clicking a node's name opens it.", function (_a) {
            var find = _a.find, not = _a.not;
            var name_elem = find("editor/node-tree/node/pattern/name");
            find("html/event/click", { element: name_elem });
            var tree_node = name_elem.tree_node;
            not(function () { return tree_node.open; });
            return [tree_node.add("open", "true")];
        })
            .commit("Clicking an open node's hex closes it.", function (_a) {
            var find = _a.find;
            var hex = find("editor/node-tree/node/hex");
            find("html/event/click", { element: hex });
            var tree_node = hex.tree_node;
            tree_node.open;
            return [tree_node.remove("open")];
        })
            .commit("Clicking outside an open tree node closes it.", function (_a) {
            var find = _a.find, not = _a.not;
            var tree_node = find("editor/node-tree/node", { open: "true" });
            find("html/event/click");
            not(function () { return find("html/event/click", { element: tree_node }); });
            return [tree_node.remove("open")];
        })
            .commit("Clicking inside a child node of an open tree node closes it.", function (_a) {
            var find = _a.find;
            var tree_node = find("editor/node-tree/node", { open: "true" });
            var child_node = find("editor/node-tree/node");
            child_node.node.parent == tree_node.node;
            find("html/event/click", { element: child_node });
            return [tree_node.remove("open")];
        })
            .bind("Clicking the delete node button removes its node from the block.", function (_a) {
            var find = _a.find, record = _a.record;
            var delete_node = find("editor/node-tree/node/delete");
            find("html/event/click", { element: delete_node });
            var tree_node = delete_node.tree_node;
            var node = tree_node.node;
            return [record("editor/event/delete-node", { node: node })];
        })
            .bind("Deleting a node deletes its children.", function (_a) {
            var find = _a.find, choose = _a.choose, gather = _a.gather, record = _a.record;
            var parent = find("editor/event/delete-node").node;
            var node = find("node", { parent: parent });
            return [record("editor/event/delete-node", { node: node })];
        })
            .commit("Deleting a node nukes it and removes it from it's blocks.", function (_a) {
            var find = _a.find, choose = _a.choose, gather = _a.gather, record = _a.record;
            var node = find("editor/event/delete-node").node;
            var block = find("editor/root").block;
            return [
                block.remove("node", node),
                node.remove()
            ];
        })
            .commit("Deleting a node removes it from its parent.", function (_a) {
            var find = _a.find, choose = _a.choose, gather = _a.gather, record = _a.record;
            var child = find("editor/event/delete-node").node;
            var node = find("node");
            var attribute = node.attribute;
            attribute.value == child.entity;
            return [
                node.remove("attribute", attribute),
            ];
        })
            .commit("Clicking the new node button opens it.", function (_a) {
            var find = _a.find, not = _a.not, record = _a.record;
            var new_node = find("editor/node-tree/node/new");
            not(function () { return new_node.open; });
            find("html/event/click", { element: new_node });
            return [new_node.add("open", "true")];
        })
            .commit("Clicking outside an open new node closes it.", function (_a) {
            var find = _a.find, not = _a.not, record = _a.record;
            var new_node = find("editor/node-tree/node/new", { open: "true" });
            find("html/event/click");
            not(function () { return find("html/event/click", { element: new_node }); });
            return [new_node.remove("open")];
        })
            .bind("Clicking the new node save button saves it.", function (_a) {
            var find = _a.find, not = _a.not, record = _a.record;
            var save_new = find("editor/node-tree/node/new/save");
            find("html/event/click", { element: save_new });
            var new_node = save_new.new_node;
            return [record("editor/event/save-node", { new_node: new_node })];
        })
            .bind("selecting a tag in the new node autocomplete saves it.", function (_a) {
            var find = _a.find, not = _a.not, record = _a.record;
            var tag_autocomplete = find("editor/node-tree/node/new/tag");
            var new_node = tag_autocomplete.new_node, selected = tag_autocomplete.selected;
            return [record("editor/event/save-node", { new_node: new_node })];
        })
            .commit("Saving a new node commits, resets, and closes it.", function (_a) {
            var find = _a.find, not = _a.not, record = _a.record;
            var new_node = find("editor/event/save-node").new_node;
            var tag_autocomplete = find("editor/node-tree/node/new/tag");
            var value = tag_autocomplete.value;
            var tag_input = find("ui/autocomplete/input", { autocomplete: tag_autocomplete });
            value != "";
            var tree = new_node.tree;
            var active_block = tree.editor.active_block;
            var sort = active_block.next_node_sort;
            return [
                active_block.add("node", [
                    record("node", { sort: sort, attribute: record({ sort: sort, attribute: "tag", value: value }) })
                ]),
                new_node.remove("open"),
                tag_input.remove("value")
            ];
        })
            .commit("Clicking a field opens it.", function (_a) {
            var find = _a.find, not = _a.not;
            var field = find("editor/node-tree/node/pattern/field");
            find("html/event/click", { element: field });
            not(function () { return field.open; });
            return [field.add("open", "true")];
        })
            .commit("Clicking outside an open field closes it.", function (_a) {
            var find = _a.find, not = _a.not;
            var field = find("editor/node-tree/node/pattern/field", { open: "true" });
            find("html/event/click");
            not(function () { return find("html/event/click", { element: field }); });
            return [field.remove("open")];
        })
            .commit("Clicking the new field save button focuses it's autocomplete.", function (_a) {
            var find = _a.find;
            var add_field = find("editor/node-tree/node/field/new");
            var tree_node = add_field.tree_node;
            var event = find("html/event/click", { element: add_field });
            var field_autocomplete = find("editor/node-tree/node/field/new/attribute", { tree_node: tree_node });
            return [field_autocomplete.add("tag", "html/trigger-focus")];
        })
            .bind("Clicking the new field save button saves it.", function (_a) {
            var find = _a.find, record = _a.record;
            var add_field = find("editor/node-tree/node/field/new");
            var event = find("html/event/click", { element: add_field });
            var tree_node = add_field.tree_node;
            var field_autocomplete = find("editor/node-tree/node/field/new/attribute", { tree_node: tree_node });
            field_autocomplete.value != "";
            return [
                record("editor/event/save-field", { node: tree_node.node, attribute: field_autocomplete.value }),
                record("ui/event/clear", { autocomplete: field_autocomplete })
            ];
        })
            .bind("Selecting a completion in the new field autocomplete saves it.", function (_a) {
            var find = _a.find, not = _a.not, record = _a.record;
            var field_autocomplete = find("editor/node-tree/node/field/new/attribute");
            var tree_node = field_autocomplete.tree_node, selected = field_autocomplete.selected;
            return [
                record("editor/event/save-field", { node: tree_node.node, attribute: selected.text }),
                record("ui/event/clear", { autocomplete: field_autocomplete })
            ];
        })
            .commit("Saving a new field adds a new attribute to the node.", function (_a) {
            var find = _a.find, choose = _a.choose, gather = _a.gather, record = _a.record;
            var _b = find("editor/event/save-field"), node = _b.node, attribute = _b.attribute;
            attribute != "";
            // @FIXME: busted as frig...
            // let [count] = choose(() => {
            //   let {attribute} = node;
            //   1 == gather(attribute.sort).per(node).sort("down");
            //   return attribute.sort + 1;
            // }, () => 1);
            return [node.add("attribute", record("node/attribute", { sort: "@FIXME", attribute: attribute }))];
        })
            .commit("Clicking the delete field button removes its attribute from the node.", function (_a) {
            var find = _a.find, choose = _a.choose, gather = _a.gather, record = _a.record;
            var delete_field = find("editor/node-tree/node/field/delete");
            find("html/event/click", { element: delete_field });
            var tree_node = delete_field.tree_node, attribute = delete_field.attribute;
            var node = tree_node.node;
            return [
                node.remove("attribute", attribute),
                attribute.remove()
            ];
        })
            .bind("Blurring a field's value input saves it.", function (_a) {
            var find = _a.find, record = _a.record;
            var field_value = find("editor/node-tree/node/pattern/value");
            var value = find("html/event/blur", { element: field_value }).value;
            var tree_node = field_value.tree_node, attribute = field_value.attribute;
            return [record("editor/event/save-value", { tree_node: tree_node, attribute: attribute, value: value })];
        })
            .commit("Saving a field value commits it to the attribute.", function (_a) {
            var find = _a.find, record = _a.record;
            var _b = find("editor/event/save-value"), attribute = _b.attribute, value = _b.value;
            return [attribute.remove("value").add("value", value)];
        });
    };
    //--------------------------------------------------------------------
    // Query Editor
    //--------------------------------------------------------------------
    EditorWatcher.prototype.queryEditor = function () {
        this.editor
            .bind("Display a node tree for the active block.", function (_a) {
            var find = _a.find, record = _a.record;
            var content = find("editor/block/content", { type: "query" });
            var editor = content.editor;
            return [
                content.add("children", [
                    record("editor/node-tree", "editor/query-tree", { sort: 1, editor: editor }),
                    record("editor/molecule-list", "editor/query-molecules", { sort: 2, editor: editor })
                ])
            ];
        })
            .bind("Fill the tree with the active block's nodes.", function (_a) {
            var find = _a.find, record = _a.record;
            var node_tree = find("editor/query-tree");
            var editor = node_tree.editor;
            return [node_tree.add("node", editor.active_block.node)];
        })
            .bind("Fill the list with the active block's molecules.", function (_a) {
            var find = _a.find, record = _a.record;
            var molecule_list = find("editor/query-molecules");
            var editor = molecule_list.editor;
            var molecule = find("editor/molecule", { block: editor.active_block });
            return [molecule_list.add("molecule", molecule)];
        });
    };
    //--------------------------------------------------------------------
    // Molecule Generator
    //--------------------------------------------------------------------
    EditorWatcher.prototype.moleculeGenerator = function () {
        this.editor
            .bind("Create a molecule generator for the active block if it has any nodes.", function (_a) {
            var find = _a.find, record = _a.record;
            var editor = find("editor/root");
            var block = editor.active_block;
            block.node.attribute;
            return [
                block.add("molecule_generator", [
                    record("editor/molecule/generator", "eve/compiler/block", { block: block, name: "Generate molecules.", type: "watch", watcher: "send-to-editor" })
                ])
            ];
        })
            .bind("Create an atom record and output for each node of the block with attributes.", function (_a) {
            var find = _a.find, record = _a.record;
            var generator = find("editor/molecule/generator");
            var block = generator.block;
            var node = block.node;
            return [
                node.entity.add("tag", "eve/compiler/var"),
                generator.add("constraint", [
                    record("editor/atom/record", "eve/compiler/record", { generator: generator, node: node, record: node.entity }).add("attribute", [
                        //@FIXME: Bogus scan to hint to the compiler watcher that it shouldn't try to gen an id.
                        record({ attribute: "tag", value: record("editor/bogus-var", "eve/compiler/var", { node: node }) })
                    ]),
                    record("editor/atom/output", "eve/compiler/output", {
                        generator: generator, node: node, record: record("editor/atom/output/entity", "eve/compiler/var", { node: node })
                    }).add("attribute", [
                        record({ attribute: "tag", value: "editor/atom" }),
                        record({ attribute: "record", value: node.entity }),
                        record({ attribute: "node", value: node })
                    ]),
                    record("editor/record/output", "eve/compiler/output", { generator: generator, node: node, record: node.entity }).add("attribute", [
                        record({ attribute: "tag", value: "editor/record" })
                    ])
                ])
            ];
        })
            .bind("Attributes with no value are free fields.", function (_a) {
            var find = _a.find, not = _a.not, record = _a.record;
            var generator = find("editor/molecule/generator");
            var block = generator.block;
            var node = block.node;
            var attribute = node.attribute;
            not(function () { return attribute.value; });
            return [record("editor/molecule/free-field", "eve/compiler/var", { node: node, attribute: attribute })];
        })
            .bind("Attach attributes to atom records and outputs.", function (_a) {
            var find = _a.find, choose = _a.choose, record = _a.record;
            var atom_record = find("editor/atom/record");
            var generator = atom_record.generator, node = atom_record.node;
            var record_output = find("editor/record/output", { node: node });
            var attribute = node.attribute;
            var value = choose(function () { return attribute.value; }, function () { return find("editor/molecule/free-field", { node: node, attribute: attribute }); })[0];
            var identifying = choose(function () { attribute.value == find("node").entity; return "eve/compiler/attribute/non-identity"; }, function () { return "eve/compiler/attribute/identity"; })[0];
            return [
                atom_record.add("attribute", record({ attribute: attribute.attribute, value: value })),
                record_output.add("attribute", [
                    record({ tag: identifying, attribute: attribute.attribute, value: value }),
                ])
            ];
        })
            .bind("Create a molecule output for each root node.", function (_a) {
            var find = _a.find, record = _a.record;
            var generator = find("editor/molecule/generator");
            var block = generator.block;
            var node = block.node;
            node.tag == "root-node";
            var atom_output_entity = find("editor/atom/output/entity", { node: node });
            var molecule_entity;
            return [
                molecule_entity = record("editor/molecule/entity", "eve/compiler/var", { node: node }),
                generator.add("constraint", [
                    record("editor/molecule/output", "eve/compiler/output", { generator: generator, node: node, record: molecule_entity })
                        .add("parent", node)
                        .add("attribute", [
                        record({ attribute: "atom", value: atom_output_entity }),
                        record({ attribute: "tag", value: "editor/molecule" }),
                        record({ attribute: "block", value: block }),
                        record("editor/mol-av", { attribute: "node", value: node }),
                    ])
                ])
            ];
        })
            .bind("Attach subnode atoms to their parent's molecules.", function (_a) {
            var find = _a.find, record = _a.record;
            var molecule = find("editor/molecule/output");
            var generator = molecule.generator;
            var block = generator.block;
            var node = block.node;
            node.parent == molecule.parent;
            var atom_output_entity = find("editor/atom/output/entity", { node: node });
            return [
                molecule.add({
                    parent: node,
                    attribute: record("eve/compiler/attribute/non-identity", { attribute: "atom", value: atom_output_entity })
                })
            ];
        });
    };
    //--------------------------------------------------------------------
    // Molecule Layout
    //--------------------------------------------------------------------
    EditorWatcher.prototype.moleculeLayout = function () {
        this.editor
            .bind("Decorate a molecule list.", function (_a) {
            var find = _a.find;
            var molecule_list = find("editor/molecule-list");
            return [molecule_list.add({ tag: "ui/row" })];
        })
            .bind("Draw some molecules.", function (_a) {
            var find = _a.find, record = _a.record;
            var molecule_list = find("editor/molecule-list");
            var molecule = molecule_list.molecule;
            var atom = molecule.atom;
            var side = 21;
            var molecule_cell;
            return [
                molecule_list.add("children", [
                    molecule_cell = record("editor/molecule-list/molecule", "html/element", "html/listener/hover", { tagname: "div", molecule_list: molecule_list, molecule: molecule }),
                    molecule_cell.add("children", [
                        record("editor/molecule-list/molecule/grid", "shape/hex-grid", { molecule_cell: molecule_cell, molecule: molecule, side: side, gap: 5 })
                    ]),
                ])
            ];
        })
            .bind("Add cells to molecules.", function (_a) {
            var find = _a.find, record = _a.record;
            var molecule_grid = find("editor/molecule-list/molecule/grid");
            var molecule_cell = molecule_grid.molecule_cell;
            var molecule = molecule_cell.molecule;
            var atom = molecule.atom;
            var side = 21;
            return [
                molecule_grid.add("cell", [
                    record("editor/molecule-list/molecule/cell", { molecule_cell: molecule_cell, molecule: molecule, atom: atom, side: side })
                ])
            ];
        })
            .bind("A molecule's size is it's largest atom sort.", function (_a) {
            var find = _a.find, not = _a.not, record = _a.record;
            var cell = find("editor/molecule-list/molecule/cell");
            var molecule_cell = cell.molecule_cell;
            not(function () {
                var other_cell = find("editor/molecule-list/molecule/cell", { molecule_cell: molecule_cell });
                other_cell.sort > cell.sort;
            });
            return [
                molecule_cell.add("size", cell.sort),
            ];
        })
            .bind("HACK: workaround double choose bug offset + mag.", function (_a) {
            var find = _a.find, math = _a.lib.math, choose = _a.choose, record = _a.record;
            var molecule_cell = find("editor/molecule-list/molecule");
            var size = molecule_cell.size;
            // @FIXME: This isn't quite right due to 0,0 offset
            var offset = math.mod(size - 1, 6);
            // @FIXME: this measure doesn't take into account the fact that shell size increases...
            var mag = math.floor((size - 1) / 7) + 1;
            return [
                molecule_cell.add({ offset: offset, mag: mag }),
            ];
        })
            .bind("A molecule's width and height are derived from it's size.", function (_a) {
            var find = _a.find, choose = _a.choose, record = _a.record;
            var molecule_cell = find("editor/molecule-list/molecule");
            var offset = molecule_cell.offset, mag = molecule_cell.mag;
            var cell_width = 39 + 5;
            var cell_height = 44 + 5;
            var width = cell_width * 3 * mag;
            var height = cell_height * 3 * mag;
            var padLeft = width / 2 - cell_width / 2;
            var padTop = height / 2 - cell_height / 2;
            return [
                molecule_cell.add("style", record({ width: width + "px", height: height + "px", "padding-left": padLeft + "px", "padding-top": padTop + "px" }))
            ];
        })
            .bind("Populate atom cells from their atoms.", function (_a) {
            var find = _a.find, choose = _a.choose, record = _a.record;
            var atom_cell = find("editor/molecule-list/molecule/cell");
            var molecule_cell = atom_cell.molecule_cell, molecule = atom_cell.molecule, atom = atom_cell.atom, side = atom_cell.side, x = atom_cell.x, y = atom_cell.y;
            var node = atom.node;
            var lineWidth = 1; //, strokeStyle = "#AAA";
            var strokeStyle = choose(function () { atom_cell.open; return "#4444ff"; }, function () { atom_cell.tag == "html/hovered"; return "#4444ff"; }, function () { molecule_cell.tag == "html/hovered"; return "#aaaaff"; }, function () { return "#aaa"; })[0];
            return [
                atom_cell.add({ tag: ["shape/hexagon", "html/listener/hover"], side: side, lineWidth: lineWidth, strokeStyle: strokeStyle, x: x, y: y }).add("content", [
                    record("ui/text", { atom_cell: atom_cell, text: node.label, style: record({ color: node.color }) })
                ])
            ];
        })
            .bind("Sort atoms by id.", function (_a) {
            var find = _a.find, gather = _a.gather, record = _a.record;
            var atom = find("editor/atom");
            var molecule = find("editor/molecule", { atom: atom });
            var node = atom.node;
            var ix = gather(atom).per(molecule, node).sort();
            return [atom.add("sort", ix)];
        })
            .bind("Sort atom cells by id.", function (_a) {
            var find = _a.find, gather = _a.gather, record = _a.record;
            var atom_cell = find("editor/molecule-list/molecule/cell");
            var molecule = atom_cell.molecule, atom = atom_cell.atom;
            var ix = gather(atom_cell.atom.node.sort, atom_cell).per(molecule).sort();
            return [atom_cell.add("sort", ix)];
        })
            .bind("Position atom cells in a spiral.", function (_a) {
            var find = _a.find, choose = _a.choose, record = _a.record;
            var atom_cell = find("editor/molecule-list/molecule/cell");
            var molecule = atom_cell.molecule, atom = atom_cell.atom;
            var _b = find("spiral", { row: 0, sort: atom_cell.sort }), x = _b.x, y = _b.y;
            return [atom_cell.add({ x: x, y: y })];
        })
            .bind("When a molecule is open, display it's infobox.", function (_a) {
            var find = _a.find, record = _a.record;
            var molecule_cell = find("editor/molecule-list/molecule", { open: "true" });
            var molecule = molecule_cell.molecule;
            return [molecule_cell.add("children", [
                    record("editor/infobox", { molecule: molecule, molecule_cell: molecule_cell })
                ])];
        });
        //--------------------------------------------------------------------
        // Molecule Interactions
        //--------------------------------------------------------------------
        this.editor
            .commit("Clicking a molecule opens it.", function (_a) {
            var find = _a.find;
            var molecule_cell = find("editor/molecule-list/molecule");
            find("html/event/click", { element: molecule_cell });
            return [molecule_cell.add("open", "true")];
        })
            .commit("Clicking outside an open molecule closes it.", function (_a) {
            var find = _a.find, not = _a.not;
            var molecule_cell = find("editor/molecule-list/molecule", { open: "true" });
            find("html/event/click");
            not(function () { return find("html/event/click", { element: molecule_cell }); });
            return [molecule_cell.remove("open")];
        })
            .commit("Clicking an atom opens it.", function (_a) {
            var find = _a.find;
            var atom_cell = find("editor/molecule-list/molecule/cell");
            find("html/event/click", { element: atom_cell });
            return [atom_cell.add("open", "true")];
        })
            .commit("Clicking an atom closes any other open atoms.", function (_a) {
            var find = _a.find;
            var atom_cell = find("editor/molecule-list/molecule/cell");
            find("html/event/click", { element: atom_cell });
            var other = find("editor/molecule-list/molecule/cell", { open: "true" });
            other != atom_cell;
            return [other.remove("open")];
        })
            .commit("Close any open atoms of closed molecules.", function (_a) {
            var find = _a.find, not = _a.not;
            var atom_cell = find("editor/molecule-list/molecule/cell", { open: "true" });
            not(function () { return atom_cell.molecule_cell.open; });
            return [atom_cell.remove("open")];
        });
    };
    //--------------------------------------------------------------------
    // Infobox
    //--------------------------------------------------------------------
    EditorWatcher.prototype.infobox = function () {
        this.editor
            .bind("A molecule infobox is a column of node infoboxes.", function (_a) {
            var find = _a.find, record = _a.record;
            var infobox = find("editor/infobox");
            return [infobox.add("tag", "ui/column")];
        })
            .bind("A molecule infobox has a node infobox for each unique node.", function (_a) {
            var find = _a.find, record = _a.record;
            var infobox = find("editor/infobox");
            var molecule = infobox.molecule;
            var atom = molecule.atom;
            var node = atom.node;
            return [infobox.add("children", [
                    record("editor/infobox/node", { sort: node.sort, infobox: infobox, node: node }).add("atom", atom)
                ])];
        })
            .bind("A node infobox has the node name, a plus field button.", function (_a) {
            var find = _a.find, record = _a.record;
            var node_infobox = find("editor/infobox/node");
            var node = node_infobox.node;
            return [
                node_infobox.add("tag", "ui/column").add("children", [
                    record("editor/infobox/node/header", "ui/row", { sort: 1, node_infobox: node_infobox }).add("children", [
                        record("editor/infobox/node/name", "ui/text", { sort: 1, node_infobox: node_infobox, text: node.name }),
                    ]),
                    record("ui/row", { sort: 3, node_infobox: node_infobox }).add("children", [
                        record("editor/infobox/field/new", "ui/button", { sort: 1, node_infobox: node_infobox, icon: "android-add" }),
                        record("editor/infobox/field/attribute", "ui/autocomplete", { sort: 2, node_infobox: node_infobox }) // , placeholder: "field..."
                    ])
                ])
            ];
        })
            .bind("A node infobox with multiple atoms shows a paginator in it's name row.", function (_a) {
            var find = _a.find, gather = _a.gather, record = _a.record;
            var node_infobox = find("editor/infobox/node");
            var infobox_header = find("editor/infobox/node/header", { node_infobox: node_infobox });
            var node = node_infobox.node, count = node_infobox.count;
            count > 1;
            return [
                infobox_header.add("children", [
                    record("ui/row", { sort: 2, node_infobox: node_infobox, class: "editor-paginator" }).add("children", [
                        // record("ui/button", {sort: 1, node_infobox, icon: "arrow-left-b"}),
                        record("ui/text", { sort: 2, text: "(1/" + count + ")" }),
                    ])
                ])
            ];
        })
            .bind("A node infobox's count is the number of atoms it has that match.", function (_a) {
            var find = _a.find, gather = _a.gather, record = _a.record;
            var node_infobox = find("editor/infobox/node");
            var node = node_infobox.node, infobox = node_infobox.infobox;
            var molecule = infobox.molecule;
            var atom = molecule.atom;
            atom.node == node;
            var count = gather(atom).per(node).count();
            return [node_infobox.add("count", count)];
        })
            .bind("A molecule infobox atom's fields are derived from its record AVs. Show the greatest if there are multiple and none are open.", function (_a) {
            var find = _a.find, lookup = _a.lookup, gather = _a.gather, not = _a.not, record = _a.record;
            var node_infobox = find("editor/infobox/node");
            var node = node_infobox.node, atom = node_infobox.atom;
            var _b = lookup(atom.record), attribute = _b.attribute, value = _b.value;
            // @FIXME: Strongly coupled to molecule list here. Instead, pass in an `active` attribute on the infobox.
            not(function () { return find("editor/molecule-list/molecule/cell", { open: "true" }).atom.node == node; });
            // @FIXME: This bug isn't not related, it's due to atom sharing.
            // not(() => {
            //   let other_atom = find("editor/atom", {node});
            //   node_infobox.atom == other_atom;
            //   other_atom.sort > atom.sort;
            // })
            // 2 < gather(atom).per(node).sort();
            // @FIXME: This will be sad with reused atoms.
            atom.sort < 2;
            attribute != "tag";
            not(function () { return value.tag; });
            return [
                node_infobox.add("children", [
                    record("editor/infobox/atom", "ui/field-table", { sort: 2, node_infobox: node_infobox, atom: atom, record: atom.record }).add("field", [
                        record({ node_infobox: node_infobox, record: atom.record, attribute: attribute }).add("value", value)
                    ])
                ])
            ];
        })
            .bind("A molecule infobox atom's fields are derived from its record AVs. Show the open atom if it exists.", function (_a) {
            var find = _a.find, lookup = _a.lookup, not = _a.not, record = _a.record;
            var node_infobox = find("editor/infobox/node");
            var node = node_infobox.node, atom = node_infobox.atom, infobox = node_infobox.infobox;
            // @FIXME: Strongly coupled to molecule list here.
            var atom_cell = find("editor/molecule-list/molecule/cell", { molecule_cell: infobox.molecule_cell, open: "true" });
            atom_cell.atom == atom;
            var _b = lookup(atom.record), attribute = _b.attribute, value = _b.value;
            attribute != "tag";
            not(function () { return value.tag; });
            return [
                node_infobox.add("children", [
                    record("editor/infobox/atom", "ui/field-table", { sort: 2, node_infobox: node_infobox, atom: atom, record: atom.record }).add("field", [
                        record({ node_infobox: node_infobox, record: atom.record, attribute: attribute }).add("value", value)
                    ])
                ])
            ];
        })
            .bind("Fill infobox attribute completions.", function (_a) {
            var find = _a.find, record = _a.record;
            var new_attribute = find("editor/infobox/field/attribute");
            var node_infobox = new_attribute.node_infobox;
            var completion = find("editor/existing-node-attribute", { node: node_infobox.node });
            return [new_attribute.add("completion", completion)];
        })
            .bind("An infobox require completions.", function (_a) {
            var find = _a.find;
            var node_infobox = find("editor/infobox/node");
            var node = node_infobox.node;
            return [node.add("completing", "true")];
        });
        //--------------------------------------------------------------------
        // Infobox Interactions
        //--------------------------------------------------------------------
        this.editor
            .commit("Clicking the infobox new field button focuses it's autocomplete.", function (_a) {
            var find = _a.find;
            var add_field = find("editor/infobox/field/new");
            var node_infobox = add_field.node_infobox;
            var event = find("html/event/click", { element: add_field });
            var field_autocomplete = find("editor/infobox/field/attribute", { node_infobox: node_infobox });
            return [field_autocomplete.add("tag", "html/trigger-focus")];
        })
            .bind("Selecting a completion in the new field autocomplete saves it.", function (_a) {
            var find = _a.find, not = _a.not, record = _a.record;
            var field_autocomplete = find("editor/infobox/field/attribute");
            var node_infobox = field_autocomplete.node_infobox, selected = field_autocomplete.selected;
            return [
                record("editor/event/save-field", { node: node_infobox.node, attribute: selected.text }),
                record("ui/event/clear", { autocomplete: field_autocomplete })
            ];
        });
    };
    //--------------------------------------------------------------------
    // Completion Generator
    //--------------------------------------------------------------------
    EditorWatcher.prototype.completionGenerator = function () {
        //--------------------------------------------------------------------
        // Tag completions
        //--------------------------------------------------------------------
        this.program
            .watch("Make existing tags in the program available to the editor.", function (_a) {
            var find = _a.find, record = _a.record;
            var rec = find();
            return [record("editor/existing-tag", { text: rec.tag })];
        })
            .asDiffs(watcher_1.forwardDiffs(this.editor, "Send tags to editor.", false));
        //--------------------------------------------------------------------
        // Node -> Attribute completions
        //--------------------------------------------------------------------
        this.editor
            .bind("Create a completion generator for node -> attribute.", function (_a) {
            var find = _a.find, record = _a.record;
            var node = find("node", { completing: "true" });
            return [
                record("editor/node/attribute/completer", "eve/compiler/block", { node: node, name: "Node attribute completer.", type: "watch", watcher: "send-to-editor" })
                    .add("joined_node", node)
            ];
        })
            .bind("Create a record for each node.", function (_a) {
            var find = _a.find, record = _a.record;
            // @NOTE: This is intentionally local. Do we want it to be block-level filtering vs node-level?
            var completer = find("editor/node/attribute/completer");
            var node = completer.joined_node;
            return [
                node.entity.add("tag", "eve/compiler/var"),
                completer.add("constraint", [
                    record("editor/node/record", "eve/compiler/record", { completer: completer, node: node, record: node.entity }).add("attribute", [
                        //@FIXME: Bogus scan to hint to the compiler watcher that it shouldn't try to gen an id.
                        record({ attribute: "tag", value: record("editor/bogus-var", "eve/compiler/var", { node: node }) })
                    ])
                ])
            ];
        })
            .bind("Attributes with no value are free fields.", function (_a) {
            var find = _a.find, not = _a.not, record = _a.record;
            var completer = find("editor/node/attribute/completer");
            var node = completer.joined_node;
            var attribute = node.attribute;
            not(function () { return attribute.value; });
            return [record("editor/molecule/free-field", "eve/compiler/var", { node: node, attribute: attribute })]; // @FIXME: Rename this tag something more generic.
        })
            .bind("Attach attributes to node record.", function (_a) {
            var find = _a.find, choose = _a.choose, record = _a.record;
            var node_record = find("editor/node/record");
            var completer = node_record.completer, node = node_record.node;
            var attribute = node.attribute;
            var value = choose(function () { return attribute.value; }, function () { return find("editor/molecule/free-field", { node: node, attribute: attribute }); })[0];
            var identifying = choose(function () { attribute.value == find("node").entity; return "eve/compiler/attribute/non-identity"; }, function () { return "eve/compiler/attribute/identity"; })[0];
            return [node_record.add("attribute", record({ attribute: attribute.attribute, value: value }))];
        })
            .bind("Parent nodes are joined nodes.", function (_a) {
            var find = _a.find, choose = _a.choose, record = _a.record;
            var completer = find("editor/node/attribute/completer");
            var node = completer.joined_node;
            return [completer.add("joined_node", node.parent)];
        })
            .bind("The completions are the attributes of any records that still match.", function (_a) {
            var find = _a.find, record = _a.record;
            var completer = find("editor/node/attribute/completer");
            var node = completer.node;
            var output_var;
            var attr_match;
            var val_match;
            return [
                output_var = record("eve/compiler/var", "editor/node/attribute/completer/output", { node: node }),
                attr_match = record("eve/compiler/var", "editor/node/attribute/completer/attribute", { node: node }),
                val_match = record("eve/compiler/var", "editor/node/attribute/completer/value", { node: node }),
                completer.add("constraint", [
                    record("eve/compiler/output", { node: node, record: output_var }).add("attribute", [
                        record({ attribute: "tag", value: "editor/existing-node-attribute" }),
                        record({ attribute: "node", value: node }),
                        record({ attribute: "text", value: attr_match }),
                        // @FIXME: We can't gen a choose so we have to send it over and figure out is_record editor-side.
                        record("eve/compiler/attribute/non-identity", { attribute: "value", value: val_match }),
                    ]),
                    record("eve/compiler/lookup", { record: node.entity, attribute: attr_match, value: val_match }),
                ])
            ];
        })
            .bind("Compute is_record based on the values of existing node attributes.", function (_a) {
            var find = _a.find, string = _a.lib.string;
            var existing = find("editor/existing-node-attribute");
            string["index-of"](existing.value, "|"); // @FIXME: hacky gen id detection.
            return [existing.add("is_record", "true")];
        });
    };
    return EditorWatcher;
}(watcher_1.Watcher));
watcher_1.Watcher.register("editor", EditorWatcher);
//# sourceMappingURL=editor.js.map