"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var runtime_1 = require("../runtime/runtime");
var dsl2_1 = require("../runtime/dsl2");
exports.Program = dsl2_1.Program;
var uuid_1 = require("uuid");
//------------------------------------------------------------------------------
// Watcher
//------------------------------------------------------------------------------
var Watcher = (function () {
    function Watcher(_program) {
        this._program = _program;
        this.setup();
    }
    Watcher.register = function (id, watcher) {
        if (this._registry[id]) {
            if (this._registry[id] === watcher)
                return;
            throw new Error("Attempting to overwrite existing watcher with id '" + id + "'");
        }
        this._registry[id] = watcher;
    };
    Watcher.unregister = function (id) {
        delete this._registry[id];
    };
    Watcher.get = function (id) {
        var watcher = this._registry[id];
        if (watcher)
            return watcher;
    };
    Object.defineProperty(Watcher.prototype, "program", {
        get: function () { return this._program; },
        enumerable: true,
        configurable: true
    });
    Watcher.prototype.setup = function () { };
    Watcher._registry = {};
    return Watcher;
}());
exports.Watcher = Watcher;
;
;
;
var Exporter = (function () {
    function Exporter() {
        var _this = this;
        this._diffTriggers = {};
        this._objectTriggers = {};
        this._blocks = [];
        this.handle = function (blockChanges) {
            for (var _i = 0, _a = _this._blocks; _i < _a.length; _i++) {
                var blockId = _a[_i];
                var changes = blockChanges[blockId];
                if (changes && changes.length) {
                    var diffTriggers = _this._diffTriggers[blockId];
                    if (diffTriggers) {
                        var output = { adds: [], removes: [] };
                        for (var _b = 0, changes_1 = changes; _b < changes_1.length; _b++) {
                            var change = changes_1[_b];
                            var eav = change.toRawEAV();
                            if (change.count > 0) {
                                output.adds.push(eav);
                            }
                            else {
                                output.removes.push(eav);
                            }
                        }
                        for (var _c = 0, diffTriggers_1 = diffTriggers; _c < diffTriggers_1.length; _c++) {
                            var trigger = diffTriggers_1[_c];
                            trigger(output);
                        }
                    }
                    var objectTriggers = _this._objectTriggers[blockId];
                    if (objectTriggers) {
                        var output = _this.accumulateChangesAs(changes);
                        for (var _d = 0, objectTriggers_1 = objectTriggers; _d < objectTriggers_1.length; _d++) {
                            var trigger = objectTriggers_1[_d];
                            trigger(output);
                        }
                    }
                }
            }
        };
    }
    Exporter.prototype.triggerOnDiffs = function (blockId, handler) {
        if (!this._diffTriggers[blockId])
            this._diffTriggers[blockId] = runtime_1.createArray();
        if (this._diffTriggers[blockId].indexOf(handler) === -1) {
            this._diffTriggers[blockId].push(handler);
        }
        if (this._blocks.indexOf(blockId) === -1) {
            this._blocks.push(blockId);
        }
    };
    Exporter.prototype.triggerOnObjects = function (blockId, handler) {
        if (!this._objectTriggers[blockId])
            this._objectTriggers[blockId] = runtime_1.createArray();
        if (this._objectTriggers[blockId].indexOf(handler) === -1) {
            this._objectTriggers[blockId].push(handler);
        }
        if (this._blocks.indexOf(blockId) === -1) {
            this._blocks.push(blockId);
        }
    };
    Exporter.prototype.accumulateChangesAs = function (changes) {
        var adds = {};
        var removes = {};
        for (var _i = 0, changes_2 = changes; _i < changes_2.length; _i++) {
            var change = changes_2[_i];
            var _a = change.reverse(), e = _a.e, a = _a.a, v = _a.v, count = _a.count;
            if (count === 1) {
                var record = adds[e] = adds[e] || Object.create(null);
                if (record[a])
                    throw new Error("@FIXME: accumulateChanges supports only a single value per attribute.");
                record[a] = v;
            }
            else {
                var record = removes[e] = removes[e] || Object.create(null);
                if (record[a])
                    throw new Error("@FIXME: accumulateChanges supports only a single value per attribute.");
                record[a] = v;
            }
        }
        return { adds: adds, removes: removes };
    };
    return Exporter;
}());
exports.Exporter = Exporter;
//------------------------------------------------------------------------------
// Convenience Diff Handlers
//------------------------------------------------------------------------------
function _isId(value) {
    return ("" + value).indexOf("|") != -1;
}
exports._isId = _isId;
function maybeIntern(value) {
    if (value === undefined)
        return value;
    return _isId(value) ? runtime_1.GlobalInterner.get(value) : value;
}
exports.maybeIntern = maybeIntern;
function asJS(value) {
    if (typeof value === "number")
        return value;
    if (value === "true")
        return true;
    if (value === "false")
        return false;
    return value;
}
exports.asJS = asJS;
function forwardDiffs(destination, name, debug) {
    if (name === void 0) { name = "Unnamed"; }
    if (debug === void 0) { debug = false; }
    return function (diffs) {
        var eavs = [];
        for (var _i = 0, _a = diffs.removes; _i < _a.length; _i++) {
            var _b = _a[_i], e = _b[0], a = _b[1], v = _b[2];
            eavs.push([e, a, v, -1]);
        }
        for (var _c = 0, _d = diffs.adds; _c < _d.length; _c++) {
            var _e = _d[_c], e = _e[0], a = _e[1], v = _e[2];
            eavs.push([e, a, v, 1]);
        }
        if (eavs.length) {
            if (debug) {
                console.log("FWD", name, "=>", destination.name);
                console.log(eavs.map(function (c) { return "[" + c.map(maybeIntern).join(", ") + "]"; }).join("\n"));
            }
            destination.inputEAVs(eavs);
        }
    };
}
exports.forwardDiffs = forwardDiffs;
//--------------------------------------------------------------------
// Watcher / Program Utils
//--------------------------------------------------------------------
function createId() {
    return "|" + uuid_1.v4();
}
exports.createId = createId;
function isRawValue(x) {
    return x !== undefined && (typeof x === "string" || typeof x === "number");
}
exports.isRawValue = isRawValue;
function isRawValueArray(x) {
    if (x && x.constructor === Array) {
        for (var _i = 0, x_1 = x; _i < x_1.length; _i++) {
            var value = x_1[_i];
            if (!isRawValue(value))
                return false;
        }
        return true;
    }
    return false;
}
exports.isRawValueArray = isRawValueArray;
function isRawEAVArray(x) {
    if (x && x.constructor === Array) {
        for (var _i = 0, x_2 = x; _i < x_2.length; _i++) {
            var value = x_2[_i];
            if (!isRawValueArray(value))
                return false;
            if (value.length !== 3)
                return false;
        }
        return true;
    }
    return false;
}
exports.isRawEAVArray = isRawEAVArray;
function isRawEAVArraySet(x) {
    return x && x.constructor === Array && isRawEAVArray(x[0]);
}
exports.isRawEAVArraySet = isRawEAVArraySet;
function isRecord(x) {
    return x && typeof x === "object" && x.constructor !== Array;
}
exports.isRecord = isRecord;
function isRecordSet(x) {
    return x && x.constructor === Array && isRecord(x[0]);
}
exports.isRecordSet = isRecordSet;
function appendAsEAVs(eavs, record, id) {
    if (id === void 0) { id = createId(); }
    for (var attr in record) {
        var value = record[attr];
        if (isRawValue(value)) {
            eavs.push([id, attr, value]);
        }
        else if (isRawValueArray(value)) {
            // We have a set of scalars
            for (var _i = 0, value_1 = value; _i < value_1.length; _i++) {
                var val = value_1[_i];
                eavs.push([id, attr, val]);
            }
        }
        else if (isRawEAVArray(value)) {
            // We have a single nested sub-object (i.e. a set of EAVs).
            var childEAVs = value;
            var childId = childEAVs[0][0];
            eavs.push([id, attr, childId]);
            for (var _a = 0, childEAVs_1 = childEAVs; _a < childEAVs_1.length; _a++) {
                var childEAV = childEAVs_1[_a];
                eavs.push(childEAV);
            }
        }
        else if (isRawEAVArraySet(value)) {
            // We have a set of nested sub-objects.
            for (var _b = 0, value_2 = value; _b < value_2.length; _b++) {
                var childEAVs = value_2[_b];
                var childId = childEAVs[0][0];
                eavs.push([id, attr, childId]);
                for (var _c = 0, childEAVs_2 = childEAVs; _c < childEAVs_2.length; _c++) {
                    var childEAV = childEAVs_2[_c];
                    eavs.push(childEAV);
                }
            }
        }
        else if (isRecord(value)) {
            var ix = eavs.length;
            appendAsEAVs(eavs, value);
            var childId = eavs[ix][0];
            eavs.push([id, attr, childId]);
        }
        else if (isRecordSet(value)) {
            for (var _d = 0, value_3 = value; _d < value_3.length; _d++) {
                var record_1 = value_3[_d];
                var ix = eavs.length;
                appendAsEAVs(eavs, record_1);
                var childId = eavs[ix][0];
                eavs.push([id, attr, childId]);
            }
        }
    }
    return eavs;
}
exports.appendAsEAVs = appendAsEAVs;
//# sourceMappingURL=watcher.js.map